/* forward_exs.h */

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */


/* collect all forward declarations here at the beginning */
static void
our_free_qp(struct exs_conn *cb);

static int
our_setup_qp(struct exs_conn *cb, struct rdma_cm_id *cm_id);

static void
our_free_transaction_block(struct transaction_block *block);

static int
our_merge_pending_accept(struct exs_conn *listen_conn,
			 struct rdma_cm_event *cm_event);

static int
our_accept_established(struct exs_conn *listen_conn);

static int
our_connect_returning(struct exs_conn *new_conn, int err_level, int ret);

static void
our_connect_done(struct exs_conn *new_conn);

static int
our_connect_addr_resolved(struct exs_conn *new_conn);

static int
our_connect_route_resolved(struct exs_conn *new_conn);

static int
our_connect_established(struct exs_conn *new_conn, struct rdma_cm_event *event);

static int
our_rdma_disconnect(struct exs_conn *this_conn);

static void
our_free_all_pending_lists(struct exs_conn *this_conn);


static void
our_flush_stranded_send_blocks(struct exs_conn *this_conn);


static void
our_flush_all_stranded_recv_blocks(struct exs_conn *this_conn);


static int
our_xmit_if_match_seqpacket(struct exs_conn *this_conn,
			struct transaction_block *block);

static void
print_send_wr(FILE *fptr, struct ibv_send_wr *wr);

static struct transaction_block *
our_init_send_transaction_block(struct exs_conn *cb,
		struct transaction_block *block);

static struct transaction_block *
our_init_control_transaction_block(struct transaction_block *block,
		struct exs_buf *msgbuf,
		void *buf_ptr, size_t msg_size);

static void
our_uninit_transaction_block(struct transaction_block *block);

static int
our_enqueue_send_seqpacket(struct exs_conn *this_conn, struct transaction_block *this,
		const void *buffer, size_t length);

static int
our_send_indirect(struct exs_conn *cb, struct transaction_block *block);

static int
our_send_advert(struct exs_conn *this_conn, struct transaction_block *block);

static void
our_destroy_completion_synch(struct exs_conn *this_conn);

static int
our_shutdown_finished(struct exs_conn *this_conn,
			struct transaction_block *block,
			int how);

static int
our_send_eof(struct exs_conn *cb, struct transaction_block *block,
					int *send_flags, int shutdown_how);

static int
exs_buf_free(struct exs_buf *buf);

static int
exs_buf_alloc(struct exs_conn *cb, struct exs_buf *buf, size_t size,
					int access_flags, const char *message);
