/* exs.c */

/*
 * Copyright 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015 by
 *	University of New Hampshire Interoperability Laboratory
 *	121 Technology Drive, Suite 2
 *	Durham, NH 03824-4716
 *	phone: 1-603-862-0090
 *	fax: 1-603-862-4181
 *	e-mail: info@iol.unh.edu
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

/*
 * The UNH IOL implementation of the Extended Sockets API (ES-API) Issue 1.0
 *
 * defined in:	The Interconnect Software Consortium Issue 1.0
 *		ISBN: 1-931624-52-6
 *		Published by The open Group, January 2005.
 *
 * all page references in this source code are to pages in that document.
 */

/*
 * Most of the coding in this file done by:
 *	Bob Russell
 *	rdr@iol.unh.edu
 */


/* This was implemented using OFED 1.5.4.1
 * OFED .h files stored in: /usr/include/rdma and /usr/include/infiniband
 *
 * In ofed library, int rdma_xxx() functions return 0 if ok, else -1 with errno set
 *			rdma_create_id()		checked
 *			rdma_destroy_id()		ok, return value never used
 *			rdma_bind_addr()		checked
 *			rdma_resolve_addr()		checked
 *			rdma_resolve_route()		checked
 *			rdma_create_qp()		checked
 *			rdma_connect()			checked
 *			rdma_listen()			checked
 *			rdma_accept()			checked
 *			rdma_reject()			checked
 *			rdma_migrate_id()		checked
 *			rdma_notify()			never called
 *			rdma_disconnect()		checked
 *			rdma_join_multicast()		never called
 *			rdma_leave_multicast()		never called
 *			rdma_ack_cm_event()		checked
 *			rdma_get_cm_event()		checked
 *	    the following return 0 if ok, else the errno value itself > 0
 *			ibv_dealloc_pd()		ok, return value never used
 *			ibv_destroy_comp_channel()	checked
 *			ibv_destroy_cq()		checked
 *			ibv_req_notify_cq()		checked
 *			ibv_dereg_mr()			checked
 *			ibv_post_recv()			checked
 *	    the following return 0 if ok, else -(errno value) < 0
 *			ibv_post_send()			checked
 *	    the following return non-NULL if ok, else NULL with errno set
 *			rdma_get_devices()		ok, we set errno = EAGAIN
 *			rdma_create_event_channel()	checked
 *	    the following return non-NULL if ok, else NULL on error
 *			(and we assume they set errno before return)
 *			ibv_alloc_pd()			checked
 *			ibv_create_comp_channel()	checked
 *			ibv_create_cq()			checked
 *			ibv_reg_mr()			checked
 *	    the following return nothing (i.e., void)
 *			rdma_free_devices()		ok, return value never used
 *	    the following return nothing (i.e., void)
 *			(but man page says errno set)
 *			rdma_destroy_event_channel()	ok, return value never used
 *			rdma_destroy_qp()		ok, return value never used
 *	    the following (are static inline uint16_t)
 *			and return != 0 ok, == 0 not connected/bound
 *			rdma_get_dst_port()		return value always used
 *			rdma_get_src_port()		return value always used
 *
 */

/*	We use the following ofed structures defined in
 *		/usr/include/infiniband/verbs.h
 *
 *	enumb ibv_wr_opcode {
 *			IBV_WR_RDMA_WRITE,
 *			IBV_WR_RDMA_WRITE_WITH_IMM,
 *			IBV_WR_SEND,
 *			IBV_WR_SEND_WITH_IMM,
 *			IBV_WR_RDMA_READ,
 *			IBV_WR_ATOMIC_CMP_AND_SWP,
 *			IBV_WR_ATOMIC_FETCH_AND_ADD
 *		};
 *
 *	enum ibv_send_flags {
 *			IBV_SEND_FENCE		= 1 << 0,
 *			IBV_SEND_SIGNALED	= 1 << 1,
 *			IBV_SEND_SOLICITED	= 1 << 2,
 *			IBV_SEND_INLINE		= 1 << 3
 *		};
 *
 *	struct ibv_sge {
 *			uint64_t		addr;
 *			uint32_t		length;
 *			uint32_t		lkey;
 *		};
 *
 *	struct ibv_send_wr {
 *			uint64_t		wr_id;
 *			struct ibv_send_wr	*next;
 *			struct ibv_sge		*sg_list;
 *			int			num_sge;
 *			enum ibv_wr_opcode	opcode;
 *			enum ibv_send_flags	send_flags;
 *			uint32_t		imm_data; network byte order
 *			union {
 *			} wr;
 *			uint32_t		xrc_remote_srq_num;
 *		};
 *
 *	struct ibv_recv_wr {
 *			uint64_t		wr_id;
 *			struct ibv_recv_wr	*next;
 *			struct ibv_sge		*sg_list;
 *			int			num_sge;
 *		};
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _POSIX_C_SOURCE 200112L
#define _ISOC99_SOURCE
#define _XOPEN_SOURCE 600

#include <assert.h>
#include <limits.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <semaphore.h>
#include <signal.h>
#include <sys/stat.h>
#include <time.h>
#include <byteswap.h>
#include <pthread.h>
#include <sys/mman.h>

/* /usr/include/rdma/rdma_cma.h includes:
 *	/usr/include/infiniband/verbs.h
 *	/usr/include/infiniband/sa.h
 */
#include <rdma/rdma_cma.h>

#include <evt_queue.h>
#include <exs_private.h>
#include <affinity.h>
#include <atomic.h>
#include <serial.h>

#if __BYTE_ORDER == __LITTLE_ENDIAN
static inline uint64_t htonll(uint64_t x) { return bswap_64(x); }
static inline uint64_t ntohll(uint64_t x) { return bswap_64(x); }
#else
static inline uint64_t htonll(uint64_t x) { return x; }
static inline uint64_t ntohll(uint64_t x) { return x; }
#endif


/* this is made available via the nonstandard exs_get_version() function */
static const char our_version[] = VERSION;

/* this is made available externally in exs.h,
 * to enable debugging, set to it nonzero
 * for now, default is disable debugging
 */
int exs_debug_enable = 0;

int _exs_global_flags = 0;

/* table of discovered maximum inline_data sizes
 *
 * 280 seems to be the max accepted by chelsio T4 iwarp cards
 * 460 seems to be the max accepted by Mellanox DDR InfiniBand cards
 * 912 seems to be the max accepted by Mellanox QDR InfiniBand cards
 *	any more than this and the rdma_create_qp() fails with ENOMEM
 * NOTE intel iwarp cards accept 280 or 460 but return 0 when queried
 *
 * NOTE: maximum is 256 for now because anything more decreases performance, at
 * least on the Mellanox QDR InfiniBand card that I was testing with.
 */
#define	KNOWN_INLINE_DATA_SIZE_LIMIT	4
static const uint32_t	inline_data_size[] = {912, 460, 280, 0};

/* table to print enum ibv_mtu values */
static const int	path_size[] = {0, 256, 512, 1024, 2048, 4096};

/* Cache line size.  This is queried in exs_init, and if the value cannot be
 * queried for some reason, we assume a worst-case default. */
#define OUR_CACHE_LINE_SIZE_GUESS 256
static unsigned int our_cache_line_size = OUR_CACHE_LINE_SIZE_GUESS;

/* table to print exs_event_t values in exs_evt_type_to_string() */
static const char *exs_evt_type_strings[] = {
			"<illegal exs_evt_type>",	/*  0 */
			"EXS_EVT_POLL",			/*  1 */
			"EXS_EVT_CONNECT",		/*  2 */
			"EXS_EVT_ACCEPT",		/*  3 */
			"EXS_EVT_SEND",			/*  4 */
			"EXS_EVT_RECV",			/*  5 */
			"EXS_EVT_SENDMSG",		/*  6 */
			"EXS_EVT_RECVMSG",		/*  7 */
			"EXS_EVT_SENDFILE",		/*  8 */
			"EXS_EVT_SENDTIMEOUT",		/*  9 */
			"EXS_EVT_RECVTIMEOUT",		/* 10 */
			"EXS_EVT_CLOSE",		/* 11 */
			"EXS_EVT_SHUTDOWN",		/* 12 */
};	/* exs_evt_type_strings */


enum our_message_header_types {
	OUR_ADVERTISEMENT = 1,	/* 1 */
	OUR_WRITE_NOTIFY = 2,	/* 2 */
	OUR_EOF = 3,		/* 3 */
	OUR_SETUP_REQUEST,	/* 4 */
	OUR_SETUP_REPLY,	/* 5 */
	OUR_ACK,		/* 6 */
};	/* enum our_message_header_types */

#define HEADER_TYPE_MASK	(0x07)

#define EXS_FD_ALL_FLAGS_MASK	(EXS_FD_SP_COPY_UNREG_ON_SEND	\
				| EXS_FD_SP_COPY_UNREG_ON_RECV	\
				| EXS_FD_BYPASS_PARAM_CHECKING	\
				| EXS_FD_NODELAY		\
				| EXS_FD_BUSYPOLL               \
				| EXS_FD_NO_WRITE_WITH_IMM      \
				| EXS_FD_NO_STREAM_ADVERT	\
				| EXS_FD_STREAM_BUF_NOALIGN	\
				)

enum conn_states {
	EXS_CONN_FINISHED,	/* after exs_close */
	EXS_CONN_UNOPENED,	/* at end of good calloc() */
	EXS_CONN_UNBOUND,	/* at end of good exs_socket() */
	EXS_CONN_BOUND,		/* at end of good exs_bind() */
	EXS_CONN_ADDRESSING,	/* at start of rdma_resolve_addr() */
	EXS_CONN_ROUTING,	/* at start of rdma_resolve_route() */
	EXS_CONN_CONNECTING,	/* at start of rdma_connect() */
	EXS_CONN_RUNNING,	/* after good rdma_connect() */
	EXS_CONN_LISTENING,	/* at end of good exs_listen() */
	EXS_CONN_ACCEPTING,	/* after exs_accept and CONNECT_REQUEST */
	EXS_CONN_CLOSING,	/* at start of mainline rdma_disconnect() */
	EXS_CONN_TERMINATING,	/* remotely triggered rdma_disconnect() */
	EXS_CONN_UNLISTENING,	/* closed but owns still active accepts */
	EXS_CONN_CLOSED,	/* old state during exs_close() */
	EXS_CONN_CLIENT_SETUP,	/* during client setup exchange */
	EXS_CONN_AGENT_SETUP,	/* during agent setup exchange */
	EXS_CONN_DELAY_CLOSING,	/* after exs_close with pending sends */
	EXS_CONN_TERMINATED,	/* after remote termination */
};	/* enum conn_states */


/* state transitions
 * Listener connection sequence:
 *		|
 *		|	exs_socket()
 *	       \|/
 *	EXS_CONN_UNOPENED
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_UNBOUND
 *		|
 *		|	exs_bind()
 *	       \|/
 *	EXS_CONN_BOUND
 *		|
 *		|	exs_listen()
 *	       \|/
 *	EXS_CONN_LISTENING
 *		|
 *		|	exs_close()
 *	       \|/
 *	EXS_CONN_UNLISTENING
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_FINISHED
 *
 * Agent connection sequence:
 *		|
 *		|	exs_accept()
 *	       \|/
 *	EXS_CONN_UNOPENED
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_UNBOUND
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_ACCEPTING
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_ACCEPT_SETUP
 *		|
 *		|
 *	       \|/	  RDMA_CM_EVENT_DISCONNECTED
 *	EXS_CONN_RUNNING ----------------------------->	EXS_CONN_TERMINATING
 *	 /		\					|
 *	/		 \ exs_close()				|
 *     /		  \					|
 * EXS_CONN_CLOSING	 EXS_CONN_DELAY_CLOSING			|
 *		\	  /					|
 *		 \	 / RDMA_CM_EVENT_DISCONNECTED		|
 *		  \	/					|
 *		EXS_CONN_FINISHED <-----------------------------+
 *
 * Client connection sequence:
 *		|
 *		|	exs_socket()
 *	       \|/
 *	EXS_CONN_UNOPENED
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_UNBOUND
 *		|
 *		|	exs_connect()
 *	       \|/
 *	EXS_CONN_ADDRESSING
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_ROUTING
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_CONNECTING
 *		|
 *		|
 *	       \|/
 *	EXS_CONN_CLIENT_SETUP
 *		|
 *		|
 *	       \|/	  RDMA_CM_EVENT_DISCONNECTED
 *	EXS_CONN_RUNNING ----------------------------->	EXS_CONN_TERMINATING
 *	 /		\					|
 *	/		 \ exs_close()				| exs_close()
 *     /		  \					|
 * EXS_CONN_CLOSING	 EXS_CONN_DELAY_CLOSING			|
 *		\	  /					|
 *		 \	 / RDMA_CM_EVENT_DISCONNECTED		|
 *		  \	/					|
 *		EXS_CONN_FINISHED <-----------------------------+
 *
 */

/* These next 4 values need to be understood and tuned better! */
#define	RESPONDER_RESOURCES	4	/* no. of RDMA_READs we respond to */
#define	INITIATOR_DEPTH		4	/* no. of RDMA_READs we initiate */
#define	RETRY_COUNT		5	/* no. of retries before failure */
#define	RNR_RETRY_COUNT		5	/* no. of Recv Not Ready retries */

/*	default posted struct transaction_block items in each direction
 *	used to initialize posted_credits field in cb
 *	user can reduce these by fcntl(EXS_F_SETFLOWCONTROLCREDITS)
 *	the value actually used is negotiated at connect/accept time
 */
#define DEFAULT_FLOW_CONTROL_CREDITS	32	/* initial credit limit */

/* maximum allocated struct transaction_block items per connection:
 *
 *	to satisfy exs_send, we need locally:
 *	posted_credits recv blocks to catch remote advertisement
 *	posted_credits send_wwi blocks to push data to receiver
 *
 *	to satisfy exs_recv, we need locally:
 *	posted_credits send_advert blocks to send our advertisement
 *	posted_credits recv blocks to handle WWI
 *	1/3 * posted_credits recv blocks to handle ACK (SOCK_STREAM only)
 *
 * Thus, in total, we need:
 * 	2 * posted_credits recv blocks
 * 	1 * posted_credits send_wwi blocks
 * 	1 * posted_credits send_advert blocks
 */

/*	maximum send queue depth -- 1 * credits ADVERT, 1.5 * credits DATA */
#define SQ_DEPTH(cb)	(7 * ((cb)->posted_credits) / 2)

/*	maximum recv queue depth -- 1 * credits ADVERT, 2 * credits DATA */
#define RQ_DEPTH(cb)	(4 * ((cb)->posted_credits))

/*	maximum completion queue depth -- SQ_DEPTH + RQ_DEPTH */
#define CQ_DEPTH(cb)	(16 * ((cb)->posted_credits))

/*	number of EOF blocks */
#define EOF_BLOCK_COUNT	2

#define EXS_LISTEN_BACKLOG 12


/* default small packet max size */
#define SMALL_PACKET_MAX_SIZE		0

/* maximum number of immediate bytes we allow as immediate data */
#define MAX_SMALL_PACKET_MAX_SIZE	(64 * 1024 * 1024)


/* minimum size to allocate for control message */
#define MIN_ALLOC_CONTROL_MSG_SIZE	64

/* maximum size of an EXS message */
enum { MAX_EXS_MESSAGE_SIZE = 1 << 30 };
#define MAX_EXS_MESSAGE_SIZE MAX_EXS_MESSAGE_SIZE


/* timeout values in seconds */

/* timeout used in our_delay_for_flushing */
#define FLUSHING_TIMEOUT_SECS	1
#define FLUSHING_TIMEOUT_NSECS	250000000	/* quarter of a second */

/* block state bits
 *
 * Bits used:
 * SOCK_SEQPACKET exs_send(): SEND_READY|ADVERT_READY
 * SOCK_SEQPACKET exs_recv(): (none)
 * SOCK_STREAM EXS_NODELAY exs_send():
 *        SEND_READY|ADVERT_READY|SEND_DONE|ACK_DONE
 * SOCK_STREAM EXS_NODELAY exs_recv(): RECV_READY|RECV_DONE|ACK_DONE */
enum block_state_bits {
	SEND_READY_BIT = 1 << 0,
	ADVERT_READY_BIT = 1 << 1,
	SEND_DONE_BIT = 1 << 2,
	ACK_DONE_BIT = 1 << 3,
	RECV_READY_BIT = 1 << 4,
	RECV_DONE_BIT = 1 << 5,
	ADVERT_SENDING_BIT = 1 << 6,
	ACK_PENDING_BIT = 1 << 7,
};

enum imm_flag_bits {
	IMM_DIRECT_BIT = 0x80000000,
	IMM_FLAG_MASK = 0x80000000,

	IMM_PHASE_MASK = 0x7fff0000,
	IMM_PHASE_SHIFT = 16,
	IMM_SEQNO_MASK = 0x0000ffff,
	IMM_SEQNO_SHIFT = 0,
};

/* all fields in an advertisement are stored, sent in network byte order */
struct our_advert
{
	uint64_t	ad_buf_seqno;
	uint64_t	ad_buf_addr;
	uint32_t	ad_buf_key;
	uint32_t	ad_buf_len;
	uint32_t	ad_buf_reserved;
	uint32_t	ad_buf_flags;
	uint32_t	ad_buf_phase;
	uint32_t	ad_ack_empty;
	uint32_t	ad_ack_count;
};	/* struct our_advert */

struct our_eof
{
	uint32_t	flags;
};

struct our_notify
{
	uint32_t	write_len;
	uint32_t	write_index;
	uint32_t	write_flags;
};	/* struct our_notify */

struct our_stream_ack {
	uint32_t	recv_buf_empty;
	uint32_t	recv_buf_ack_count;
};	/* struct our_stream_ack */


/* structure of the negotiation request/reply message
 * used only during negotiation exchange
 *
 * is limited to 56 bytes on RDMA_PS_TCP on InfiniBand
 */
struct our_nego_mess
{
	uint32_t		client_credits;
	uint32_t		recv_buf_align;
	uint32_t		sp_max_size;
	uint8_t			version;
	uint8_t			subversion;
	uint8_t			subsubversion;
	uint8_t			is_seqpacket;
	uint32_t		fd_flags;
	uint64_t		recv_buf_addr;
	uint32_t		recv_buf_len;
	uint32_t		recv_buf_rkey;
};	/* struct our_nego_mess */


/* hd_type is in network byte order for send headers, but is converted to
 *	host byte order immediately upon reception into recv headers.
 * hd_seqno is in network byte order in send headers, but is converted to
 *	host byte order immediately upon reception into recv headers.
 */
struct our_header
{
	uint32_t	hd_type;
	uint32_t	hd_seqno;
};	/* struct our_header */


/* internal flag bits for transactions
 * these must be compatible with user-level flags that are defined as
 * user-visible constants for exs_recv() and exs_send() in exs.h
 * so far, these user-visible constants are:
 *	EXS_DONTWAIT
 *	EXS_BLOCK
 *	EXS_UNSIGNALED
 * and all smaller system-defined constants)
 *
 * These flags are stored in block->flags
 */
#define EXS_NOTIFY_ALREADY_DONE		(0x100000)

/* Used to signal that memory registration handle needs to be
 * deregistered after event completion. */
#define EXS_MR_DYNAMIC			(0x200000)

/* Copy data into unregistered buffer (SOCK_SEQPACKET only) */
#define EXS_COPY_UNREG			(0x400000)

struct exs_buf {
	void		*ptr;
	size_t		size;
	struct ibv_mr	*mr;
};

struct stream_advert_block
{
	struct our_advert		advert_buf;
	unsigned int			offset;
	unsigned int			advert_state;
	struct stream_advert_block	*next;
};

struct transaction_block
{
	int			expected_comp;	/* expected completion opcode --
						 * necessary because wc->opcode
						 * NOT valid unless wc->status
						 * is IBV_WC_SUCCESS
						 */
	uint32_t		flags;		/* for in-place buf & recycle*/
	struct evt_queue	*qhandle;	/*queue to post done event on*/
	struct exs_mhandle	*mhandle;
	struct exs_event	*user_event_ptr;/* user-level completion info*/
	size_t			recv_byte_len;  /* no. of bytes given to recv*/
	volatile size_t		buf_offset;	/*offset to unused byt in buf*/
	void			*user_buffer;	/* buffer supplied by user */
	size_t			user_length;	/* length requested by user */

	struct transaction_block*next;
	sem_t			sem;
	unsigned int		in_progress;
	int			index;
	unsigned int		send_state;
	unsigned int		outstanding_completion_count;
	struct ibv_mr		*block_mr;

	struct ibv_send_wr	send_wr;	/* send work request */
	struct ibv_send_wr	writeack_wr;	/* iWARP post-write ack */
	struct ibv_sge		sg_list[3];     /* 2-element send s/g list */
	struct our_header	*send_header;	/* 2-word header on sends */
	size_t			send_total_bytes;/* total data bytes sent */

	/* data to be sent in small packets may be copied into a separately
	 * allocated preregistered buffer that we keep track of here */
	struct exs_buf		send_buf;

	/* data to be received in small packets may be copied from a separately
	 * allocated preregistered buffer that we keep track of here */
	struct exs_buf		recv_rdma_buf;

	struct ibv_mr		*data_mr;	/* memory region of user data */

	struct our_eof		*eof_buf;	/* holds the EOF notification */
	struct our_advert	*advert_buf;	/* holds the advertisement */
	struct our_stream_ack	*ack_buf;	/* holds the SOCK_STREAM ack */
	struct our_notify	*writeack_buf;	/* holds the WRITE NOTIFY */
	unsigned int		unsignaled_count;

	struct ibv_recv_wr	recv_wr;	/* receive work request */

	/* the next 2 fields must be contiguous, and must be at the end
	 * of "struct transaction_block" because we will receive untagged
	 * send packets directly into here off the wire, and any immediate
	 * data in those send packets immediately follows the header off
	 * the wire into this memory.  The size of the memory to catch this
	 * immediate data is added to the size of "struct transaction_block"
	 * when "struct transaction_block" is dynamically allocated, so that
	 * the "recv_buf" field will be big enough for all immediate data.
	 * This additional size is given by "cb->small_packet_max_size",
	 * and can be set by the user with the SMALL_PACKET_MAX_SIZE attribute.
	 */
	struct our_header	*recv_header;	/* 2-word header on all recvs*/
	char			*recv_buf;
};	/* struct transaction_block */

/* This pending_accept structure contains parameters and pointers to
 * allocated memory for pending accepts.
 *
 * These structures are created dynamically in exs_accept(), where
 * they are threaded onto the "pending_accept_list" of the listening
 * socket passed in as a parameter to exs_accept().
 *
 * When our_con_man_thread() gets an RDMA_CM_EVENT_CONNECT_REQUEST on
 * a listening socket, it calls our_merge_pending_accept() which
 * tries to remove one of these structures from the listening socket's
 * "pending_accept_list" and to initialize lots of stuff for the new
 * connection before calling rdma_accept() to actually accept the connection.
 *
 * If the "pending_accept_list" is empty, we instead store the event's cm_id
 * in a new (or recyled) struct conn_requested, add that struct onto
 * the listening socket's conn_requested_list, and then increment the listening
 * socket's conn_requested_backlog count.
 *
 * When our_con_man_thread() gets an RDMA_CM_EVENT_ESTABLISHED on
 * an agent socket, it calls our_accept_established() which
 * adds the embedded (now accepted) connection * to the master
 * "conn_list", posts the EXS_EVT_ACCEPT event and finally
 * frees this pending_accept structure.
 *
 * All accesses to the listening socket's
 *	pending_accept_list
 *	conn_requested_list
 *	conn_requested_free_list
 * are  protected by that listening socket's "event_mutex".
 */
struct pending_accept {
	struct exs_conn		*accepted_conn;
	struct exs_acceptaddr	addrvec;
	struct exs_event	*evt_item;
	struct our_nego_mess	nego_send;
	struct evt_queue 	*qhandle;
	int			err_numb;
};	/* struct pending_accept */

struct conn_requested {
	struct rdma_cm_event	cm_event;
	struct our_nego_mess	nego_recv;
};	/* struct conn_requested */

/* internal flags for conn_flags field in struct exs_conn */

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off on user call to exs_start_close()
 * tested in our_completion_thread()
 *	to prevent start of new EXS I/O
 */
#define EXS_NEED_LOCAL_CLOSE	(0x10000)

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off upon con_man's receipt of RDMA_CM_EVENT_DISCONNECTED,
 * also turned off just before calling rdma_disconnect()
 * tested before each call to ibv_post_send() and ibv_post_recv()
 *	to prevent start of new OFED I/O
 */
#define EXS_NEED_DISCONNECT	(0x20000)

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off on user call to exs_shutdown(SHUT_RD), exs_close()
 *            iff recv buffer has been drained
 * turned off upon draining last byte from recv buffer
 *            if state is not EXS_CONN_RUNNING
 * tested in exs_recv()
 *	to ensure start of new EXS I/O is allowed
 */
#define EXS_OK_TO_ISSUE_RECV	(0x40000)

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off on user call to exs_close() or exs_shutdown(SHUT_WR)
 * tested in exs_send(), exs_sendfile()
 *	to ensure start of new EXS I/O is allowed
 */
#define EXS_OK_TO_ISSUE_SEND	(0x08000)

/* ok to issue exs_send or exs_recv */
#define EXS_OK_TO_ISSUE_IO	(0x48000)

/* turned on after our_completion_thread is up and joinable
 * turned off by our_completion_thread before it calls pthread_exit()
 * turned off by our_con_man_thread before it calls pthread_cancel()
 * tested in our_con_man_thread before calling pthread_cancel()
 */
#define EXS_NEED_COMP_CANCEL	(0x80000)

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off on send EOF for sending
 * tested on recv EOF before calling rdma_disconnect()
 * tested on exs_recv(), exs_send() in EXS_CONN_TERMINATING state
 *     to determine what error code to return
 */
#define EXS_NEED_EOF_SEND	(0x04000)

/* turned on after completely successful EXS_ACCEPT or EXS_CONNECT
 * turned off on send EOF for receiving
 * tested on recv EOF before calling rdma_disconnect()
 * tested on exs_recv(), exs_send() in EXS_CONN_TERMINATING state
 *     to determine what error code to return
 */
#define EXS_NEED_EOF_RECV	(0x02000)

/* EXS_NEED_EOF_SEND | EXS_NEED_EOF_RECV */
#define EXS_NEED_EOF		(0x06000)

/* turned on after init'd completion sync mutex/cond
 * turned off after destroy completion sync mutex/cond
 * tested prior to destruction
 */
#define EXS_NEED_DESTROY_COMP_SYNC (0x01000)

#define EXS_ALL_CONN_FLAGS	(0xff000)


/* Connection operations that vary based on the chosen transfer operation */
static const struct exs_conn_ops {
	int (*enqueue_send)(struct exs_conn *this_conn,
			    struct transaction_block *block, const void *buf,
			    size_t length);
	int (*xmit_if_match)(struct exs_conn *this_conn,
			     struct transaction_block *block);
	int (*handle_recv_advert)(struct exs_conn *cb,
			          struct transaction_block *block);
	int (*handle_send_advert)(struct exs_conn *cb,
				  struct transaction_block *block);
	int (*handle_send_write)(struct exs_conn *cb,
				 struct transaction_block *block);
	int (*handle_recv_write_notify)(struct exs_conn *cb,
					uint32_t byte_len, uint32_t imm_data);
} ops_seqpacket, ops_stream;


enum { EXS_MAX_STREAM_BUF_SIZE = 3221225472L, };

struct exs_stream_send_buf {
	/* Invariants:
	 *  n_full + n_empty == size
	 * Guarded by cb->stream_sync_mutex
	 */

	struct exs_buf	buf;

	/* new send data is copied here */
	unsigned int	send_offset;

	/* send is complete up to here */
	unsigned int	done_offset;

	/* WWI starting from here */
	unsigned int	take_offset;

	unsigned int	n_full;
	unsigned int	n_empty;
	unsigned int	n_remote; /* empty bytes in receiver buffer */
};

/* The layout of the stream buffer is really an array of cache lines
 *
 * +------+---------------------------------------------------------+
 * | 100  | RDMA WRITE 1 ...........................................|
 * +------+---------------------------------------------------------+
 * | 48   |..................END                                    |
 * +------+---------------------------------------------------------+
 * |      | RDMA WRITE 2 ............................END            |
 * +------+---------------------------------------------------------+
 *
 * Each RDMA WRITE will be aligned at a cache line boundary, so we must keep
 * track of the "holes" in the buffer in the used_bytes_list array.  Each entry
 * in this array contains the number of bytes from a single RDMA WRITE into our
 * buffer.  When we copy data out of the buffer, we copy up to the end of the
 * next RDMA WRITE, then increment the pointer to the next cache line.
 */
struct exs_stream_recv_buf {
	/* Invariants:
	 *  n_full <= size
	 * Guarded by cb->stream_sync_mutex
	 */

	struct exs_buf	buf;

	/* Represents the number of bytes in the next contiguous chunk */
	unsigned int	*full_bytes_array;
	unsigned int	*full_bytes_array_end;
	unsigned int	*next_entry_primed;
	unsigned int	*next_entry;

	/* data copied to application from here */
	unsigned int	take_offset;

	/* remote WWI from recv_offset, which is tracked only on sending side */

	unsigned int	n_full;
	unsigned int	n_empty;
	unsigned int	n_empty_threshold;
};

/* there are 6 pthread condition variables defined in struct exs_conn and 1
 * in struct evt_queue.  These all MUST be paired with a mutex in that same
 * structure that is locked when testing for the condition to be true, and
 * is unlocked automatically by the pthread_cond_wait() when waiting for a
 * signal.  In addition, most of these are also paired with an int counter
 * in the same struct that is positive when someone is waiting on the
 * pthread_cond_wait().  This counter is used by the signaller to know
 * whether or not issue the pthread_cond_broadcast().
 *
 *	pthread_cond_t		mutex_t		int counter
 *	----------------	-----------	----------------
 * struct exs_conn
 *	transmit_ok_notify	event_mutex	---
 * struct evt_queue
 *	full_notify		event_list_lock	waiting_for_notify
 *
 * We utilize this information in our_con_man_thread so that after it
 * detects and processes a RDMA_CM_EVENT_DISCONNECTED event, it will do a
 * broadcast on all condition variables whose counters are not zero.
 * This requires the condition test around the pthread_cond_wait() to test
 * for some disconnect condition as part of getting out of the wait loop.
 */

struct exs_conn
{
	unsigned int		refcount; /* number of pointers to me */
	int			sockno;	/* socket this connection represents */
	unsigned int		conn_flags; /* connection flags */
	enum conn_states	conn_state;/* state of the connection */
						/* guarded by event_mutex */

	struct rdma_cm_id	*cm_id;
	struct ibv_pd		*pd;

	struct transaction_block	*send_advert_array;
	struct exs_buf			send_advert_buf;
	struct transaction_block	*send_wwi_array;
	struct exs_buf			send_writeack_buf;
	struct transaction_block	*recv_array;
	struct our_list_head		recved_advert_queue;
	struct our_list_head		send_ack_empty_list;

	struct exs_buf			recv_control_buf;

	struct transaction_block	*send_ack_array;
	struct exs_buf			send_ack_buf;

	struct stream_advert_block	*recved_advert_initial_array;
	struct our_list_head		recved_advert_empty_list;
	struct our_list_head		recved_advert_extra_list;

	/* extra transaction block used only during shutdown */
	struct transaction_block	eof_block_array[EOF_BLOCK_COUNT];
	struct exs_buf			send_eof_buf;

	volatile unsigned int		pending_recv_count;
	struct transaction_block	*next_recv;
	struct transaction_block	*next_recv_primed;

	volatile unsigned int		pending_send_count;	/*all waiting*/
	struct transaction_block	*next_send;
	struct transaction_block	*next_send_primed;
	struct transaction_block	*next_send_unacked;

	pthread_mutex_t			advert_sequence_mutex;
	pthread_mutex_t			data_sequence_mutex;
	pthread_mutex_t			stream_sync_mutex;

	unsigned int			stream_wwi_counter;
	unsigned int			stream_ack_counter;
	unsigned int			stream_advert_counter;

	/* if positive, more unacked receives than sends;
	 * if negative, more unacked sends than receives */
	int				stream_advert_imbalance;

	unsigned int			ack_signal_count;
	unsigned int			advert_signal_count;
	unsigned int			data_signal_count;
	unsigned int			signal_limit;

	uint32_t			inline_limit;

	const struct exs_conn_ops	*ops;

	unsigned int		posted_credits; /* limit on exs_send blocks*/
	struct pending_connect	*conn_pending;	  /*to post completion events*/
	struct pending_connect	*close_pending;

	pthread_cond_t		transmit_ok_notify;  /* start completn thread*/

	struct pending_accept	*accept_pending;  /*to post completion events*/
	struct our_list_head	pending_accept_list; /* connections to accept*/
	struct exs_conn		*accept_parent;	  /* for agent, back pointer
						   * to listener "parent"; NULL
						   * for client, listener
						   */

	struct our_list_head	conn_requested_list; /*unmatched conn request*/
	struct our_list_head	conn_requested_free_list;
	int			conn_requested_backlog;	/* number on list */
	int			backlog_limit;		/* from exs_listen() */

	struct our_list_head	queue_cache_list;  /* struct evt_queue cache */

	/* set to non-zero after a call to our_socket_new() when type of
	 * newly allocated socket is SOCK_SEQPACKET
	 */
	uint8_t			is_seqpacket;

	/* set by our_con_man_thread when it is up and running
	 * initially clear
	 * protected by event_mutex while waiting in create_our_con_man_thread
	 * on wait condition is transmit_ok_notify
	 */
	uint8_t			con_man_thread_is_active;

	uint8_t			filler_bytes[2];

	/* requested queue pair capabilities,
	 * and those actually obtained by successful rdma_create_qp()
	 */
	struct ibv_qp_cap	requested_cap;
	struct ibv_qp_cap	obtained_cap;

	int			requested_cq_nelements;
	int			obtained_cq_nelements;

	int			obtained_path_mtu_size;

	uint64_t		recv_seqno;	/* seq no of last data recv */
	uint64_t		source_seqno;	/* seq no of next data recv */
	uint64_t		send_seqno;	/* seq no of next data send */
	uint32_t		last_unacked_advert_msg_seqno;
	uint32_t		next_send_advert_msg_seqno;
	uint32_t		next_recv_advert_msg_seqno;

	unsigned int		send_phase;
	unsigned int		recv_phase;

	struct rdma_event_channel *event_channel;
	pthread_mutex_t		event_mutex;	/* protects cm event CSs */
	pthread_t		event_thread;	/* processes asynch cm events*/

	struct ibv_cq		*cq;	/* 1 completion queue per connection */
	struct ibv_qp		*qp;	/* 1 queue pair per connection */
	struct ibv_comp_channel *comp_channel;
	struct ibv_wc		*wc_array; /* holds polled wcs */
	pthread_t		comp_thread;	/* processes completions */
	int			comp_thread_cpu; /*cpu binding for comp thread*/
	int			max_n_wc_slots;	/* no. of slots in wc_array */
	unsigned int		n_cq_events_that_need_ack;

	/* attributes that can be configured by a user */
	uint32_t		small_packet_max_size;
	size_t			stream_buf_size;
	uint32_t		fd_flags;	/* used by GETFD/SETFD */

	struct exs_stream_recv_buf	recv_buf;
	struct our_advert		remote_recv_buf;
	unsigned int			n_unacked_wwi;
	unsigned int			n_unacked_wwi_threshold;
	unsigned int			remote_buf_align;
	unsigned int			remote_buf_n_empty;
	unsigned int			remote_buf_offset;
	unsigned int			ack_seqno;

	struct our_nego_mess		nego_send;

	/* pointers to extra memory used only during initialization */
	struct transaction_block *nego_in_ptr, *nego_out_ptr;
};	/* struct exs_conn */


/* open_socket_table is the master hash array of all known "struct exs_conn"
 * connections of any type in any state, indexed by a hash of the fd
 *
 *	the table is initialized so that all entries are empty lists by calling
 *		our_init_open_socket_table()
 *	a connection is added to the end of conn_list by calling
 *		our_add_socket_to_list()
 *	a connection is removed from anywhere on conn_list by calling
 *		our_rem_socket_from_list()
 *	a connection with sockno equal to a given sockno is found by calling
 *		our_lookup_socket()
 *
 * all access to conn_list is protected by open_socket_rwlock
 */

/* use low-order 10 bits of the fd as the hash index into open_socket_table */
#define	OPEN_SOCKET_TABLE_POWER	10
#define OPEN_SOCKET_TABLE_SIZE	(1 << OPEN_SOCKET_TABLE_POWER)
#define OPEN_SOCKET_TABLE_MASK	(OPEN_SOCKET_TABLE_SIZE - 1)

struct hash_list {
	int sockno;
	struct exs_conn *conn;
	struct hash_list *next;
};

/* Reader-writer lock protecting open_socket_table
 * Write lock is held when adding or removing an entry
 * Read lock is held while referencing an entry
 * Lock should be held through the duration between getting the exs_conn
 * reference and incrementing the reference count (needed to avoid race between
 * thread calling exs_close() while another thread calls some other operation on
 * that socket)
 */
static pthread_rwlock_t open_socket_rwlock = PTHREAD_RWLOCK_INITIALIZER;

/* Hash table of open sockets indexed by (fd & OPEN_SOCKET_TABLE_MASK) */
static struct hash_list *open_socket_table[OPEN_SOCKET_TABLE_SIZE];


/* we define one protection domain (pd) per RDMA device on this system.
 * we assume that the number of RDMA devices will be small (in fact,
 * usually 1, and for a dual-ported system it will be 2).  Therefore, we use
 * a global fixed-size table, rather than a linked list, to store these pds.
 */
#define	MAX_PD_SLOTS	8

/* protects global protection domain table
 * is initialized in our_init_pd_table(), called from exs_init()
 */
static pthread_rwlock_t	pd_table_rwlock;

/* Table of protection domains, in order of creation.
 * Empty table slots are initialized to NULL.
 * New domains are created in the first NULL slot, starting at slot 0.
 * Searches start at slot 0 and continue until a slot value is NULL.
 * (Therefore, the table can only contain MAX_PD_SLOTS-1 actual pds.)
 *
 * Table entries are created (if needed) by our_get_pd(), called in 2 places:
 * 1.	exs_init() - for all the devices known at initialization time
 * 2.	our_setup_qp() - for each new connection
 *
 */

static struct ibv_pd *pd_table[MAX_PD_SLOTS] = {NULL, };


#if defined(ALLOW_EXS_STATISTICS)
static uint64_t statistics_counters[max_statistics_counter_type] = {0};
#endif


/* collect all forward declarations here at the beginning */
#include "forward_exs.h"


/* start declaring function bodies here */

#define PHASE_IS_INDIRECT(phase) (((phase) & 0x1) == 0x1)
#define PHASE_IS_DIRECT(phase) !(PHASE_IS_INDIRECT(phase))

/*
 *	exs_evt_type_to_string()
 *			- not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns pointer to printable string for each exs_event_t value
 */
EXS_EXPORT const char *
exs_evt_type_to_string(int value)
{
	if (value < 0 || value > EXS_EVT_SHUTDOWN)
		value = 0;
	return exs_evt_type_strings[value];
}	/* exs_evt_type_to_string */


/* Returns the string corresponding to the EXS wait type. */
EXS_EXPORT const char *
exs_wait_type_str(int wait_type)
{
	static const char *strs[] = {
		[EXS_WAIT_ADAPTIVE] = "EXS_WAIT_ADAPTIVE",
		[EXS_WAIT_NOTIFY] = "EXS_WAIT_NOTIFY",
		[EXS_WAIT_BUSY_POLL] = "EXS_WAIT_BUSY_POLL",
	};

	if (wait_type < EXS_WAIT_ADAPTIVE || wait_type > EXS_WAIT_BUSY_POLL) {
		return "unknown";
	}

	return strs[wait_type];
}	/* exs_wait_type_str */


/*
 * called from exs_accept(), exs_connect(), exs_send(), and exs_recv()
 *
 * gets pointer to next struct evt_queue on cb's queue_cache_list
 * if list is empty, tries to create a new struct evt_queue
 *
 * Returns pointer if struct exists or created ok, else NULL
 */
static inline struct evt_queue *
our_get_from_queue_cache(struct exs_conn *cb)
{
	struct evt_queue *local_qhandle;

	local_qhandle = exs_p_list_del_head(&cb->queue_cache_list);
	if (!local_qhandle) {
		local_qhandle = _evt_queue_init_internal(1);
		/* if couldn't create, local_qhandle NULL, errno already set */
	}
	if (local_qhandle) {
		atomic_add(1, &local_qhandle->refcount);
		atomic_add(1, &local_qhandle->npending);
	}

	return local_qhandle;		/* use our local qhandle */
}	/* our_get_from_queue_cache */


/*
 * called from exs_accept(), exs_connect(), exs_send(), and exs_recv()
 *
 * threads struct evt_queue pointed to by local_qhandle
 * onto end of cb's queue_cache_list
 */
static inline void
our_put_to_queue_cache(struct exs_conn *cb, struct evt_queue *local_qhandle)
{
	exs_p_list_add_tail(&cb->queue_cache_list, local_qhandle);
}	/* our_put_to_queue_cache */


/*
 * called from exs_accept(), our_close_accepting()
 *
 * undo the allocations and initializations done by our_create_queue_cache()
 *	frees up all items on cb's queue_cache_list
 *	then destroys cb's queue_cache_mutex
 */
static void
our_destroy_queue_cache(struct exs_conn *cb)
{
	int			j;
	struct evt_queue	*local_qhandle;

	TRACE_DBG("free all struct evt_queue on connection %p", (void *)cb);

	j = 0;
	while ((local_qhandle = exs_p_list_del_head(&cb->queue_cache_list))) {
		local_qhandle = _evt_queue_unref(local_qhandle, 1, 0);
		j++;
	}
	exs_p_list_destroy(&cb->queue_cache_list);

	TRACE_DBG("freed %d struct evt_queue from connection %p",
			j, (void *)cb);
}	/* our_destroy_queue_cache */


/*
 * called only from our_socket_new()
 *
 * initializes cb's queue_cache_list and cb's queue_cache_mutex
 *
 * returns 0 if all ok, else -1 on any error
 */
static int
our_create_queue_cache(struct exs_conn *cb)
{
	int	ret;

	ret = exs_p_list_init(&cb->queue_cache_list, 0, 0);
	if(ret)
	{
		OUR_PERROR(ret, "queue_cache_list init failed");
		errno = ret;
		ret = -1;
	}
	return ret;
}	/* our_create_queue_cache */


/*
 * called only from our_socket_new()
 */
static void
our_init_pending(struct exs_conn *this_conn)
{
	/* init list to track exs_accepts on this listening socket*/
	exs_p_list_init(&this_conn->pending_accept_list, 0, 0);

	/* init 2 lists to track connect_request events (handled in
	 * our_con_man_thread()) with no pending accepts to match
	 */
	exs_p_list_init(&this_conn->conn_requested_list, 0, 0);
	exs_p_list_init(&this_conn->conn_requested_free_list, 0, 0);
}	/* our_init_pending */


/*
 * called from exs_init()
 *
 * initializes the global rwlock for the global pd_table
 * (the table itself is statically initialized to NULL)
 */
static inline void
our_init_pd_table(void)
{
	pthread_rwlock_init(&pd_table_rwlock, NULL);
}	/* our_init_pd_table */


/* finds the global protection domain (pd) for this context.
 * In most cases, this will already be defined.
 * However, if it is not defined, we will define it here
 * and add it to the global table.
 *
 * Normally, need pd's are created and added only when
 * this function is called from exs_init().
 *
 * Once added to this table, pd's are never deleted.
 */
static struct ibv_pd *
our_get_pd(struct ibv_context *context)
{
	struct ibv_pd	**ptr;

	OUR_RWLOCK_RDLOCK(pd_table_rwlock);

	for (ptr = &pd_table[0]; *ptr != NULL; ptr++) {
		if ((*ptr)->context == context)
			break;
	}

	OUR_RWLOCK_UNLOCK(pd_table_rwlock);

	if (*ptr != NULL) {
		TRACE_DBG("returning pd %p", (void *)*ptr);
		return *ptr;
	}

	/* the pd for this context does not yet exist,
	 * create it if there is room in the table
	 */

	/* do the search again, this time with write locking so we
	 * have exclusive access to this table
	 */
	OUR_RWLOCK_WRLOCK(pd_table_rwlock);

	for (ptr = &pd_table[0]; *ptr != NULL; ptr++) {
		if ((*ptr)->context == context)
			break;
	}

	if (*ptr == NULL) {
		/* this context is still not in the table, try to add it */
		if (ptr >= &pd_table[MAX_PD_SLOTS-1]) {
			/* no room in the table for another pd */
			TRACE_ERR("global pd_table full, max of %d protection domains",
				MAX_PD_SLOTS - 1);
			errno = ENOMEM;
		} else {/* create a protection domain for this context and
			 * store it in this slot */
			*ptr = ibv_alloc_pd(context);
			if(*ptr == NULL) {
				OUR_PERROR(errno, "ibv_alloc_pd failed");
			} else {
				TRACE_DBG("created ibv pd %p", (void *)*ptr);
			}
		}
	}

	OUR_RWLOCK_UNLOCK(pd_table_rwlock);

	TRACE_DBG("returning pd %p", (void *)*ptr);
	return *ptr;
}	/* our_get_pd */


/*
 * so far called only by exs_init() in error situation
 *
 * deallocates all pd's in the global pd_table
 * and destroys the global rwlock for the global pd_table
 */
void
our_destroy_pd_table(void)
{
	struct ibv_pd	**ptr;

	OUR_RWLOCK_WRLOCK(pd_table_rwlock);

	for (ptr = &pd_table[0]; *ptr != NULL; ptr++) {
		ibv_dealloc_pd(*ptr);
		*ptr = NULL;
	}

	OUR_RWLOCK_UNLOCK(pd_table_rwlock);

	pthread_rwlock_destroy(&pd_table_rwlock);
}	/* our_destroy_pd_table */


/* Increments the refcount of the socket object. */
static struct exs_conn *
our_socket_ref(struct exs_conn *conn)
{
	if (conn) {
#if defined(ALLOW_EXS_TRACE) || !defined(NDEBUG)
		int new_refcount =
#endif
		atomic_add(1, &conn->refcount);
		assert(conn->conn_state == EXS_CONN_UNOPENED
							|| new_refcount != 1);
		TRACE_DBG("cb %p increment refcount to %d",
						(void *)conn, new_refcount);
	}
	return conn;
}	/* our_socket_ref */


/*
 * called from:
 *	exs_accept() when a new accept is setup for an agent socket
 *	exs_socket() when user creates a new listening or client socket
 *
 * allocates a new struct exs_conn, and partially initializes it:
 *	(any fields not explicitly set are initialized to 0 by default)
 *	allocates a new struct pending_connect and points conn_pending at it
 *	creates the queue_cache_list and queue_cache_mutex
 *
 * this is the ONLY place where a struct exs_conn is allocated!
 *
 * returns a pointer to a newly allocated struct exs_conn for this socket
 * or NULL if can't get memory for this struct
 */
static struct exs_conn *
our_socket_new(void)
{
	struct exs_conn	*new_conn;

	/* allocate a new, zeroed-out struct exs_conn */
	new_conn = (struct exs_conn*)EXS_CALLOC(1, sizeof(struct exs_conn));
	if(new_conn == NULL) {
		errno = ENOMEM;
		goto err0;
	}

	/* initialize thread, state, flags */
	new_conn->conn_state = EXS_CONN_UNOPENED;
	new_conn->conn_flags = EXS_NEED_LOCAL_CLOSE;

	new_conn->conn_pending = (struct pending_connect *)
				EXS_CALLOC(1, sizeof(struct pending_connect));
	if(new_conn->conn_pending == NULL) {
		errno = ENOMEM;
		goto err1;
	}

	new_conn->close_pending = (struct pending_connect *)
				EXS_CALLOC(1, sizeof(struct pending_connect));
	if(new_conn->close_pending == NULL) {
		errno = ENOMEM;
		goto err1a;
	}

	if(our_create_queue_cache(new_conn))
		goto err2;

	/* set up some empty lists that are actually used only by
	 * listening sockets
	 */
	our_init_pending(new_conn);

	new_conn->inline_limit = INT_MAX;
	new_conn->stream_buf_size = EXS_MAX_STREAM_BUF_SIZE;

	goto err0;

err2:
	EXS_FREE(new_conn->close_pending);
err1a:
	EXS_FREE(new_conn->conn_pending);
err1:
	EXS_FREE(new_conn);
	return NULL;
err0:
	return our_socket_ref(new_conn);
}	/* our_socket_new */


static void
our_free_all_blocks(struct exs_conn *cb)
{
	struct stream_advert_block *advert;
	unsigned int i;

	/* first recycle all the blocks we created */
	if (cb->send_wwi_array) {
		for (i = 0; i < cb->posted_credits; ++i) {
			EXS_FREE(cb->send_wwi_array[i].advert_buf);
			our_uninit_transaction_block(&cb->send_wwi_array[i]);
			sem_destroy(&cb->send_wwi_array[i].sem);
		}
		EXS_FREE(cb->send_wwi_array);
		if (cb->fd_flags & EXS_FD_NO_WRITE_WITH_IMM) {
			exs_buf_free(&cb->send_writeack_buf);
		}
	}
	if (cb->send_advert_buf.ptr) {
		for (i = 0; i < cb->posted_credits; ++i) {
			our_uninit_transaction_block(&cb->send_advert_array[i]);
			sem_destroy(&cb->send_advert_array[i].sem);
		}
		exs_buf_free(&cb->send_advert_buf);
		EXS_FREE(cb->send_advert_array);
	}

	if (cb->recv_control_buf.ptr) {
		exs_buf_free(&cb->recv_control_buf);
		EXS_FREE(cb->recv_array);
		cb->recv_array = NULL;
	}

	if (cb->recved_advert_initial_array) {
		EXS_FREE(cb->recved_advert_initial_array);
		while ((advert = exs_p_list_del_head(
					&cb->recved_advert_extra_list))) {
			EXS_FREE(advert);
		}
		exs_p_list_destroy(&cb->recved_advert_extra_list);
		exs_p_list_destroy(&cb->recved_advert_empty_list);
		exs_p_list_destroy(&cb->recved_advert_queue);
	}
	if (cb->send_ack_buf.ptr) {
		/* free the initially created ACK blocks */
		for (i = 0; i < cb->posted_credits; ++i) {
			our_uninit_transaction_block(
					&cb->send_ack_array[i]);
		}
		exs_buf_free(&cb->send_ack_buf);
		EXS_FREE(cb->send_ack_array);
		exs_p_list_destroy(&cb->send_ack_empty_list);
	}

	if(cb->nego_out_ptr != NULL)
	{	/* need to free nego block */
		TRACE_DBG("free nego out transaction block %p",
			(void *)cb->nego_out_ptr);
		/* FIXME: hack to get rid of memory leak -- need a more
		 * permanent solution */
		ibv_dereg_mr(cb->nego_out_ptr->block_mr);
		EXS_FREE(cb->nego_out_ptr->send_header);
		our_free_transaction_block(cb->nego_out_ptr);
		cb->nego_out_ptr = NULL;	/* so we don't free it again */
	}
}	/* our_free_all_blocks */


static void
our_ibv_socket_free(struct exs_conn *cb)
{
	if (cb->qp) {
		our_free_qp(cb);
		assert(cb->qp == NULL);
	}
	if (cb->cm_id) {
		if (rdma_destroy_id(cb->cm_id)) {
			OUR_PERROR(errno, "rdma_destroy_id");
		}
		cb->cm_id = NULL;
	}
	if (cb->event_channel) {
		rdma_destroy_event_channel(cb->event_channel);
		cb->event_channel = NULL;
	}
}	/* our_ibv_socket_free */


/* Undo the allocations and initializations taken by our_socket_new()
 * Called only from our_socket_unref() when refcount reaches 0. */
void
our_socket_free(struct exs_conn *cb)
{
	struct pending_connect *close_pending = cb->close_pending;
	int i;

	assert(cb->conn_state == EXS_CONN_UNLISTENING
					|| cb->conn_state == EXS_CONN_BOUND
					|| cb->conn_state == EXS_CONN_UNBOUND
					|| cb->conn_state == EXS_CONN_UNOPENED
					|| cb->conn_state == EXS_CONN_FINISHED
					|| cb->conn_state == EXS_CONN_TERMINATED
					|| cb->conn_state == EXS_CONN_CLOSED);
	assert(cb->refcount == 0);
	if (!cb->is_seqpacket) {
		exs_buf_free(&cb->recv_buf.buf);
		EXS_FREE(cb->recv_buf.full_bytes_array);
	}
	our_ibv_socket_free(cb);
	our_destroy_completion_synch(cb);

	our_free_all_pending_lists(cb);
	if (cb->send_eof_buf.ptr) {
		for (i = 0; i < EOF_BLOCK_COUNT; ++i) {
			our_uninit_transaction_block(&cb->eof_block_array[i]);
		}
		exs_buf_free(&cb->send_eof_buf);
	}
	our_destroy_queue_cache(cb);
	EXS_FREE(cb->conn_pending);
	TRACE_DBG("FREEd conn %p on socket %d", (void *)cb, cb->sockno);
	EXS_FREE(cb);

	if (close_pending->qhandle) {
		_post_event_to_full_evt_queue(close_pending);
	}
	EXS_FREE(close_pending);
}	/* our_socket_free */


/* Remove reference on socket, and free it if there are no more references.
 * Returns NULL, and intended to be called like this:
 *
 *     cb = our_socket_unref(cb);
 */
EXS_WARN_UNUSED_RETURN static struct exs_conn *
our_socket_unref(struct exs_conn *cb)
{
	int new_refcount = atomic_sub(1, &cb->refcount);
	TRACE_DBG("cb %p decrement refcount to %d", (void *)cb, new_refcount);
	if (!new_refcount) {
		assert(cb->conn_state == EXS_CONN_UNOPENED
				|| cb->conn_state == EXS_CONN_ACCEPTING
				|| cb->conn_state == EXS_CONN_AGENT_SETUP
				|| cb->close_pending->qhandle != NULL);
		our_socket_free(cb);
	}
	return NULL;
}	/* our_socket_unref() */


/*
 * called from exs_init()
 *
 * initializes the open_socket_table so all entries are empty lists
 */
static void
our_init_socket_table(void)
{
	memset(open_socket_table, 0, sizeof(open_socket_table));
}	/* our_init_socket_table */


/*
 * Acquires lock on socket table (write lock if exclusive, read lock if not
 * exclusive)
 */
static inline void
our_socket_table_acquire(int exclusive)
{
	if (exclusive) {
		OUR_RWLOCK_WRLOCK(open_socket_rwlock);
	} else {
		OUR_RWLOCK_RDLOCK(open_socket_rwlock);
	}
}	/* our_socket_table_acquire */


/*
 * Releases current thread's lock on socket table, whether or not it is
 * exclusive.
 */
static inline void
our_socket_table_release(void)
{
	OUR_RWLOCK_UNLOCK(open_socket_rwlock);
}	/* our_socket_table_release */


/*
 * called from our_accept_established() by an agent,
 *	and exs_socket() by a client or a listener
 *
 * adds a connection to end of conn_list
 *
 * open_socket_rwlock must NOT be held before calling this method, as this
 * method will grab the write lock
 *
 * returns 0 on success; errno on failure
 */
static int
our_add_socket_to_list(struct exs_conn *this_conn)
{
	unsigned int	index;
	struct hash_list *ptr;
	int ret;

	index = this_conn->sockno & OPEN_SOCKET_TABLE_MASK;

	our_socket_table_acquire(1);

	for (ptr = open_socket_table[index]; ptr != NULL; ptr = ptr->next) {
		if (ptr->sockno == -1) {
			ptr->sockno = this_conn->sockno;
			ptr->conn = this_conn;
			ret = 0;
			goto out;
		}
	}
	if (!(ptr = malloc(sizeof(*ptr)))) {
		ret = errno;
		goto out;
	}
	ptr->sockno = this_conn->sockno;
	ptr->conn = this_conn;
	ptr->next = open_socket_table[index];
	open_socket_table[index] = ptr;
	TRACE_DBG("Add new node %p with fd %d at index %d",
			(void *)ptr, this_conn->sockno, index);
	ret = 0;

out:
	our_socket_table_release();
	return ret;
}	/* our_add_socket_to_list */


/* returns a pointer to the struct exs_conn that corresponds to this socket
 * or NULL if no such connection exists yet
 *
 * open_socket_rwlock MUST be locked before calling this method, because the
 * caller may either acquire the read lock or the write lock, and the caller
 * must hold the lock until it increments the reference count on the exs_conn
 * object
 */
static inline struct exs_conn *
our_lookup_socket_(int sockno)
{
	struct hash_list *ptr;
	struct exs_conn		*this_conn;
	unsigned int		index;

	index = sockno & OPEN_SOCKET_TABLE_MASK;
	this_conn = NULL;

	for (ptr = open_socket_table[index]; ptr != NULL; ptr = ptr->next) {
		if (ptr->sockno == sockno) {
			/* found the connection that matches this socket */
			this_conn = ptr->conn;
			break;
		}
	}
	/* normal loop finish if found no connection matching this socket */

	return this_conn;

}	/* our_lookup_socket_ */


/* returns a pointer to the struct exs_conn that corresponds to this socket
 * or NULL if no such connection exists yet
 *
 * open_socket_rwlock MUST NOT be locked before calling this method, as this
 * method will grab the read lock on the socket table.
 */
static inline struct exs_conn *
our_lookup_socket(int sockno)
{
	struct exs_conn *conn;
	our_socket_table_acquire(0);
	conn = our_socket_ref(our_lookup_socket_(sockno));
	our_socket_table_release();
	return conn;
}	/* our_lookup_socket */


/*
 * called from exs_close() and exs_blocking_close()
 *
 * removes a connection from anywhere on conn_list
 *
 * the write lock of open_socket_rwlock MUST be locked prior to calling
 * our_lookup_socket, and the lock must be held continuously until this method
 * is called
 */
static void
our_rem_socket_from_list(struct exs_conn *old_conn)
{
	struct hash_list *ptr;
	unsigned int index;

	index = old_conn->sockno & OPEN_SOCKET_TABLE_MASK;

	for (ptr = open_socket_table[index]; ptr != NULL; ptr = ptr->next) {
		if (ptr->sockno == old_conn->sockno) {
			ptr->sockno = -1;
			ptr->conn = NULL;
			return;
		}
	}
}	/* our_rem_socket_from_list */


static inline const char *
our_conn_state_str(int state)
{
	/* We take an int here instead of "enum conn_states" because older gcc
	 * versions will complain about our check that state is non-negative
	 * (effectively), since the enum type happens to be unsigned, even
	 * though the C standard does not require it to be so. */
	static const char *state_labels[] = {
		[EXS_CONN_FINISHED] = "EXS_CONN_FINISHED",
		[EXS_CONN_UNOPENED] = "EXS_CONN_UNOPENED",
		[EXS_CONN_UNBOUND] = "EXS_CONN_UNBOUND",
		[EXS_CONN_BOUND] = "EXS_CONN_BOUND",
		[EXS_CONN_ADDRESSING] = "EXS_CONN_ADDRESSING",
		[EXS_CONN_ROUTING] = "EXS_CONN_ROUTING",
		[EXS_CONN_CONNECTING] = "EXS_CONN_CONNECTING",
		[EXS_CONN_RUNNING] = "EXS_CONN_RUNNING",
		[EXS_CONN_LISTENING] = "EXS_CONN_LISTENING",
		[EXS_CONN_ACCEPTING] = "EXS_CONN_ACCEPTING",
		[EXS_CONN_CLOSING] = "EXS_CONN_CLOSING",
		[EXS_CONN_TERMINATING] = "EXS_CONN_TERMINATING",
		[EXS_CONN_UNLISTENING] = "EXS_CONN_UNLISTENING",
		[EXS_CONN_CLOSED] = "EXS_CONN_CLOSED",
		[EXS_CONN_CLIENT_SETUP] = "EXS_CONN_CLIENT_SETUP",
		[EXS_CONN_AGENT_SETUP] = "EXS_CONN_AGENT_SETUP",
		[EXS_CONN_DELAY_CLOSING] = "EXS_CONN_DELAY_CLOSING",
		[EXS_CONN_TERMINATED] = "EXS_CONN_TERMINATED",
	};
	assert(state >= EXS_CONN_FINISHED && state <= EXS_CONN_TERMINATED);
	return state_labels[state];
}	/* our_conn_state_str */


/* routine to print the values of "enum rdma_cm_event_type" as defined in
 * /usr/include/rdma/rdma_cma.h
 */
static void
our_print_event_type(const char *message1, const char *message2,
			enum rdma_cm_event_type event_type)
{
	const char *ptr;

	ptr = rdma_event_str(event_type);
	TRACE_ERR("%s: %s %s(%d)", message1, message2, ptr, event_type);
}	/* our_print_event_type */


#if defined(ALLOW_EXS_TRACE)
/* routine to print the values of "enum ibv_wc_status" as defined in
 * /usr/include/infiniband/verbs.h
 */
static void
our_pstatus(const char *message, enum ibv_wc_status status)
{
	TRACE_ERR("%s: %s", message, ibv_wc_status_str(status));
}	/* our_pstatus */


/* routine to trace the values of "enum rdma_cm_event_type" as defined in
 * /usr/include/rdma/rdma_cma.h
 */
static void
our_trace_event_type(struct exs_conn *cb, const char *message1,
		     const char *message2, enum rdma_cm_event_type event_type)
{
	const char *ptr;

	ptr = rdma_event_str(event_type);
	TRACE_ERR("%s state %s %s(%d): %s",
		message1, our_conn_state_str(cb->conn_state),
		message2, event_type, ptr);
}	/* our_trace_event_type */
#endif /* defined(ALLOW_EXS_TRACE) */



static socklen_t
our_copy_src_addr(struct rdma_cm_id *cm_id, struct sockaddr *name,
			socklen_t namelen)
{
	struct sockaddr	*tmp;
	int		this_family;
	socklen_t	n;

	tmp = rdma_get_local_addr(cm_id);
	if (tmp->sa_family != PF_INET6) {
		this_family = PF_INET;
		n = sizeof(struct sockaddr_in);
	} else {
		this_family = PF_INET6;
		n = sizeof(struct sockaddr_in6);
	}
	if (n > 2) {
		memcpy(name, tmp, MIN(n, namelen));
		name->sa_family = this_family;
	}
	return n;
}	/* our_copy_src_addr */


static socklen_t
our_copy_dst_addr(struct rdma_cm_id *cm_id, struct sockaddr *name,
			socklen_t namelen)
{
	struct sockaddr	*tmp;
	int		this_family;
	socklen_t	n;

	tmp = rdma_get_peer_addr(cm_id);
	if (tmp->sa_family != PF_INET6) {
		this_family = PF_INET;
		n = sizeof(struct sockaddr_in);
	} else {
		this_family = PF_INET6;
		n = sizeof(struct sockaddr_in6);
	}
	if (n > 2) {
		memcpy(name, tmp, MIN(n, namelen));
		name->sa_family = this_family;
	}
	return n;
}	/* our_copy_dst_addr */


/*
 * called only from our_con_man_thread(), 4 times
 *
 * When called, event_mutex MUST be locked.
 */
static void
our_deal_with_bad_event(struct exs_conn *cb, struct rdma_cm_event *event)
{
	int	ret = 0, level;

	TRACE_EVENT_TYPE(cb, __func__, "got unexpected cm event type", event->event);

	TRACE_DBG("cb %p error code in event status = %d",
			(void *)cb, event->status);
	if(event->status < 0)
		{
		ret = -event->status;
		OUR_PERROR(ret, " cb %p bad event status, conn_state %s",
			(void *)cb,
			our_conn_state_str(cb->conn_state));
		}

	switch(cb->conn_state) {
	case EXS_CONN_ADDRESSING:
		/* EADDRNOTAVAIL, Cannot assign requested address */
	case EXS_CONN_ROUTING:
		/* EADDRNOTAVAIL, Cannot assign requested address ??? */
		if(ret == 0)
			ret = EADDRNOTAVAIL;
		level = 1;
		break;

	case EXS_CONN_ACCEPTING:
	case EXS_CONN_CONNECTING:
		if(ret == 0) {
			if(event->event == RDMA_CM_EVENT_UNREACHABLE)
				ret = ENETUNREACH;  /* Network is unreachable */
			else	/* event->event == RDMA_CM_EVENT_REJECTED or
				 * event->event == RDMA_CM_EVENT_CONNECT_ERROR
				 */
				ret = ECONNREFUSED; /*Connection reset by peer*/
		}
		level = 4;
		break;

	default:
		our_print_event_type(__func__, "can't handle event type",
								event->event);
		if(ret == 0)
			ret = EIO;		    /* Generic I/O error */
		level = 1;
		break;
	}	/* switch */

	our_connect_returning(cb, level, ret);

}	/* our_deal_with_bad_event */


static inline int
our_ack_cm_event(struct rdma_cm_event *event)
{
	int ret;

	ret = rdma_ack_cm_event(event);
	if(ret)
	{
		ret = errno;
		TRACE_ERR("rdma_ack_cm_event failed, error %d", ret);
	}
	return ret;
}	/* our_ack_cm_event */


/*
 * called from:
 *		our_start_close()   (on client close after remote DISCONNECTED)
 *		our_merge_pending_accept()	(on an error)
 *		our_connect_returning()		(on an error)
 *		our_close_accepting()		(on agent remote DISCONNECTED
 *						and agent local exs_close())
 *		our_close_connecting()		(on client local DISCONNECTED)
 *
 * purpose is to cancel our_completion_thread(), as the name implies
 *
 * when called, event_mutex MUST be locked
 * to prevent a race with our_completion_thread()
 */
static void
our_cancel_completion_thread(struct exs_conn *this_conn)
{
	int	ret;

	/* give completion thread a chance to get the "last" event */
	sched_yield();

	/* FIXME: unportable pthread_t print */
	TRACE_DBG("CANCELing comp_thread %lu on conn %p, conn_state %s, conn_flags %06x",
		this_conn->comp_thread, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state),
		this_conn->conn_flags);

	if (!(this_conn->conn_flags & EXS_NEED_COMP_CANCEL))
	{
		/* our_completion_thread did pthread_exit(), no need to cancel*/
		TRACE_DBG("pthread_cancel of comp_thread %lu not done, conn %p, conn_state %s, conn_flags %06x",
			this_conn->comp_thread, (void *)this_conn,
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);
	} else {
		if ((ret = pthread_cancel(this_conn->comp_thread))) {
			OUR_PERROR(ret, "pthread_cancel of comp_thread %lu failed, conn %p, conn_state %s, conn_flags %06x",
				this_conn->comp_thread, (void *)this_conn,
				our_conn_state_str(this_conn->conn_state),
				this_conn->conn_flags);
			/* ignore this error, it means we just lost a race to do
			 * it */
		} else {
			TRACE_DBG("CANCELed  comp_thread %lu on conn %p",
				this_conn->comp_thread, (void *)this_conn);
		}

		/* ensure this is off */
		atomic_clear_bits(EXS_NEED_COMP_CANCEL, &this_conn->conn_flags);
		sched_yield();
	}
}	/* our_cancel_completion_thread */


/* This function clears connection flags in response to an event, except that if
 * there is still data in the local recv buffer, the EXS_OK_TO_ISSUE_RECV flag
 * will *not* be cleared.  This flag must then be cleared when the local recv
 * buffer is drained.  The previous value of the flags is returned. */
static int
our_clear_conn_flags(struct exs_conn *cb, int flags_to_clear)
{
	int ret;

	if (!cb->is_seqpacket) {
		OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
		if ((flags_to_clear & EXS_OK_TO_ISSUE_RECV)
						&& cb->recv_buf.n_full) {
			flags_to_clear &= ~EXS_OK_TO_ISSUE_RECV;
		}
	}

	ret = atomic_clear_bits(flags_to_clear, &cb->conn_flags);

	if (!cb->is_seqpacket) {
		OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
	}
	return ret;
}	/* our_clear_conn_flags */


/* used only in our_con_man_cancel_handler and our_con_man_thread */
struct con_man_cancel_params {
	struct exs_conn		*cb;
	struct rdma_cm_event	*event;
};


static int
our_check_nego_recv(struct exs_conn *listen_conn,
					const struct our_nego_mess *nego_recv)
{
	/* client sent private data, check the version number and socktype */
	if (nego_recv->version != EXS_VERSION
		|| nego_recv->subversion != EXS_SUBVERSION) {
		TRACE_ERR("client sent version %u.%u, expected %u.%u",
			nego_recv->version, nego_recv->subversion,
			EXS_VERSION, EXS_SUBVERSION);
		/* Operation not supported on transport endpoint */
		errno = EOPNOTSUPP;
		return -1;
	}

	/* check the socktype */
	if (nego_recv->is_seqpacket != listen_conn->is_seqpacket) {
		TRACE_ERR("client requested %s, server expected %s",
			nego_recv->is_seqpacket ? "SOCK_SEQPACKET" : "SOCK_STREAM",
			listen_conn->is_seqpacket ? "SOCK_SEQPACKET" : "SOCK_STREAM");
		/* Operation not supported on transport endpoint */
		errno = EOPNOTSUPP;
		return -1;
	}

	/* check the fd_flags */
	if (!nego_recv->is_seqpacket && (nego_recv->fd_flags & EXS_FD_NODELAY)) {
		errno = EOPNOTSUPP;
		return -1;
	}

	return 0;
}	/* our_check_nego_recv */


static inline void
our_set_send_credit_limits(struct exs_conn *cb, int value)
{
	if((cb->posted_credits = value) < 2)
		cb->posted_credits = 2;
	if ((cb->signal_limit = value / 6) < 1)
		cb->signal_limit = 1;
	TRACE_DBG("cb %p set posted_credits %d, signal_limit %d",
			(void *)cb, cb->posted_credits, cb->signal_limit);
	cb->stream_wwi_counter = cb->posted_credits;
	cb->stream_ack_counter = cb->posted_credits;
	cb->stream_advert_counter = cb->posted_credits;
}	/* our_set_send_credit_limits */


/*
 * called from our_merge_pending_accept() and exs_socket()
 *
 * initializes mutexes, condition variables, lists needed by all sockets
 */
static int
our_init_completion_synch(struct exs_conn *this_conn)
{
	int	ret;

	ret = pthread_mutex_init(&this_conn->event_mutex, NULL);
	if(ret)
	{
		OUR_PERROR(ret, "pthread_mutex_init event_mutex failed");
		goto err1;
	}

	ret = pthread_cond_init(&this_conn->transmit_ok_notify, NULL);
	if(ret)
	{
		OUR_PERROR(ret, "pthread_cond_init transmit_ok_notify failed");
		goto err5;
	}

	atomic_set_bits(EXS_NEED_DESTROY_COMP_SYNC, &this_conn->conn_flags);
	goto err;

err5:
	pthread_mutex_destroy(&this_conn->event_mutex);
err1:
err:
	return ret;
}	/* our_init_completion_synch */


static void
our_setup_accepted_conn(struct exs_conn *listen_conn,
		struct exs_conn *accepted_conn,
		const struct our_nego_mess *nego_recv)
{
	uint32_t value;

	/* accept_parent back pointer used by our_completion_thread()
	 * to prevent processing completions until setup is finished here
	 */
	accepted_conn->accept_parent = listen_conn;

	/* client's version matches ours, negotiate parameter values
	 * by keeping the minimum of client's and listener's values
	 */
	TRACE_DBG("%21s %u.%u.%u", "client's exs version",
		nego_recv->version, nego_recv->subversion,
		nego_recv->subsubversion);

	/* agent's credits set to min(client's local, listener's remote)
	 */
	value = ntohl(nego_recv->client_credits);
	if (value > listen_conn->posted_credits) {
		value = listen_conn->posted_credits;
	}
	our_set_send_credit_limits(accepted_conn, value);

	/* agent's sp_max_size set to min(client's, listener's) */
	value = ntohl(nego_recv->sp_max_size);
	if (value > listen_conn->small_packet_max_size) {
		value = listen_conn->small_packet_max_size;
	}
	accepted_conn->small_packet_max_size = value;

	if (!accepted_conn->is_seqpacket
				&& ntohl(nego_recv->recv_buf_len) > 0) {
		/* grab remote recv buffer information */
		accepted_conn->remote_buf_n_empty
			= ntohl(nego_recv->recv_buf_len);
		accepted_conn->remote_recv_buf.ad_buf_addr
			= ntohll(nego_recv->recv_buf_addr);
		accepted_conn->remote_recv_buf.ad_buf_len
			= accepted_conn->remote_buf_n_empty;
		accepted_conn->remote_recv_buf.ad_buf_key
			= ntohl(nego_recv->recv_buf_rkey);
		accepted_conn->remote_buf_align
			= ntohl(nego_recv->recv_buf_align);
	} else {
		accepted_conn->stream_buf_size = 0;
	}

	/* other accepted connection attributes are same as listener's */
	accepted_conn->signal_limit
				= listen_conn->signal_limit;
	accepted_conn->fd_flags = listen_conn->fd_flags;
	accepted_conn->stream_buf_size = listen_conn->stream_buf_size;
	accepted_conn->is_seqpacket = listen_conn->is_seqpacket;
	accepted_conn->ops = listen_conn->ops;

	/* redundant, but makes it easier to post exs_event_t events */
	accepted_conn->conn_pending->sockno = accepted_conn->sockno;
}	/* our_setup_accepted_conn */


static inline struct ibv_mr *
our_call_ibv_reg_mr(struct ibv_pd *pd, void *addr, size_t size, int flags,
			const char *mess)
{
	struct ibv_mr	*mr_ptr;
	int		ret;

	TRACE_DBG("pd %p, addr %p, size %zu, flags %0x for %s",
			(void *)pd, (void *)addr, size, flags, mess);

	if((mr_ptr = ibv_reg_mr(pd, addr, size, flags)) == NULL)
	{
		ret = errno;
		OUR_PERROR(ret, "failed ibv_reg_mr for %s: pd %p, addr %p, size %zu, flags 0x%x",
			mess, (void *)pd, (void *)addr, size, flags);
		errno = ret;
	}
	else
	{
		STATISTICS_INCREMENT(ibv_reg_mr_calls);
		TRACE_DBG("done pd %p, addr %p, size %zu, flags 0x%x for %s at mr %p",
			(void *)pd, (void *)addr, size, flags, mess,
			(void *)mr_ptr);
	}
	return mr_ptr;
}	/* our_call_ibv_reg_mr */


/* Deregisters and frees an EXS internal buffer.  Returns 0 on success or the
 * value of errno on failure. */
static int
exs_buf_free(struct exs_buf *buf)
{
	if (buf->ptr != NULL) {
		int ret;
		STATISTICS_INCREMENT(ibv_dereg_mr_calls);
		ret = ibv_dereg_mr(buf->mr);
		EXS_FREE(buf->ptr);
		buf->ptr = NULL;
		return ret;
	} else {
		return 0;
	}
}	/* exs_buf_free */


/* Allocates and registers an EXS internal buffer.  Returns 0 on success or the
 * value of errno on failure. */
static int
exs_buf_alloc(struct exs_conn *cb, struct exs_buf *buf, size_t size,
					int access, const char *message)
{
	long align;
	long min_align;
	int ret;

	if (buf->ptr != NULL) {
		TRACE_ERR("Attempt to allocate EXS buffer \"%s\" twice",
				message);
	}

	min_align = 64;
	align = sysconf(_SC_PAGESIZE);
	if (align < min_align) {
		align = min_align;
	}

	TRACE_DBG("memalign buffer %s align %ld size %zu",
							message, align, size);
	if ((ret = EXS_MEMALIGN(&buf->ptr, align, size)) != 0) {
		goto out;
	}
	buf->size = size;

	if ((buf->mr = our_call_ibv_reg_mr(cb->pd, buf->ptr, buf->size,
					access, message)) == NULL) {
		ret = errno;
		EXS_FREE(buf->ptr);
		buf->ptr = NULL;
		goto out;
	}
	ret = 0;

out:
	return ret;
}	/* exs_buf_alloc */


static inline int
our_create_stream_recv_buf(struct exs_conn *cb)
{
	long page_size;

	errno = 0;
	page_size = sysconf(_SC_PAGE_SIZE);
	if (page_size < 0) {
		if (errno != 0 && errno != EINVAL) {
			/* Some other serious error happened */
			OUR_PERROR(errno, "Could not get system page size");
			return -1;
		}
		/* Default the page size to a reasonable value */
		page_size = 4096;
	}

	do {
		if ((errno = exs_buf_alloc(cb, &cb->recv_buf.buf,
					cb->stream_buf_size,
			                IBV_ACCESS_LOCAL_WRITE
					|IBV_ACCESS_REMOTE_WRITE,
					"recv stream buffer")) == 0) {
			break;
		} else {
			cb->stream_buf_size /= 2;
		}
	} while (cb->stream_buf_size > (unsigned long)page_size);
	if (errno != 0) {
		OUR_PERROR(errno, "Could not allocate local stream recv buf");
		return -1;
	}

	cb->recv_buf.full_bytes_array =
		EXS_CALLOC(cb->stream_buf_size / our_cache_line_size,
					sizeof(*cb->recv_buf.full_bytes_array));
	if (!cb->recv_buf.full_bytes_array) {
		exs_buf_free(&cb->recv_buf.buf);
		return -1;
	}
	cb->recv_buf.next_entry_primed = cb->recv_buf.next_entry
		= cb->recv_buf.full_bytes_array;
	cb->recv_buf.full_bytes_array_end
		= cb->recv_buf.full_bytes_array
		+ cb->stream_buf_size / our_cache_line_size;
	cb->recv_buf.n_empty_threshold
		= cb->stream_buf_size / 3;
	cb->n_unacked_wwi_threshold
		= 1;
	TRACE_DBG("local recv buf is %p rkey %#.8" PRIx32 " len %zu",
			cb->recv_buf.buf.ptr,
			cb->recv_buf.buf.mr->rkey,
			cb->recv_buf.buf.size);
	return 0;
}	/* our_create_stream_recv_buf */


/*	agent's recv credits become client's send credits
 *	agent's send credits become client's recv credits
 */
static inline void
our_setup_agent_nego_mess(struct exs_conn *cb, struct our_nego_mess *nego_send)
{
	nego_send->client_credits = htonl(cb->posted_credits);
	nego_send->sp_max_size = htonl(cb->small_packet_max_size);
	nego_send->version = EXS_VERSION;
	nego_send->subversion = EXS_SUBVERSION;
	nego_send->subsubversion = EXS_SUBSUBVERSION;
	nego_send->is_seqpacket = cb->is_seqpacket;
	nego_send->fd_flags = cb->fd_flags & ~EXS_FD_NODELAY;
	if (!cb->is_seqpacket && cb->stream_buf_size > 0) {
		nego_send->recv_buf_addr
			= htonll((uint64_t)cb->recv_buf.buf.ptr);
		nego_send->recv_buf_rkey = htonl(cb->recv_buf.buf.mr->rkey);
		nego_send->recv_buf_len = htonl(cb->recv_buf.buf.size);
		if (cb->fd_flags & EXS_FD_STREAM_BUF_NOALIGN) {
			nego_send->recv_buf_align = htonl(0);
		} else {
			nego_send->recv_buf_align = htonl(our_cache_line_size);
		}
	}
}	/* our_setup_agent_nego_mess */


/*
 * called only if we get canceled in our_completion_thread()
 * after we have received an event which we need to acknowledge
 */
static void
our_con_man_cancel_handler_event(void *arg)
{
	struct con_man_cancel_params	*ptr;

	ptr = arg;
	TRACE_DBG("[%lu] %s leave",
		pthread_self(), "our_con_man_thread");

	/* acknowledge the event */
	our_ack_cm_event(ptr->event);

	OUR_MEMBER_MUTEX_UNLOCK(ptr->cb, event_mutex);
	ptr->cb = our_socket_unref(ptr->cb);
}	/* our_con_man_cancel_handler_event */


/*
 * called only if we get canceled in our_completion_thread()
 * when waiting to get a CM event
 */
static void
our_con_man_cancel_handler_wait(void *arg)
{
	struct exs_conn	*cb = arg;

	TRACE_DBG("[%lu] %s leave",
		pthread_self(), "our_con_man_thread");
	cb = our_socket_unref(cb);
}	/* our_con_man_cancel_handler_wait */


/*
 *	our_con_man_thread - private, handles rdma connection manager events.
 *
 *	repeatedly waits for a connection manager event,
 *	then	locks the event_mutex
 *		dispatches the event
 *		unlocks the event_mutex
 *
 *	The arg passed in at creation is a pointer to the cb of
 *	either a client connection, a listening connection, or
 *	an agent connection.
 *
 */
static void *
our_con_man_thread(void *arg)
{
	int				ret;
	struct exs_conn			*cb = arg;
	struct rdma_cm_event		*event;
	volatile enum rdma_cm_event_type event_type;
	pthread_cond_t			*to_broadcast;
	struct con_man_cancel_params	params;
	TRACE_VAR_DECLARE(volatile enum rdma_cm_event_type, prev_event_type);

	TRACE_DBG("enter for cb %p", (void *)cb);

	/* tell our creator that this con_man_thread has started */
	OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);

	cb->con_man_thread_is_active = 0xff;
	pthread_cond_signal(&cb->transmit_ok_notify);

	OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);

	event_type = RDMA_CM_EVENT_MULTICAST_ERROR;
	for( ; ; )
	{
		/* wait for next connection management event on this channel */
		TRACE_DBG("about to get cm event for cb %p", (void *)cb);

		/* for debugging, remember last event type */
		TRACE_VAR_ASSIGN(prev_event_type, event_type);
		pthread_cleanup_push(our_con_man_cancel_handler_wait, cb);
		pthread_testcancel();
		ret = rdma_get_cm_event(cb->event_channel, &event);
		pthread_testcancel();
		pthread_cleanup_pop(0);

		if (ret) {
			ret = errno;
			TRACE_ERR("rdma_get_cm_event failed on cb %p, error %d",
				(void *)cb, ret);
			errno = ret;
			cb = our_socket_unref(cb);
			pthread_exit(NULL);
		}

		event_type = event->event;
		TRACE_EVENT_TYPE(cb, __func__, "got cm event type", event_type);
		TRACE_EVENT_TYPE(cb, __func__, "prev cm event type",
							prev_event_type);

		/* push a cancel handler that unlocks event_mutex
		 *	and acknowledges the event
		 */
		params.event = event;
		params.cb = cb;
		pthread_cleanup_push(our_con_man_cancel_handler_event, &params);

		OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);

		to_broadcast = NULL;
		switch(event_type)
		{
		case RDMA_CM_EVENT_CONNECT_REQUEST:
			/* executed by listening connection only */
			if (our_merge_pending_accept(cb, event) != 0) {
				/* reject the connection an any error */
			}
			break;

		case RDMA_CM_EVENT_ESTABLISHED:
			/* executed by both client and agent connections */
			if(cb->conn_state == EXS_CONN_ACCEPTING)
			{	/* this is an agent connection */
				if(our_accept_established(cb) == 0) {
					/* everything connected ok, allow the
					 * completion thread to get started */
					 to_broadcast = &cb->transmit_ok_notify;
				}
				/* TO DO -- deal with error result */
			}
			else if(cb->conn_state == EXS_CONN_CONNECTING)
			{	/* this is a client connection */
				if((ret = our_connect_established(cb, event))
									== 0) {
					/* everything connected ok, allow the
					 * completion thread to get started */
					 to_broadcast = &cb->transmit_ok_notify;
				} else {
					our_connect_returning(cb, 5, ret);
				}
				/* TO DO -- deal with error result */
			} else {/* this connection is in an unexpected state */
				our_deal_with_bad_event(cb, event);
			}
			break;

		case RDMA_CM_EVENT_ADDR_RESOLVED:
			if(cb->conn_state == EXS_CONN_ADDRESSING)
			{	/* this is a client connection */
				our_connect_addr_resolved(cb);
				/* TO DO -- deal with result */
			} else {/* this connection is in an unexpected state */
				our_deal_with_bad_event(cb, event);
			}
			break;

		case RDMA_CM_EVENT_ROUTE_RESOLVED:
			if(cb->conn_state == EXS_CONN_ROUTING)
			{	/* this is a client connection */
				our_connect_route_resolved(cb);
			} else {/* this connection is in an unexpected state */
				our_deal_with_bad_event(cb, event);
			}
			break;

		case RDMA_CM_EVENT_DISCONNECTED:
			/* executed by both client and agent connections */
			TRACE_DBG("RDMA_CM_EVENT_DISCONNECTED, connection %p in state %s conn_flags %#x",
					(void *)cb,
					our_conn_state_str(cb->conn_state),
					cb->conn_flags);

			if(cb->conn_state == EXS_CONN_CLOSING
							|| cb->conn_state
						== EXS_CONN_DELAY_CLOSING) {
				/* a previous exs_close() finishes
				 * asynchronously here
				 * on local or remote disconnect
				 * make sure no one finds this conn again!
				 */
				cb->conn_state = EXS_CONN_FINISHED;
			} else {
				/* this connection was hit by disconnect
				 * from remote side before it issued
				 * local exs_close()
				 */

#if defined(ALLOW_PARANOID_CHECKING)
				if(cb->conn_state != EXS_CONN_RUNNING) {
					/* should never happen */
					TRACE_ERR("unexpected RDMA_CM_EVENT_DISCONNECTED!");
				}
#endif

				cb->conn_state = EXS_CONN_TERMINATING;
			}

			/* for now, turn on debugging */
			/*****_our_debug_flag = 1; *****/

			/* perform a local rdma_disconnect() to provoke
			 * FLUSH_ERR status on all local posted recv blocks
			 * (completion thread will handle these)
			 * This must be done even in CLOSING states since
			 * exs_close() sends EOF and waits for reply before
			 * calling rdma_disconnect() itself, but we still need
			 * to wake up the completion thread (via the FLUSH_ERR)
			 * if the connection is broken during this time.
			 */
			if (our_clear_conn_flags(cb, EXS_NEED_DISCONNECT
							| EXS_OK_TO_ISSUE_IO)
							& EXS_NEED_DISCONNECT) {
				ret = our_rdma_disconnect(cb);
				TRACE_DBG("back from our_rdma_disconnect, ret %d, errno %d, conn_state %s, conn_flags %06x",
					ret, errno,
					our_conn_state_str(cb->conn_state),
					cb->conn_flags);
			}
			break;

		case RDMA_CM_EVENT_TIMEWAIT_EXIT:
			/* the QP associated with a connection has exited its
			 * timewait state and is now ready to be re-used.
			 * After a QP has been disconnected, it is maintained
			 * in a timewait state to allow any in flight packets
			 * to exit the network.
			 * After the timewait state has completed, the
			 * rdma_cm will report this event.
			 *
			 * My observations:
			 * 1. This only happens with IB interfaces, not iWARP,
			 *    and only for server agent connections.
			 * 2. This happens a few seconds after a server agent
			 *    is terminated
			 * 3. The event id appears to be that of the server
			 *    agent -- it is NOT that of the listener
			 */
			TRACE_DBG("RDMA_CM_EVENT_TIMEWAIT_EXIT for cb %p, conn_state %s, conn_flags %06x, cm_id %p, event id %p",
				(void *)cb,
				our_conn_state_str(cb->conn_state),
				cb->conn_flags,
				(void *)cb->cm_id, (void *)event->id);
			break;
		case RDMA_CM_EVENT_ADDR_ERROR:
		case RDMA_CM_EVENT_ROUTE_ERROR:
		case RDMA_CM_EVENT_CONNECT_ERROR:
		case RDMA_CM_EVENT_CONNECT_RESPONSE:
		case RDMA_CM_EVENT_UNREACHABLE:
		case RDMA_CM_EVENT_REJECTED:
		case RDMA_CM_EVENT_DEVICE_REMOVAL:
		case RDMA_CM_EVENT_MULTICAST_JOIN:
		case RDMA_CM_EVENT_MULTICAST_ERROR:
		case RDMA_CM_EVENT_ADDR_CHANGE:
			our_deal_with_bad_event(cb, event);
			break;
		}	/* switch */

		TRACE_EVENT_TYPE(cb, __func__, "acknowledging cm event type",
								event_type);
		ret = our_ack_cm_event(event);
		if(ret)
		{
			TRACE_ERR("our_ack_cm_event returned %d, event_type %d",
				ret, event_type);
			/* now what ?? */
		}

		OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);

		/* pop the cancellation handler */
		pthread_cleanup_pop(0);		/* i.e., do NOT do the unlock */

		if(to_broadcast != NULL)
		{
			pthread_cond_broadcast(to_broadcast);
		}
		if(cb->conn_state == EXS_CONN_TERMINATING
					|| cb->conn_state == EXS_CONN_FINISHED
					|| cb->conn_state == EXS_CONN_CLOSED)
		{	/* self-terminate this con_man_thread */
			TRACE_DBG("leave conn_state %s",
					our_conn_state_str(cb->conn_state));
			cb = our_socket_unref(cb);
			pthread_exit(NULL);
			/* con-man thread's flow never goes beyond this */
		}
	}	/* for ever */
}	/* our_con_man_thread */


/*
 * called only from exs_socket() and our_continue_merge_pending_accept()
 */
static int
create_our_con_man_thread(struct exs_conn *cb)
{
	sigset_t newsigset, oldsigset;
	int	ret;

	/* Temporarily block all signals in this thread while we create the new
	 * thread, so that it inherits our full set of blocked signals.  Then
	 * restore the caller's original blocked signal set. */
	sigfillset(&newsigset);
	pthread_sigmask(SIG_BLOCK, &newsigset, &oldsigset);
	ret = pthread_create(&cb->event_thread, NULL, our_con_man_thread,
							our_socket_ref(cb));
	pthread_sigmask(SIG_SETMASK, &oldsigset, NULL);
	if(ret)
	{
		cb = our_socket_unref(cb);
		OUR_PERROR(ret, "pthread_create of event thread failed");
		return ret;

	}
	/* FIXME: pthread_t != unsigned long */
	TRACE_DBG("CREATE event_thread %lu on cb %p",
			cb->event_thread, (void *)cb);

	ret = pthread_detach(cb->event_thread);
	if(ret)
	{
		OUR_PERROR(ret, "pthread_detach of event thread failed");
		/* ignore this error */
		ret = 0;
	}

	if (cb->con_man_thread_is_active == 0) {
		/* con_man_thread not up yet, wait for it to get started */
		OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);

		while (cb->con_man_thread_is_active == 0) {
			pthread_cond_wait(&cb->transmit_ok_notify,
							&cb->event_mutex);
		}

		OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
	}

	return ret;
}	/* create_our_con_man_thread */


/*
 * called from our_start_close() when exs_socket() on this
 * conn has completed ok and
 * either it has only progressed through bind before the close
 * or it is a listener, not a client or an agent
 * also called from our_continue_merge_pending_accept() on error
 *
 * when called, caller has locked event_mutex
 */
static void
our_terminate_con_man_thread(struct exs_conn *cb)
{
	int	ret;

	TRACE_DBG("CANCELing our_con_man_thread %lu on cb %p",
		cb->event_thread, (void *)cb);

	assert(cb->conn_state == EXS_CONN_UNBOUND
				|| cb->conn_state == EXS_CONN_BOUND
				|| cb->conn_state == EXS_CONN_UNLISTENING
				|| cb->conn_state == EXS_CONN_ADDRESSING
				|| cb->conn_state == EXS_CONN_ROUTING
				|| cb->conn_state == EXS_CONN_CONNECTING
				|| cb->conn_state == EXS_CONN_ACCEPTING);
	ret = pthread_cancel(cb->event_thread);
	if(ret)
	{
		OUR_PERROR(ret, "pthread_cancel of our_con_man_thread failed");
		/* ignore this error */
	} else {
		TRACE_DBG("CANCELed  our_con_man_thread %lu on cb %p",
			cb->event_thread, (void *)cb);
		cb->event_thread = 0;
	}

	sched_yield();
}	/* our_terminate_con_man_thread */


/* global statistics counters */
#if defined(ALLOW_EXS_STATISTICS)

static char* statistics_counter_labels[max_statistics_counter_type] = {
	"total completion loops",
	"total ibv poll cqs",
	"empty ibv poll cqs",
	"total ibv get cq events",
	"total completions",
	"max ibv_poll_cq completions",
	"send ack completions",
	"send advert completions",
	"send rdma_write completions",
	"recv advert completions",
	"recv rdma_with_imm completions",
	"recv rdma_with_imm bytes",
	"recv eof completions",
	"setup completions",
	"matched recv await",
	"unmatched recv await",
	"send done after sending",
	"send done after WWI completion",
	"total setup sends",
	"advert blocks sent",
	"signaled inline advert blocks sent",
	"unsignaled inline advert blocks sent",
	"signaled inline rdma_write blocks sent",
	"unsignaled inline rdma_write blocks sent",
	"rdma_write blocks sent",
	"rdma_write bytes sent",
	"ack_blocks sent",
	"recv ack completions",
	"recv ack buf completions",
	"eof_blocks sent",
	"no tblock on send ACK",
	"extra advert_blocks",
	"direct rdma_write blocks sent",
	"indirect rdma_write blocks sent",
	"no credits on send WWI",
	"max used wwi_credits",
	"no credits on send ACK",
	"max used ack_credits",
	"no credits on send ADVERT",
	"max used advert_credits",
	"direct indirect switches",
	"ibv_reg_mr calls",
	"ibv_dereg_mr calls",
	"max recv_buf n_full",
	"max remote_recv_buf n_full",
	"recv_buf full on send",
	"recv_buf empty on recv",
};

/* NOTE:	advertisements are always inline (if the CA supports it)
		rdma_writes can be inline or not

		advertisement blocks can be signaled or unsignaled
		rdma_writes can be signaled or unsignaled
 */

#endif


/*
 *	exs_reset_counters() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	resets current value of statistics counters
 */
EXS_EXPORT void
exs_reset_counters(void)
{
#if defined(ALLOW_EXS_STATISTICS)
	enum statistics_counter_type	i;

	TRACE_DBG("exs_reset_counters");
	for (i = completion_loops; i < max_statistics_counter_type; i++) {
		statistics_counters[i] = 0;
	}
#endif
}	/* exs_reset_counters */


/*
 *	exs_print_counters() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	prints current value of statistics counters
 */
EXS_EXPORT void
exs_print_counters(FILE *fptr)
{
	(void)fptr;
#if defined(ALLOW_EXS_STATISTICS)
	enum statistics_counter_type	i;

	for (i = completion_loops; i < max_statistics_counter_type; i++) {
		fprintf(fptr, "%2d %20llu %s\n", i,
			(long long unsigned)statistics_counters[i],
			statistics_counter_labels[i]);
	}
	fflush(fptr);
#endif

}	/* exs_print_counters */


/*
 *	exs_dump_counters() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	prints current value of statistics counters
 */
EXS_EXPORT void
exs_dump_counters(void)
{
#if defined(ALLOW_EXS_STATISTICS)
	exs_print_counters(stderr);
#endif
}	/* exs_dump_counters */


/*
 *	exs_print_calloc_report() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	prints out any remaining unfreed heap allocations
 */
EXS_EXPORT void
exs_print_calloc_report(FILE *fptr)
{
	(void)fptr;
	EXS_CALLOC_REPORT(fptr);
}	/* exs_print_calloc_report */


/* holds the pid of the last process that successfully called exs_init */
static pid_t global_owner = 0;

/* Return true if and only if the current PID (or any PID) called
 * exs_init(). */
#if defined(ALLOW_PID_CHECKING)
static inline int
our_check_init(void)
{
	return global_owner == getpid();
}	/* our_check_init */
#else
static inline int
our_check_init(void)
{
	return global_owner != 0;
}	/* our_check_init */
#endif


/* Query the cache line size if possible.  We start by querying the level 3
 * cache, as this is the one most likely to be accessed by the HCA.  This
 * function always succeeds; it returns a worst-case value if the underlying
 * query fails for some reason. */
static inline unsigned int
our_query_cache_line_size(void)
{
	int ret;
#if defined(_SC_LEVEL3_CACHE_LINESIZE)
	if ((ret = sysconf(_SC_LEVEL3_CACHE_LINESIZE)) > 0) {
		return ret;
	}
#endif
#if defined(_SC_LEVEL2_CACHE_LINESIZE)
	if ((ret = sysconf(_SC_LEVEL2_CACHE_LINESIZE)) > 0) {
		return ret;
	}
#endif
#if defined(_SC_LEVEL1_DCACHE_LINESIZE)
	if ((ret = sysconf(_SC_LEVEL1_DCACHE_LINESIZE)) > 0) {
		return ret;
	}
#endif
	(void)ret;
	return OUR_CACHE_LINE_SIZE_GUESS;
}


static int
our_ibv_init(void)
{
	int			num_devices, count, ret;
	struct ibv_context	**list, **lptr;
	TRACE_VAR_DECLARE(char *, tptr);

	num_devices = 0;
	list = rdma_get_devices(&num_devices);
	if(list == NULL || num_devices == 0)
	{
		TRACE_ERR("No RDMA devices found");
		errno = ENXIO;		/* No such device or address */
		return -1;
	}

	TRACE_DBG("found %d RDMA devices", num_devices);

	our_init_pd_table();

	for(count = 0, lptr = list; *lptr != NULL; lptr++, count++)
	{
		switch ((*lptr)->device->transport_type)
		{
		case IBV_TRANSPORT_IB:
			TRACE_VAR_ASSIGN(tptr, "InfiniBand");
			break;
		case IBV_TRANSPORT_IWARP:
			TRACE_VAR_ASSIGN(tptr, "iWARP");
			break;
		default:
			TRACE_VAR_ASSIGN(tptr, "Unknown transport type");
			break;
		}	/* switch */
		TRACE_DBG("%d %s, %s",
			count, (*lptr)->device->name, tptr);
		if (our_get_pd(*lptr) == NULL) {
			ret = errno;
			our_destroy_pd_table();
			errno = ret;
			ret = -1;
			goto errout;
		}
	}	/* for */
	ret = 0;

errout:
	rdma_free_devices(list);
	return ret;
}	/* our_ibv_init */


/*
 *	exs_init - page 21 of Interconnect Software Consortium Issue 1.0
 *		extended socket API initialization
 *		Must be called once per process
 *		prior to calling any other exs_xxx function.
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_init(int version)
{
	pid_t current;
	int ret;

	ret = 0;
	exs_trace_init();

	current = getpid();
	TRACE_DBG("pid %d start version %d", current, version);

	if(version != EXS_VERSION)
	{
		errno = ENOTSUP;
		return -1;
	}

	if(global_owner == current)
	{
		/* already called once by this process */
		errno = EALREADY;	/* Operation already in progress */
		return -1;
	}

	TRACE_INFO("UNH-EXS version %s compiled on %s at %s",
		our_version, __DATE__, __TIME__);

	our_init_socket_table();
	ret = our_ibv_init();
	if (ret < 0) {
		goto errout;
	}

	/* Query the system to get the cache line size.  This function will
	 * always succeed; it will return a worst-case guess if it querying for
	 * the cache line size fails or is unsupported. */
	our_cache_line_size = our_query_cache_line_size();

	/* Mark that exs_init has been called successfully by this process.
	 * This allows all other exs_xxx functions to be called by this process.
	 */
	global_owner = current;

errout:
	TRACE_DBG("pid %d: finished, ret %d", current, ret);
	sched_yield();
	return ret;
}	/* exs_init */


struct exs_mhandle
{
	void			*addr;		/* address registered by user*/
	size_t			size;		/* size registered by user */
	enum ibv_access_flags	access_flags;
	pthread_rwlock_t	mr_table_rwlock;
	struct ibv_mr		*recent_mr;
	struct ibv_mr		*mr_table[MAX_PD_SLOTS];
};	/* struct exs_mhandle */


static inline struct ibv_mr *
our_ensure_buffer_registered(struct exs_conn *this_conn, exs_mhandle_t mhandle,
				const char *message)
{
	struct ibv_mr	*mr, **mr_ptr;

	TRACE_DBG("mhandle %p should be registered, mhandle->addr %p",
		(void *)mhandle, mhandle->addr);

	/* see if most recently used mr for this mhandle fits this conn */
	if ((mr = mhandle->recent_mr) != NULL && mr->pd == this_conn->pd) {
		return mr;
	}

	/* most recent use of this mhandle does not fit this conn's pd
	 * search the mr_table to find one that does
	 */

	OUR_MEMBER_RWLOCK_RDLOCK(mhandle, mr_table_rwlock);

	for (mr_ptr = &mhandle->mr_table[0]; *mr_ptr != NULL; mr_ptr++) {
		if ((*mr_ptr)->pd == this_conn->pd)
			break;
	}

	OUR_MEMBER_RWLOCK_UNLOCK(mhandle, mr_table_rwlock);

	if (*mr_ptr != NULL) {
		/* we found a fit, cache it as most recently used mr */
		mhandle->recent_mr = *mr_ptr;
		goto fit_ok;
	}

	/* this should rarely happen, but sometime after this mhandle was
	 * created, a new device must have come on-line, so we now need to add
	 * a registration for that device to this (old) mhandle's mr_table
	 */

	OUR_MEMBER_RWLOCK_WRLOCK(mhandle, mr_table_rwlock);

	/* make sure it is not now in the mhandle's mr_table */
	for (mr_ptr = &mhandle->mr_table[0]; *mr_ptr != NULL; mr_ptr++) {
		if ((*mr_ptr)->pd == this_conn->pd)
			break;
	}

	if (*mr_ptr != NULL) {
		/* we found a fit, cache it as most recently used mr */
		mhandle->recent_mr = *mr_ptr;
	} else if (mr_ptr >= &mhandle->mr_table[MAX_PD_SLOTS-1]) {
			/* no room in this mr_table for another mr */
			TRACE_ERR("mhandle %p mr_table full, max of %d pd-specific registrations",
				(void *)mhandle, MAX_PD_SLOTS - 1);
			errno = ENOMEM;
	} else {
		/* bad news -- there is no valid mr_table entry for conn's pd
		 * try to get one (creating it if necessary) from global table
		 */
		if (this_conn->pd == our_get_pd(this_conn->pd->context)) {
			/* ok, the global pd is set up */
			*mr_ptr = our_call_ibv_reg_mr(this_conn->pd,
						mhandle->addr, mhandle->size,
						mhandle->access_flags, message);
			if (*mr_ptr != NULL) {
				/* we registered it too, so cache it as
				 * most recently used mr
				 */
				mhandle->recent_mr = *mr_ptr;
			} else {
				errno = ENXIO;	/* No such device or address */
			}
		} else {
			TRACE_ERR("mhandle %p has no matching mr_table for conn %p pd %p",
				(void *)mhandle, (void *)this_conn,
				(void *)this_conn->pd);
			errno = ENXIO;		/* No such device or address */
		}
	}

	OUR_MEMBER_RWLOCK_UNLOCK(mhandle, mr_table_rwlock);

fit_ok:
	TRACE_DBG("returning mr %p", (void *)*mr_ptr);
	return *mr_ptr;
}	/* our_ensure_buffer_registered */


/* Converts deprecated EXS access flags to those expected by ibv_reg_mr().  This
 * function ignores flags that are not part of EXS_ACCESS_ALL and always
 * succeeds. */
static int
our_get_ibv_reg_mr_access(int flags)
{
	int access = 0;
	if (flags & EXS_LOCAL_WRITE)
		access |= IBV_ACCESS_LOCAL_WRITE;
	if (flags & EXS_REMOTE_READ)
		access |= IBV_ACCESS_REMOTE_READ;
	if (flags & EXS_REMOTE_WRITE)
		access |= IBV_ACCESS_LOCAL_WRITE | IBV_ACCESS_REMOTE_WRITE;
	return access;
}	/* our_get_ibv_reg_mr_access */


static exs_mhandle_t
our_mregister(void *addr, size_t size, int flags)
{
	exs_mhandle_t	mhandle;
	struct ibv_pd	**pd_ptr;
	struct ibv_mr	**mr_ptr;

	/***** NEED to calloc storage for mhandle to point to *****/
	if((mhandle = EXS_CALLOC(1, sizeof(*mhandle))) == NULL)
	{
		return EXS_MHANDLE_INVALID;
	}

	mhandle->addr = addr;
	mhandle->size = size;

	if (flags & EXS_ACCESS_ALL) {
		mhandle->access_flags = our_get_ibv_reg_mr_access(flags);
	} else if (!(flags & EXS_MRF_RECV_DISABLE)) {
		/* LOCAL_WRITE and REMOTE_WRITE are needed for a buffer to be
		 * the target of exs_recv() */
		mhandle->access_flags = IBV_ACCESS_LOCAL_WRITE
					| IBV_ACCESS_REMOTE_WRITE;
	}

	pthread_rwlock_init(&mhandle->mr_table_rwlock, NULL);

	/* fill the mr_table by registering this region with all known pds */

	OUR_MEMBER_RWLOCK_RDLOCK(mhandle, mr_table_rwlock);

	for (pd_ptr = &pd_table[0], mr_ptr = &mhandle->mr_table[0];
					*pd_ptr != NULL; pd_ptr++, mr_ptr++) {
		*mr_ptr = our_call_ibv_reg_mr(*pd_ptr, addr, size,
					mhandle->access_flags, "user buffer");
		if (*mr_ptr == NULL) {
			break;
		}
	}

	/* set the recent_mr to the first item in mr_table */
	if ((mhandle->recent_mr = mhandle->mr_table[0]) == NULL) {
		/* must always have a recent_mr */
		pthread_rwlock_destroy(&mhandle->mr_table_rwlock);
		EXS_FREE(mhandle);
		errno = ENXIO;		/* No such device or address */
		mhandle = EXS_MHANDLE_INVALID;
		goto errout;
	}

	OUR_MEMBER_RWLOCK_UNLOCK(mhandle, mr_table_rwlock);

errout:
	return mhandle;
}	/* our_mregister */


/*
 *	exs_mregister - page 25 of Interconnect Software Consortium Issue 1.0
 *		register application memory
 *
 *	returns handle if all ok,
 *		else EXS_MHANDLE_INVALID with errno set to error value
 */
EXS_EXPORT exs_mhandle_t
exs_mregister(void *addr, size_t size, int flags)
{
	exs_mhandle_t		ret;

	TRACE_DBG("enter addr %p, size %zu, flags 0x%x",
			(void *)addr, size, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return EXS_MHANDLE_INVALID;
	}

	if (size <= 0 || flags > EXS_ALL_FLAGS || addr == NULL
			|| ((flags & EXS_ACCESS_ALL)
				&& (flags & EXS_MRF_RECV_DISABLE))) {
		errno = EINVAL;		/* Invalid argument */
		return EXS_MHANDLE_INVALID;
	}

	if (flags & EXS_ACCESS_ALL) {
		TRACE_INFO("Use of deprecated flag(s) in %#x; please read exs_mregister(3)",
				flags);
	}

	ret = our_mregister(addr, size, flags);

	TRACE_DBG("exit returning %p, flags 0x%x", (void *)ret, flags);

	return ret;
}	/* exs_mregister */


/*
 *	exs_mmodify - page 23 of Interconnect Software Consortium Issue 1.0
 *		modify registered application memory
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_mmodify(exs_mhandle_t mhandle, size_t size, int flags)
{

	TRACE_DBG("enter mhandle %p, size %zu, flags 0x%x",
						(void *)mhandle, size, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if(size <= 0
		|| flags > EXS_ALL_FLAGS
		|| mhandle == EXS_MHANDLE_INVALID
		|| mhandle == EXS_MHANDLE_UNREGISTERED
		|| mhandle == NULL)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	errno = ENOSYS;
	return -1;
}	/* exs_mmodify */


static void
our_mderegister(exs_mhandle_t mhandle)
{
	struct ibv_mr	**mr_ptr;

	/* unregister all items in the mr_table
	 * here we assume that the mr_table array ends with a NULL entry */
	for (mr_ptr = &mhandle->mr_table[0]; *mr_ptr != NULL; mr_ptr++) {
		STATISTICS_INCREMENT(ibv_dereg_mr_calls);
		ibv_dereg_mr(*mr_ptr);
	}

	mhandle->recent_mr = NULL;

#if defined(ALLOW_EXS_TRACE)
	memset(mhandle, 0, sizeof(*mhandle));	/* for debugging only */
#endif

	pthread_rwlock_destroy(&mhandle->mr_table_rwlock);

	EXS_FREE(mhandle);
}	/* our_mderegister */


/*
 *	exs_mderegister - page 22 of Interconnect Software Consortium Issue 1.0
 *		deregister application memory
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_mderegister(exs_mhandle_t mhandle, int flags)
{

	TRACE_DBG("enter mhandle %p flags 0x%x", (void *)mhandle, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* Flags are unused in Issue 1.0 of the specification */
	if (flags) {
		errno = EINVAL;
		return -1;
	}

	if(mhandle == EXS_MHANDLE_INVALID ||
	   mhandle == EXS_MHANDLE_UNREGISTERED ||
	   mhandle == NULL)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}
	our_mderegister(mhandle);

	TRACE_DBG("exit success");
	return 0;
}	/* exs_mderegister */


/*
 *	exs_qcreate - page 33 of Interconnect Software Consortium Issue 1.0
 *		create an event queue
 *
 *	returns handle if all ok,
 *		else EXS_QHANDLE_INVALID with errno set to error value
 */
EXS_EXPORT exs_qhandle_t
exs_qcreate(int depth)
{
	struct evt_queue *queue;

	TRACE_DBG("enter depth %d", depth);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return EXS_QHANDLE_INVALID;
	}

	if((queue = _evt_queue_init(depth)) == NULL)
	{	/* EXS standard requires this return value */
		return EXS_QHANDLE_INVALID;
	}

	TRACE_DBG("exit success returning queue %p", (void *)queue);

	return _cnv_ptr_to_qnumb(queue);
}	/* exs_qcreate */


/*
 *	exs_qdelete - page 35 of Interconnect Software Consortium Issue 1.0
 *		delete an event queue
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_qdelete(exs_qhandle_t qhandle)
{
	TRACE_DBG("enter qhandle %p", (void *)qhandle);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	return _evt_queue_deinit(qhandle);	/* errno is set */
}	/* exs_qdelete */


/*
 *	exs_qmodify - page 38 of Interconnect Software Consortium Issue 1.0
 *		modify event queue attributes
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_qmodify(exs_qhandle_t qhandle, int attr_type, void *attr_value,
		 size_t attr_length)
{
	struct evt_queue 	*qptr;
	struct exs_signal	*signal;

	TRACE_DBG("enter qhandle %p", (void *)qhandle);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if((qptr = _cnv_qnumb_to_ptr(qhandle, 1, 0)) == NULL)
	{	/* Invalid argument */
		return -1;
	}

	switch (attr_type) {
	case EXS_QATTR_SIGNAL:
		if(attr_length != sizeof(struct exs_signal)) {
			break;
		}
		signal = (struct exs_signal *)attr_value;
		switch (signal->exs_sigstate) {
		case EXS_SIG_DISABLE:
			return _evt_queue_disable_signal(qptr);
		case EXS_SIG_ENABLE:
			return _evt_queue_enable_signal(qptr,
							signal->exs_signo);
		default:
			break;
		}
	case EXS_QATTR_DEPTH:
		/* currently not supported */
		break;
	case EXS_QATTR_WAIT:
		if (attr_length != sizeof(int)) {
			break;
		}
		return _set_evt_queue_wait(qptr, (int *)attr_value);
	default:
		break;
	}

	TRACE_DBG("exit failure");
	errno = EINVAL;
	qptr = _evt_queue_unref(qptr, 1, 0);
	return -1;
}	/* exs_qmodify */


/*
 *	exs_qdequeue - page 36 of Interconnect Software Consortium Issue 1.0
 *		retrieve events from an event queue
 *
 *	returns number of events dequeued >= 0 if all ok,
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_qdequeue(exs_qhandle_t qhandle, exs_event_t *evtvec, int evtvec_cnt,
		const struct timeval *timeout)
{
	struct evt_queue	*qptr;
	int			ndequeued;

	TRACE_DBG("enter qhandle %p, evtvec_cnt %d",
						(void *)qhandle, evtvec_cnt);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if(evtvec_cnt < 1) {
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	if((qptr = _cnv_qnumb_to_ptr(qhandle, 1, 0)) == NULL)
	{	/* Invalid argument */
		return -1;
	}
	ndequeued = _evt_queue_dequeue_vector(qptr,evtvec,evtvec_cnt,timeout);
	qptr = _evt_queue_unref(qptr, 1, 0);

	TRACE_DBG("exit success qhandle %p, returning %d, evtvec_cnt %d",
			(void *)qhandle, ndequeued, evtvec_cnt);

	return ndequeued;
}	/* exs_qdequeue */


/*
 *	exs_qstatus - page 40 of Interconnect Software Consortium Issue 1.0
 *		retrieve event queue attributes
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_qstatus(exs_qhandle_t qhandle, int attr_type, void *attr_value,
		size_t attr_length)
{
	struct evt_queue 	*qptr;

	TRACE_DBG("enter qhandle %p", (void *)qhandle);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if((qptr = _cnv_qnumb_to_ptr(qhandle, 1, 0)) == NULL) {
		/* Invalid argument */
		return -1;
	}

	switch (attr_type) {
	case EXS_QATTR_DEPTH:
		if(attr_length != sizeof(int)) {
			break;
		}
		return _get_evt_queue_depth(qptr, (int*)attr_value);
	case EXS_QATTR_SIGNAL:
		if(attr_length != sizeof(struct exs_signal)) {
			break;
		}
		return _get_evt_queue_signal(qptr, (exs_signal_t *)attr_value);
	case EXS_QATTR_EVENTS:
		if(attr_length != sizeof(int)) {
			break;
		}
		return _get_evt_queue_n_events(qptr, (int*)attr_value);
	case EXS_QATTR_WAIT:
		if (attr_length != sizeof(int)) {
			break;
		}
		_get_evt_queue_wait(qptr, (int*)attr_value);
		return 0;
	default:
		break;
	}

	TRACE_DBG("exit failure");
	errno = EINVAL;
	qptr = _evt_queue_unref(qptr, 1, 0);
	return -1;
}	/* exs_qstatus */


/*
 *	exs_cancel - page 15 of Interconnect Software Consortium Issue 1.0
 *		cancel pending asynchronous operations
 *
 *	returns 0 if all ok, else -1 with errno set to error value
 */
EXS_EXPORT int
exs_cancel(int flags, int fildes, exs_ahandle_t ahandle)
{
	struct exs_conn		*this_conn;

	TRACE_DBG("enter");

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		goto out;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(fildes)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		goto out;
	}

	errno = ENOSYS;		/* Function not implemented */
	(void)flags;
	(void)ahandle;
	this_conn = our_socket_unref(this_conn);
out:
	return -1;
}	/* exs_cancel */



/* called from:
 *   1.	our_exs_recv_done - to pass a user's recv buffer back to the user
 *   2. our_rest_of_exs_recv - to pass an in-place system recv buffer to user
 *
 * queue up this block's completion event for caller of exs_recv or exs_send
 * by removing it from this block and then adding it to tail of this block's
 * qhandle's full_list; then replacing it in this block with NULL
 *
 * this function posts the semaphore on the transaction block, so callers must
 * not dereference block after calling this function (other than to access
 * "constant" fields like the next pointer)
 */
static inline int
our_post_user_event(struct transaction_block *block, size_t amount)
{
	struct exs_event *evt;
	struct evt_queue *qptr;
	int	ret;

	/* first fill in actual number of bytes transfered
	 * NOTE -- this assignment assumes that exs_evt_length is at the same
	 * offset in ALL relevant overlays of the exs_evt_union!
	 * These overlays are:
	 *		exs_evt_xfer
	 *		exs_evt_xferfile
	 *		exs_evt_xfermsg		(so far unused)
	 */

	qptr = block->qhandle;
	evt = block->user_event_ptr;
	block->user_event_ptr = NULL;
	evt->exs_evt_union.exs_evt_xfer.exs_evt_length = amount;

	TRACE_DBG("post block %p with event %p amount %zu flags %#x",
		(void *)block, (void *)evt, amount, block->flags);

	/* mark that this block has notified the user and that it is free for
	 * reuse */
	assert(!(block->flags & EXS_NOTIFY_ALREADY_DONE));
	block->flags |= EXS_NOTIFY_ALREADY_DONE;
	block->in_progress = 0;
	OUR_MEMBER_SEM_POST(block, sem);
	block = NULL;

	/* then make the moves */
	/* ensure block->user_event_ptr is NULL prior to event posting */
	memory_barrier();
	ret = _put_full_evt_queue_event(qptr, evt);

	return ret;
}	/* our_post_user_event */


/* called from:
 *   1.	our_completion_thread itself when RECV_RDMA_WITH_IMM completes;
 *   2. our_rest_of_exs_recv on EOF
 */
static int
our_exs_recv_done(struct exs_conn *this_conn,
		struct transaction_block *block)
{
	TRACE_DBG("this_conn %p, block %p, amount %zu",
		(void *)this_conn, (void *)block, block->recv_byte_len);

	if((block->flags & EXS_MR_DYNAMIC)
			&& block->data_mr) {
		TRACE_DBG("deregister data_mr addr %p, size %zu",
			(void *)block->data_mr->addr,
			block->data_mr->length);
		STATISTICS_INCREMENT(ibv_dereg_mr_calls);
		ibv_dereg_mr(block->data_mr);
		block->data_mr = NULL;
	} else if (block->flags & EXS_COPY_UNREG) {
		memcpy(block->user_buffer, block->recv_rdma_buf.ptr,
							block->recv_byte_len);
		block->flags &= ~EXS_COPY_UNREG;
	}

	assert(block->qhandle == NULL || block->user_event_ptr != NULL);
	if (block->flags & EXS_NOTIFY_ALREADY_DONE) {
		/* do nothing */
	} else if ((block->qhandle == NULL)
			|| ((block->flags & EXS_UNSIGNALED)
				&& block->user_event_ptr->exs_evt_errno == 0)) {
		/* the user didn't request a signal */
		block->flags |= EXS_NOTIFY_ALREADY_DONE;
		assert(this_conn->pending_recv_count > 0);
		atomic_sub(1, &this_conn->pending_recv_count);
		if (block->qhandle) {
			/* Return the event that we took from the queue's empty
			 * list */
			_put_empty_evt_queue_event(block->qhandle,
							block->user_event_ptr);
			block->user_event_ptr = NULL;
			block->qhandle = _evt_queue_unref(block->qhandle,
						1, EVT_QUEUE_REF_PENDING);
		}

		/* since we're not posting an event, we must re-post the
		 * semaphore ourselves */
		block->in_progress = 0;
		OUR_MEMBER_SEM_POST(block, sem);

	} else {
		assert(this_conn->pending_recv_count > 0);
		atomic_sub(1, &this_conn->pending_recv_count);
		our_post_user_event(block, block->recv_byte_len);
		block = NULL;
	}

	return 0;
}	/* our_exs_recv_done */


/*
 * called only from our_all_done_with_block()
 *
 * when block is the last in the list for send/sendfile
 */
static inline int
our_last_block_sent(struct exs_conn *cb, struct transaction_block *block)
{
	unsigned int	new_send_count;
	int	ret;

	TRACE_DBG("our_last_block_sent block %p, event %p, qhandle %p, flags %#x",
		(void *)block, (void *)block->user_event_ptr,
		(void *)block->qhandle, block->flags);

	assert(block->qhandle == NULL || block->user_event_ptr != NULL);
	if(block->flags & EXS_NOTIFY_ALREADY_DONE) {
		/* user was already told about this event -- do nothing */
		return 0;
	} else if((block->qhandle == NULL)
			|| ((block->flags & EXS_UNSIGNALED)
				&& block->user_event_ptr->exs_evt_errno == 0)) {
		/* user doesn't want to know about this event */
		new_send_count = atomic_sub(1, &cb->pending_send_count);
		if (block->qhandle) {
			/* Return the event that we took from the queue's empty
			 * list */
			_put_empty_evt_queue_event(block->qhandle,
							block->user_event_ptr);
			block->user_event_ptr = NULL;
			block->qhandle = _evt_queue_unref(block->qhandle,
						1, EVT_QUEUE_REF_PENDING);
		}

		/* since we're not posting an event, we must re-post the
		 * semaphore ourselves */
		block->in_progress = 0;
		OUR_MEMBER_SEM_POST(block, sem);
		ret = 0;
	} else {
		/* user wants to know about this event, and was not previously
		 * told about it, post it now
		 */
		/* our_post_user_event fills in exs_evt_length */
		new_send_count = atomic_sub(1, &cb->pending_send_count);
		ret = our_post_user_event(block, block->send_total_bytes);
		block = NULL;
	}

	if (new_send_count == 0 && !(cb->conn_flags & EXS_OK_TO_ISSUE_SEND)) {
		/* We just decremented the send count and we are shutdown for
		 * sending; see if we need to send EOF to the remote
		 * endpoint */
		if (cb->conn_state == EXS_CONN_DELAY_CLOSING) {
			int send_flags = IBV_SEND_SIGNALED;
			OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);
			cb->conn_state = EXS_CONN_CLOSING;
			ret = our_send_eof(cb, &cb->eof_block_array[1],
						&send_flags, SHUT_RDWR);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
			OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
		} else if (cb->conn_state == EXS_CONN_RUNNING
				&& ((cb->conn_flags & (EXS_OK_TO_ISSUE_SEND
							| EXS_NEED_EOF_SEND))
						== EXS_NEED_EOF_SEND)) {
			int send_flags = IBV_SEND_SIGNALED;
			OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);
			if (cb->eof_block_array[0].in_progress
					&& htonl(cb->eof_block_array[0].eof_buf->flags)
								!= SHUT_RD) {
				ret = our_send_eof(cb,
						&cb->eof_block_array[0],
						&send_flags, SHUT_WR);
			} else if (cb->eof_block_array[1].in_progress
					&& htonl(cb->eof_block_array[1].eof_buf->flags)
								!= SHUT_RD) {
				ret = our_send_eof(cb,
						&cb->eof_block_array[1],
						&send_flags, SHUT_RDWR);
			}
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
			OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
		}
	}

	return ret;
}	/* our_last_block_sent */


/* the ibv_post_send was completed and
 * we got the completion, so block will be freed.
 */
static int
our_all_done_with_block(struct exs_conn *this_conn,
			struct transaction_block *block)
{
	int	ret;

	/* unregister source-side user buffer
	 * if we registered it just for this send only
	 */
	TRACE_DBG("block %p, event %p, flags %d, data_mr %p",
		(void *)block, (void *)block->user_event_ptr, block->flags,
		(void *)block->data_mr);
	if(block->flags & EXS_MR_DYNAMIC)
	{
		STATISTICS_INCREMENT(ibv_dereg_mr_calls);
		ret = ibv_dereg_mr(block->data_mr);
		if(ret)
		{
			OUR_PERROR(ret, "ibv_dereg_mr failed");
			/* now what ?? */
		}
		block->data_mr = NULL;
		block->flags &= ~EXS_MR_DYNAMIC;
	}

	/* sets qhandle because block will be freed */
	ret = our_last_block_sent(this_conn, block);
	TRACE_DBG("done with send transaction block %p", (void *)block);

	return ret;
}	/* our_all_done_with_block */


/*
 * When called to post send of advert or ACK,
 *	block->send_total_bytes is set up (by convention - not needed here)
 *
 * returns	-1 with errno set if ibv_post_send fails (block NOT SENT)
  *		0  if ibv_post_send succeeds (block IS SENT)
 */
static int
our_post_send(struct exs_conn *this_conn, struct ibv_send_wr *wr)
{
	int			ret;
	struct ibv_send_wr	*send_bad_wr = NULL;

	TRACE_DBG("wr %p, wr.wr_id %" PRIx64, (void *)wr, wr->wr_id);

	if(!(this_conn->conn_flags & EXS_NEED_DISCONNECT))
	{
		TRACE_DBG("ibv_post_send not done due to conn_state %s, conn_flags %0x",
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);
		errno = ECONNRESET;
		return -1;
	}
	ret = ibv_post_send(this_conn->qp, wr, &send_bad_wr);
	if(ret){
		/* ibv_post_send failed */
		OUR_PERROR(ret, "failed ibv_post_send, wr %p", (void *)wr);

		print_send_wr(stderr, send_bad_wr);

		TRACE_DBG("conn_state %s, conn_flags %0x",
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);
		/* there was an input/output error, is there a better errno? */
		errno = EIO;		/* I/O error */
		return -1;
	}
	TRACE_DBG("wr %p, wr.wr_id %" PRIx64, (void *)wr, wr->wr_id);

	return ret;
}	/* our_post_send */


/*
 * Called only in our_completion_thread() context
 * called from our_post_transaction_blocks()
 *	and our_completion_thread() on an IBV_WC_RECV
 *
 * Returns 0 if all ok, else errno on error.
 */
static int
our_try_to_post_recv_block(struct exs_conn *cb, struct transaction_block *block)
{
	int			ret;
	struct ibv_recv_wr	*recv_bad_wr;

	/* set expected completion code RECV */
	block->expected_comp = IBV_WC_RECV;

	VALGRIND_MAKE_MEM_UNDEFINED(block->recv_wr.sg_list[0].addr,
				block->recv_wr.sg_list[0].length);
	if(cb->conn_state == EXS_CONN_FINISHED
		|| (cb->conn_state >= EXS_CONN_CLOSING
			&& cb->conn_state <= EXS_CONN_CLOSED))
	{
		TRACE_DBG("ibv_post_recv not done due to conn_state %s, conn_flags %0x",
			our_conn_state_str(cb->conn_state),
			cb->conn_flags);
		ret = ENOTCONN;		/* Transport endpoint is not connected*/
	}
	else if((ret = ibv_post_recv(cb->qp, &block->recv_wr, &recv_bad_wr)))
	{
		OUR_PERROR(ret, "failed ibv_post_recv, wr %p",
			(void *)&block->recv_wr);

		TRACE_DBG("num_sge %d, length %d, addr %p",
			block->recv_wr.num_sge,
			block->recv_wr.sg_list->length,
			(void *)(block->recv_wr.sg_list->addr));
		TRACE_DBG("conn_state %s, conn_flags %0x",
			our_conn_state_str(cb->conn_state),
			cb->conn_flags);
	}
	else
	{	
		TRACE_DBG("ibv_post_recv ok for block %p, recv_wr.wr_id %" PRIx64,
				(void *)block, block->recv_wr.wr_id);
	}
	return ret;
}	/* our_try_to_post_recv_block */


/*
 * Called only in our_completion_thread() context
 * called only from our_create_and_post_blocks()
 *
 * Tries to call ibv_post_recv() 2 * posted_credits times
 *	(via calls to our_try_to_post_recv_block())
 *
 * Returns 0 if all ok, else errno on error.
 */
static int
our_post_transaction_blocks(struct exs_conn *this_conn)
{
	int				ret, j, n_blocks_to_post;

	/* post all necessary receive work requests, minus 1
	 * (which has been posted as part of the initial negotiation) */
	n_blocks_to_post = RQ_DEPTH(this_conn) - 1;
	ret = j = 0;
	for (j = 0; j < n_blocks_to_post; ++j) {
		ret = our_try_to_post_recv_block(this_conn,
				&this_conn->recv_array[j]);
		if(ret)
		{
			/* for now, we go with just the blocks already posted */
			break;
		}
	}
	if (j > 0) {
		this_conn->recv_array[j - 1].next = &this_conn->recv_array[0];
	}

	TRACE_DBG("posted %d of %d recv transaction blocks on cb %p",
		j, n_blocks_to_post, (void *)this_conn);

	if(ret == 0 && j == 0)
		ret = ENOMEM;

	return ret;
}	/* our_post_transaction_blocks	*/


/*
 * called from our_init_transaction_block()
 *
 * to create a separate buffer to hold unregistered small packets
 *	and then register it's memory area
 *
 * Returns 0 if memory allocated and registered ok, else -1
 */
static int
our_calloc_and_register_recv_buf(struct exs_conn *cb,
				struct transaction_block *block, size_t size)
{
	int ret;

	if(size <= 7)
	{	/* always allocate and register at least 8 bytes */
		size = 8;
	}

	if ((ret = exs_buf_alloc(cb, &block->recv_rdma_buf, size,
			IBV_ACCESS_LOCAL_WRITE|IBV_ACCESS_REMOTE_WRITE,
						"small message recv buf"))) {
		OUR_PERROR(errno, "alloc transaction recv_rdma_buf");
	}

	return ret;
}	/* our_calloc_and_register_recv_buf */

/*
 * called from our_create_transaction_block()
 *
 * to create a separate buffer to hold unregistered small packets
 *	and then register it's memory area
 *
 * Returns 0 if memory allocated and registered ok, else -1
 */
static int
our_calloc_and_register_send_buf(struct exs_conn *cb,
				struct transaction_block *block, int size)
{
	int ret;

	if(size <= 7)
	{	/* always allocate and register at least 8 bytes */
		size = 8;
	}

	if ((ret = exs_buf_alloc(cb, &block->send_buf, size, 0,
					"small message send buf"))) {
		OUR_PERROR(errno, "alloc transaction send_buf");
	}

	return ret;
}	/* our_calloc_and_register_send_buf */


/* create and initialize a transaction block
 *
 * called from our_create_and_post_blocks() and our_nego_init()
 *
 * no locks needed for this function
 *
 * Returns pointer to newly created and initialized block if all ok, else NULL
 */
static struct transaction_block *
our_create_transaction_block(struct exs_conn *cb, uint32_t sp_size)
{
	struct exs_buf           buf;
	struct transaction_block *block;

	block = EXS_CALLOC(1, sizeof(struct transaction_block) + sp_size);
	if(block == NULL)
	{
		OUR_PERROR(errno, "alloc transaction block");
		goto err0;
	}

	TRACE_DBG("block %p: allocated %zu bytes",
			(void *)block,
			sizeof(struct transaction_block) + sp_size);

	memset(&buf, 0, sizeof(buf));
	if (exs_buf_alloc(cb, &buf, sizeof(struct our_header) + sp_size,
					0, "transaction block buffer")) {
		goto err1;
	}

	if (!our_init_control_transaction_block(block, &buf, buf.ptr,
								buf.size)) {
		goto err2;
	}

	return block;
err2:
	exs_buf_free(&buf);
err1:
	EXS_FREE(block);
err0:
	return NULL;
}	/* our_create_transaction_block */


static struct transaction_block *
our_init_recv_transaction_block(struct exs_conn *cb,
		struct transaction_block *block,
		void *buf_ptr, int bufsize)
{
	assert(cb->recv_control_buf.ptr <= buf_ptr
					&& buf_ptr < cb->recv_control_buf.ptr
						+ cb->recv_control_buf.size);

	TRACE_DBG("block %p recv_wr %p sg_list %p recv_header %p (%lu bytes) recv_buf %p (%lu bytes)",
		(void *)block, (void *)&block->recv_wr, (void *)block->sg_list,
		(void *)&block->recv_header, sizeof(block->recv_header),
		(void *)&block->recv_buf, bufsize - sizeof(block->recv_buf));

	block->expected_comp = IBV_WC_RECV;
	block->recv_header = buf_ptr;
	block->recv_buf = buf_ptr + sizeof(*block->recv_header);
	block->block_mr = cb->recv_control_buf.mr;

	/* set up receive work request to point to 1-element s/g list */
	block->recv_wr.wr_id = (uintptr_t)block;
	block->recv_wr.sg_list = block->sg_list;
	block->recv_wr.num_sge = 1;

	/* set up recv s/g list element to point at recv header/buffer */
	block->sg_list[0].addr = (uintptr_t)block->recv_header;
	block->sg_list[0].length = bufsize;
	block->sg_list[0].lkey = block->block_mr->lkey;

	TRACE_DBG("cb %p, return block %p", (void *)cb, (void *)block);
	return block;
}	/* our_init_recv_transaction_block */


static struct transaction_block *
our_init_control_transaction_block(struct transaction_block *block,
		struct exs_buf *msgbuf,
		void *buf_ptr, size_t msg_size)
{
	assert(msgbuf->ptr <= buf_ptr && buf_ptr < msgbuf->ptr + msgbuf->size);

	memcpy(&block->send_buf, msgbuf, sizeof(block->send_buf));
	block->block_mr = msgbuf->mr;
	block->send_header = (struct our_header *)(buf_ptr);
	block->advert_buf
		= (struct our_advert *)(buf_ptr + sizeof(struct our_header));
	block->ack_buf = (struct our_stream_ack *)(buf_ptr
						+ sizeof(struct our_header));
	block->eof_buf = (struct our_eof *)(buf_ptr + sizeof(struct our_header));

	/* set up send work request to point to 2-element s/g list */
	block->send_wr.wr_id = (uint64_t)(unsigned long)block;
	block->send_wr.sg_list = block->sg_list;
	block->send_wr.num_sge = 1;
	block->send_wr.opcode = IBV_WR_SEND;

	/* point send s/g list element to point to send header */
	block->sg_list[0].addr = (uintptr_t)buf_ptr;
	block->sg_list[0].length = msg_size;
	block->sg_list[0].lkey = block->block_mr->lkey;

	return block;
}	/* our_init_control_transaction_block */


static struct transaction_block *
our_init_send_transaction_block(struct exs_conn *cb,
		struct transaction_block *block)
{
	if ((cb->fd_flags & EXS_FD_SP_COPY_UNREG_ON_SEND)
			&& cb->small_packet_max_size > 0) {
		if (our_calloc_and_register_send_buf(cb, block,
					cb->small_packet_max_size) != 0) {
			goto err2;
		}
	}
	/* else both send_buf and send_buf_mr remain 0 (NULL) */

	block->expected_comp = (cb->fd_flags & EXS_FD_NO_WRITE_WITH_IMM)
					? IBV_WC_SEND : IBV_WC_RDMA_WRITE;
	block->advert_buf = EXS_CALLOC(1, sizeof(*block->advert_buf));
	if (!block->advert_buf) {
		goto err3;
	}

	/* set up write request work request to point to a 1-element s/g list */

	TRACE_DBG("block %p: send_wr %p points at sg_list %p",
		(void *)block, (void *)&block->send_wr, (void *)block->sg_list);

	block->send_wr.wr_id = (uint64_t)(unsigned long)block;
	block->send_wr.sg_list = block->sg_list;
	block->send_wr.num_sge = 1;
	block->send_wr.send_flags = IBV_SEND_SIGNALED;
	if (!(cb->fd_flags & EXS_FD_NO_WRITE_WITH_IMM)) {
		block->send_wr.opcode = IBV_WR_RDMA_WRITE_WITH_IMM;
	} else {
		block->send_wr.opcode = IBV_WR_RDMA_WRITE;
		block->send_wr.next = &block->writeack_wr;

		block->writeack_wr.wr_id = (uintptr_t)block;
		block->writeack_wr.sg_list = block->sg_list + 1;
		block->writeack_wr.num_sge = 2;
		block->writeack_wr.opcode = IBV_WR_SEND;
		block->writeack_wr.send_flags = IBV_SEND_SIGNALED;

		block->sg_list[1].addr
			= (uintptr_t)block->send_header;
		block->sg_list[1].length = sizeof(*block->send_header);
		block->sg_list[1].lkey = block->block_mr->lkey;

		block->sg_list[2].addr
			= (uintptr_t)block->writeack_buf;
		block->sg_list[2].length = sizeof(*block->writeack_buf);
		block->sg_list[2].lkey = block->block_mr->lkey;

		block->send_header->hd_type = htonl(OUR_WRITE_NOTIFY);
	}

	TRACE_DBG("cb %p return block %p", (void *)cb, (void *)block);
	return block;

	EXS_FREE(block->advert_buf);
err3:
	exs_buf_free(&block->send_buf);
err2:
	return NULL;
}	/* our_init_send_transaction_block */


/* Return the size of an element in a big buffer to be aligned at align bytes.
 * align is expected to be a power of 2 greater than or equal to 8. desired
 * is expected to be greater than 0. */
static inline size_t
our_aligned_size(size_t desired, size_t align)
{
	size_t result;

	assert(desired > 0);
	assert(align > 0);
	assert((align & 0x7) == 0);
	result = desired & ~(align - 1);
	if (result < desired) {
		result += align;
	}
	assert(result >= desired);
	return result;
}	/* our_aligned_size */


/*
 * Called only in our_completion_thread() context
 * called from our_client_finishing() and our_agent_finishing()
 *
 * Assumes that send_wwi_array, send_advert_array, and recv_control_buf
 * are all NULL when this function is called.  They will be non-NULL if this
 * function returns successfully.
 *
 * Returns 0 if all ok, errno on error.
 */
static int
our_create_and_post_blocks(struct exs_conn *cb)
{
	long i;
	size_t buf_size;
	char *buf_ptr;

	TRACE_DBG("small_packet_max_size %d fd_flags & EXS_FD_SP_COPY_UNREG %d",
		cb->small_packet_max_size,
		cb->fd_flags & EXS_FD_SP_COPY_UNREG);


	if (!(cb->send_wwi_array = EXS_CALLOC(cb->posted_credits,
			sizeof(*cb->send_wwi_array)))) {
		goto errout;
	}
	if (cb->fd_flags & EXS_FD_NO_WRITE_WITH_IMM) {
		buf_size = our_aligned_size(MIN_ALLOC_CONTROL_MSG_SIZE, 64);
		if (exs_buf_alloc(cb, &cb->send_writeack_buf,
					buf_size * SQ_DEPTH(cb),
					0, "send_advert_buf") != 0) {
			goto errout;
		}
		buf_ptr = cb->send_writeack_buf.ptr;
	} else {
		/* Unneeded, but squelches gcc warning about possibly
		 * uninitialized variable, since it can't tell that it is only
		 * accessed within if statements on the same conditional */
		buf_ptr = NULL;
	}
	for (i = 0; i < cb->posted_credits; ++i) {
		struct transaction_block *block = &cb->send_wwi_array[i];
		block->index = i;
		block->send_state = 0;
		if (i < cb->posted_credits - 1) {
			block->next = &cb->send_wwi_array[i + 1];
		} else {
			block->next = &cb->send_wwi_array[0];
		}
		if (sem_init(&block->sem, 0, 1) < 0) {
			goto errout;
		}
		if (cb->fd_flags & EXS_FD_NO_WRITE_WITH_IMM) {
			block->block_mr = cb->send_writeack_buf.mr;
			block->send_header = (struct our_header *)buf_ptr;
			block->writeack_buf = (struct our_notify *)(buf_ptr
						+ sizeof(*block->send_header));
			buf_ptr += buf_size;
		}
		if ((our_init_send_transaction_block(cb, block)) == NULL) {
			goto errout;
		}
	}
	cb->next_send = &cb->send_wwi_array[0];
	cb->next_send_primed = &cb->send_wwi_array[0];
	cb->next_send_unacked = &cb->send_wwi_array[0];
	TRACE_DBG("created %ld send_wwi transaction blocks on cb %p",
		i, (void *)cb);

	if (!(cb->send_advert_array = EXS_CALLOC(cb->posted_credits,
			sizeof(*cb->send_advert_array)))) {
		goto errout;
	}
	buf_size = our_aligned_size(MIN_ALLOC_CONTROL_MSG_SIZE, 64);
	if (exs_buf_alloc(cb, &cb->send_advert_buf, buf_size * SQ_DEPTH(cb),
						0, "send_advert_buf") != 0) {
		goto errout;
	}
	buf_ptr = cb->send_advert_buf.ptr;
	for (i = 0; i < cb->posted_credits; ++i) {
		struct transaction_block *block = &cb->send_advert_array[i];
		block->index = i;
		if (i < cb->posted_credits - 1) {
			block->next = &cb->send_advert_array[i + 1];
		} else {
			block->next = &cb->send_advert_array[0];
		}
		if (sem_init(&block->sem, 0, 1) < 0) {
			goto errout;
		}
		if ((cb->fd_flags & EXS_FD_SP_COPY_UNREG_ON_RECV)
				&& cb->small_packet_max_size > 0) {
			if (our_calloc_and_register_recv_buf(cb, block,
						cb->small_packet_max_size) != 0) {
				goto errout;
			}
		}
		if (our_init_control_transaction_block(block,
				&cb->send_advert_buf, buf_ptr,
				sizeof(struct our_header)
				+ sizeof(struct our_advert)) == NULL) {
			exs_buf_free(&cb->send_advert_buf);
			goto errout;
		}
		block->send_header->hd_type = htonl(OUR_ADVERTISEMENT);
		buf_ptr += buf_size;
	}
	TRACE_DBG("created %ld send_advert transaction blocks on cb %p",
		i, (void *)cb);

	/* register a block of memory for receiving control messages */
	buf_size = our_aligned_size(MIN_ALLOC_CONTROL_MSG_SIZE, 64);
	if (exs_buf_alloc(cb, &cb->recv_control_buf, buf_size * RQ_DEPTH(cb),
			IBV_ACCESS_LOCAL_WRITE, "recv_control_buf") != 0) {
		goto errout;
	}
	if (!(cb->recv_array = EXS_CALLOC(RQ_DEPTH(cb),
					sizeof(*cb->recv_array)))) {
		goto errout;
	}
	buf_ptr = cb->recv_control_buf.ptr;
	for (i = 0; i < RQ_DEPTH(cb); ++i) {
		if ((our_init_recv_transaction_block(cb, &cb->recv_array[i],
					buf_ptr, buf_size)) == NULL) {
			goto errout;
		}
		buf_ptr += buf_size;
	}
	cb->next_recv = &cb->send_advert_array[0];
	cb->next_recv_primed = &cb->send_advert_array[0];
	TRACE_DBG("created %ld recv transaction blocks on cb %p", i, (void *)cb);

	if (!cb->is_seqpacket) {
		struct transaction_block *ack_initial_array;
		int limit = 3 * cb->posted_credits / 2;
		if (exs_p_list_init(&cb->recved_advert_queue, 0,
						cb->posted_credits) != 0) {
			goto errout;
		}
		if (exs_p_list_init(&cb->recved_advert_empty_list, 0,
						cb->posted_credits) != 0) {
			goto errout;
		}
		if (exs_p_list_init(&cb->recved_advert_extra_list, 0,
						cb->posted_credits) != 0) {
			goto errout;
		}
		if (!(cb->recved_advert_initial_array = EXS_CALLOC(limit,
				sizeof(*cb->recved_advert_initial_array)))) {
			goto errout;
		}
		for (i = 0; i < limit; ++i) {
			struct stream_advert_block *block;
			block = &cb->recved_advert_initial_array[i];
			exs_p_list_add_tail(&cb->recved_advert_empty_list,
									block);
		}

		buf_size = our_aligned_size(MIN_ALLOC_CONTROL_MSG_SIZE, 64);
		if (exs_buf_alloc(cb, &cb->send_ack_buf, buf_size * SQ_DEPTH(cb),
						0, "send_ack_buf") != 0) {
			goto errout;
		}
		if (exs_p_list_init(&cb->send_ack_empty_list, 0,
						cb->posted_credits) != 0) {
			goto errout;
		}
		buf_ptr = cb->send_ack_buf.ptr;
		if (!(ack_initial_array = EXS_CALLOC(cb->posted_credits,
				sizeof(struct transaction_block)))) {
			goto errout;
		}
		cb->send_ack_array = ack_initial_array;
		for (i = 0; i < cb->posted_credits; ++i) {
			struct transaction_block *block = &cb->send_ack_array[i];
			block->index = i;
			if ((our_init_control_transaction_block(block,
					&cb->send_ack_buf, buf_ptr,
					sizeof(struct our_header)
					+ sizeof(struct our_stream_ack)))
								== NULL) {
				goto errout;
			}
			block->send_header->hd_type = htonl(OUR_ACK);
			exs_p_list_add_tail(&cb->send_ack_empty_list, block);
		}
		TRACE_DBG("created %ld send_ack transaction blocks on cb %p",
			i, (void *)cb);
	}

	buf_size = our_aligned_size(MIN_ALLOC_CONTROL_MSG_SIZE, 64);
	if (exs_buf_alloc(cb, &cb->send_eof_buf, buf_size * SQ_DEPTH(cb),
					0, "send_eof_buf") != 0) {
		goto errout;
	}
	buf_ptr = cb->send_eof_buf.ptr;
	for (i = 0; i < EOF_BLOCK_COUNT; ++i) {
		struct transaction_block *block = &cb->eof_block_array[i];
		block->send_state = 0;
		if ((our_init_control_transaction_block(block,
				&cb->send_eof_buf, buf_ptr,
				sizeof(struct our_header)
					+ sizeof(struct our_eof))) == NULL) {
			exs_buf_free(&cb->send_eof_buf);
			goto errout;
		}
	}

	return our_post_transaction_blocks(cb);

errout:
	return ENOMEM;
}	/* our_create_and_post_blocks */


/*
 * Called only in our_completion_thread() context
 * twice from our_completion_thread() when
 *	a) in state EXS_CONN_CLIENT_SETUP
 *	b) cb->nego_in_ptr is not NULL
 *	c) cb->nego_out_ptr is not NULL
 *
 * OR state onto cb's nego_state_info
 * and test for both flags set -- if both are set
 *	use results sent back from server
 *	allocate the blocks and post some of them
 *	post completion to the exs_connect()
 *	free the init memory
 *
 * Returns 0 if all ok, errno on error.
 */
static int
our_client_finishing(struct exs_conn *cb, int state)
{
	int				ret;
	unsigned int			uvalue;
	struct transaction_block	*nego_in_ptr, *nego_out_ptr;
	struct our_nego_mess		*nego_mess;

	nego_in_ptr = cb->nego_in_ptr;
	nego_out_ptr = cb->nego_out_ptr;
	atomic_set_bits(state, &nego_out_ptr->send_state);
	if((nego_out_ptr->send_state & (SEND_DONE_BIT|ACK_DONE_BIT))
					== (SEND_DONE_BIT|ACK_DONE_BIT))
	{	/* got IBV_WC_SEND complete for OUR_SETUP_REQUEST
		 * and IBV_WC_RECV complete for OUR_SETUP_REPLY
		 */
		/* check values server sent back in recv_buf */
		nego_mess = (struct our_nego_mess *)nego_in_ptr->recv_buf;

		uvalue = ntohl(nego_mess->client_credits);
		if (cb->posted_credits != uvalue) {
			TRACE_ERR("client's credits %u, expected %u",
				uvalue, cb->posted_credits);
			ret = EPROTO;
			goto exit0;
		}

		uvalue = ntohl(nego_mess->sp_max_size);
		if (cb->small_packet_max_size != uvalue) {
			TRACE_ERR("client's small packet_max size %u, expected %u",
				uvalue, cb->small_packet_max_size);
			ret = EPROTO;
			goto exit0;
		}

		nego_mess = (struct our_nego_mess *)nego_in_ptr->recv_buf;
		our_set_send_credit_limits(cb,
					ntohl(nego_mess->client_credits));
		cb->small_packet_max_size = ntohl(nego_mess->sp_max_size);

		our_connect_done(cb);
	}

	ret = 0;

exit0:
	return ret;
}	/* our_client_finishing */


static inline void
our_fill_client_nego_mess(struct exs_conn *cb, struct our_nego_mess *nego_send)
{
	nego_send->client_credits = htonl(cb->posted_credits);
	nego_send->sp_max_size = htonl(cb->small_packet_max_size);
	nego_send->version = EXS_VERSION;
	nego_send->subversion = EXS_SUBVERSION;
	nego_send->subsubversion = EXS_SUBSUBVERSION;
	nego_send->is_seqpacket = cb->is_seqpacket;
	nego_send->fd_flags = cb->fd_flags & ~EXS_FD_NODELAY;
	if (!cb->is_seqpacket && cb->stream_buf_size > 0) {
		nego_send->recv_buf_addr
			= htonll((uint64_t)cb->recv_buf.buf.ptr);
		nego_send->recv_buf_rkey = htonl(cb->recv_buf.buf.mr->rkey);
		nego_send->recv_buf_len = htonl(cb->recv_buf.buf.size);
		if (cb->fd_flags & EXS_FD_STREAM_BUF_NOALIGN) {
			nego_send->recv_buf_align = htonl(0);
		} else {
			nego_send->recv_buf_align = htonl(our_cache_line_size);
		}
	}
}	/* our_fill_client_nego_mess */


/*
 * called only from our_connect_established()
 *
 * post recv into recv_buf,
 * then store parameter values into the send_nego_mess and send it
 *
 * Returns 0 if all ok, else the error number
 */
static int
our_client_nego_launch(struct exs_conn *cb)
{
	int				ret;
	struct transaction_block	*nego_out_ptr;
	struct our_nego_mess		*nego_send;
	struct ibv_send_wr		*send_bad_wr;

	nego_out_ptr = cb->nego_out_ptr;

	/* mark that this send is a setup request */
	nego_out_ptr->send_header->hd_type = htonl(OUR_SETUP_REQUEST);
	nego_out_ptr->send_header->hd_seqno = 0;

	nego_send = (struct our_nego_mess*)((uint8_t *)nego_out_ptr->send_buf.ptr
						+ sizeof(struct our_header));
	our_fill_client_nego_mess(cb, nego_send);

	/* dump everything being sent -- in hex, 16 bytes/line */
	TRACE_DBG("dump of %" PRIu32 " nego header bytes being sent from addr %p",
			nego_out_ptr->sg_list[0].length,
			(void *)(uintptr_t)nego_out_ptr->sg_list[0].addr);
	TRACE_DBG("nego bytes start at addr %p", (void *)nego_send);
	DUMP_BUF((void *)(uintptr_t)nego_out_ptr->sg_list[0].addr,
					nego_out_ptr->sg_list[0].length);

	/* set expected completion code SEND,
	 * always send this signaled, inline
	 */
	nego_out_ptr->expected_comp = IBV_WC_SEND;
	nego_out_ptr->send_wr.send_flags = IBV_SEND_SIGNALED | IBV_SEND_INLINE;

	/* Before sending the negotiation packet, we must post recvs to catch
	 * advertisements for exs_recvs on the remote side */
	ret = our_create_and_post_blocks(cb);
	if (ret != 0) {
		OUR_PERROR(ret, "failed create and post blocks");
		return ret;
	}

	if((ret = ibv_post_send(cb->qp, &nego_out_ptr->send_wr, &send_bad_wr)))
	{	/* ibv_post_send failed */
		OUR_PERROR(ret, "failed ibv_post_send");
		return ret;
	}
	STATISTICS_INCREMENT(total_setup_sends);
	TRACE_DBG("ibv_post_send ok for nego_out_ptr->send_wr %p, send_wr.wr_id %" PRIx64,
		(void *)&nego_out_ptr->send_wr,
		nego_out_ptr->send_wr.wr_id);
	return 0;
}	/* our_client_nego_launch */


/*
 * Called only in our_completion_thread() context for a server connection
 *
 * called only from our_pending_accept_done()
 *	(which is only called directly from our_completion_thread()
 *	when it gets successful IBV_WC_SEND
 *	of OUR_SETUP_REPLY that agent sent to client)
 *
 * fill in and post event structure for the exs_accept
 * then free new_pending
 *
 * Returns 0 if all ok, -1 on error.
 */
static void
our_fill_in_accept_event(struct pending_accept *new_pending, int listen_socket)
{

	TRACE_DBG("building completion event %p for accepted connection %p in state %s",
		(void *)new_pending->evt_item,
		(void *)new_pending->accepted_conn,
		our_conn_state_str(new_pending->accepted_conn->conn_state));

	memset(new_pending->evt_item, 0, sizeof(struct exs_event));
	new_pending->evt_item->exs_evt_errno = new_pending->err_numb;
	new_pending->evt_item->exs_evt_type = EXS_EVT_ACCEPT;
	new_pending->evt_item->exs_evt_socket = listen_socket;
	new_pending->evt_item->exs_evt_union.exs_evt_accept.exs_evt_new_socket
					= new_pending->accepted_conn->sockno;
	TRACE_DBG("listen_socket %d, accepted_socket %d",
		listen_socket, new_pending->accepted_conn->sockno);
	TRACE_DBG("exs_ahandle %p, exs_addr %p, exs_addrlen %d",
		(void *)new_pending->addrvec.exs_ahandle,
		(void *)new_pending->addrvec.exs_addr,
		new_pending->addrvec.exs_addrlen);
	new_pending->evt_item->exs_evt_ahandle
				= new_pending->addrvec.exs_ahandle;
	if((new_pending->evt_item->
			exs_evt_union.exs_evt_accept.exs_evt_addr
			= new_pending->addrvec.exs_addr) != NULL
			&& new_pending->evt_item->exs_evt_errno == 0)
	{
		/* get correct sockaddr of remote client */
		new_pending->evt_item->
			exs_evt_union.exs_evt_accept.exs_evt_addrlen
			= our_copy_dst_addr(
				new_pending->accepted_conn->cm_id,
				new_pending->addrvec.exs_addr,
				new_pending->addrvec.exs_addrlen);
	}

	TRACE_DBG("Posting completion event %p for accepted connection %p",
		(void *)new_pending->evt_item,
		(void *)new_pending->accepted_conn);

	_put_full_evt_queue_event(new_pending->qhandle, new_pending->evt_item);

	EXS_FREE(new_pending);

}	/* our_fill_in_accept_event */


/*
 * Called only in our_completion_thread() context for an agent connection
 * called only from our_completion_thread()
 *
 * when it gets successful IBV_WC_SEND
 * of OUR_SETUP_REPLY that agent sent to client
 *
 * Returns 0 if all ok, -1 on error.
 */
static int
our_pending_accept_done(struct exs_conn *cb)
{
	struct pending_accept	*new_pending;

	TRACE_DBG("start, cb %p, state %s",
		(void *)cb, our_conn_state_str(cb->conn_state));

	new_pending = cb->accept_pending;
#if defined(ALLOW_PARANOID_CHECKING)
	if(new_pending == NULL)
	{	/* should never happen */
		TRACE_ERR("cb %p, state %s, accept_pending NULL",
			(void *)cb, our_conn_state_str(cb->conn_state));
		return -1;
	}
#endif

	if(cb->nego_out_ptr != NULL)
	{	/* need to free nego block */
		TRACE_DBG("free nego out transaction block %p",
			(void *)cb->nego_out_ptr);
		/* FIXME: hack to get rid of memory leak -- need a more
		 * permanent solution */
		ibv_dereg_mr(cb->nego_out_ptr->block_mr);
		EXS_FREE(cb->nego_out_ptr->send_header);
		our_free_transaction_block(cb->nego_out_ptr);
		cb->nego_out_ptr = NULL;	/* so we don't free it again */
	}


	cb->conn_state = EXS_CONN_RUNNING;

	/* fills in and posts accept event, then frees new_pending */
	our_fill_in_accept_event(new_pending, cb->accept_parent->sockno);
	cb->accept_pending = NULL;		/* so we don't free it again */
	TRACE_DBG("out, cb %p, state %s, returning 0",
		(void *)cb, our_conn_state_str(cb->conn_state));

	return 0;				/* all ok */
}	/* our_pending_accept_done */


/*
 * Called only in our_completion_thread() context
 * called only from our_completion_thread() when
 *	a) got IBV_WC_RECV complete for OUR_SETUP_REQUEST
 *	b) in state EXS_CONN_AGENT_SETUP
 *	c) cb->nego_in_ptr is not NULL
 *	d) cb->nego_out_ptr is not NULL
 *
 * Uses negotiated values sent by client to calculate minimums
 * Then uses these minimums to allocate empty transaction blocks
 *	and to post some of them
 * Then sets up and sends OUR_SETUP_REPLY back to client with new values
 *
 * Returns 0 if all ok, else error number
 */
static int
our_agent_finishing(struct exs_conn *cb )
{
	int				ret;
	unsigned int			uvalue;
	struct transaction_block	*nego_in_ptr, *nego_out_ptr;
	struct our_nego_mess		*nego_mess;
	struct ibv_send_wr		*send_bad_wr;

	nego_in_ptr = cb->nego_in_ptr;
	nego_mess = (struct our_nego_mess *)nego_in_ptr->recv_buf;

	/* client's send credits should equal agent's recv credits */
	uvalue = ntohl(nego_mess->client_credits);
	if (cb->posted_credits != uvalue) {
		TRACE_ERR("client's credits %u, expected %u",
			uvalue, cb->posted_credits);
		ret = EPROTO;
		goto exit0;
	}

	uvalue = ntohl(nego_mess->sp_max_size);
	if (cb->small_packet_max_size != uvalue) {
		TRACE_ERR("client's small packet_max size %u, expected %u",
			uvalue, cb->small_packet_max_size);
		ret = EPROTO;
		goto exit0;
	}

	/* now set up and send the SETUP_REPLY packet */
	nego_out_ptr = cb->nego_out_ptr;

	/* mark that this send is a setup_reply */
	nego_out_ptr->send_header->hd_type = htonl(OUR_SETUP_REPLY);
	nego_out_ptr->send_header->hd_seqno = 0;

	nego_mess = (struct our_nego_mess *)(nego_out_ptr->send_buf.ptr
						+ sizeof(struct our_header));
	our_setup_agent_nego_mess(cb, nego_mess);

	/* dump everything being sent -- in hex, 16 bytes/line */
	TRACE_DBG("dump of %d nego bytes being sent from addr %p",
			nego_out_ptr->sg_list[0].length,
			(void *)nego_out_ptr->sg_list[0].addr);
	DUMP_BUF((void *)(unsigned long)nego_out_ptr->sg_list[0].addr,
					nego_out_ptr->sg_list[0].length);

	/* set expected completion code SEND, always send this signaled */
	nego_out_ptr->expected_comp = IBV_WC_SEND;
	nego_out_ptr->send_wr.send_flags = IBV_SEND_SIGNALED | IBV_SEND_INLINE;

	if((ret = ibv_post_send(cb->qp, &nego_out_ptr->send_wr, &send_bad_wr)))
	{	/* ibv_post_send failed */
		OUR_PERROR(ret, "failed ibv_post_send");
		goto exit0;
	}
	STATISTICS_INCREMENT(total_setup_sends);
	TRACE_DBG("ibv_post_send ok for nego_out_ptr->send_wr %p, send_wr.wr_id %" PRIx64,
		(void *)&nego_out_ptr->send_wr,
		nego_out_ptr->send_wr.wr_id);

	ret = 0;
exit0:
	return ret;
}	/* our_agent_finishing */


/*
 * Called only in our_con_man_thread() context
 * called twice from our_setup_qp()
 *
 * to allocate and initialize one negotiation exchange block
 *
 * no locks needed for this function
 *
 * returns 0 if all ok, else the errno if things go bad
 */
static int
our_nego_init(struct exs_conn *cb, struct transaction_block **cb_nego_ptr)
{
	struct transaction_block	*nego_ptr;

	TRACE_DBG("start cb %p, state %s",
			(void *)cb, our_conn_state_str(cb->conn_state));

	nego_ptr = our_create_transaction_block(cb,
						sizeof(struct our_nego_mess));
	if(nego_ptr == NULL)
	{
		return ENOMEM;
	}
	*cb_nego_ptr = nego_ptr;

	return 0;
}	/* our_nego_init */


/*
 * called only from our_completion_thread() and our_try_to_disconnect()
 *
 * returns 0 on success, errno on error after printing error message
 */
static int
our_rdma_disconnect(struct exs_conn *this_conn)
{
	int	ret;

	TRACE_DBG("calling rdma_disconnect(%p) for cb %p, new state %s, new conn_flags %0x",
		(void *)this_conn->cm_id,
		(void *)this_conn, our_conn_state_str(this_conn->conn_state),
		this_conn->conn_flags);

	ret = rdma_disconnect(this_conn->cm_id);
	if(ret != 0)
	{	/* the rdma_disconnect() didn't work, find out why */
		TRACE_DBG("rdma_disconnect cb %p returned %d, errno %d, state %s, conn_flags %0x",
			(void *)this_conn, ret, errno,
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);

		if(errno == EAGAIN || errno == EINVAL)
		{	/* our_con_man_thread waiting to disconnect */
			ret = 0;
		}
		else
		{
			OUR_PERROR(errno, "rdma_disconnect cm_id %p failed, state %s, conn_flags %0x",
				(void *)this_conn->cm_id,
				our_conn_state_str(this_conn->conn_state),
				this_conn->conn_flags);
		}
	}

	return ret;
}	/* our_rdma_disconnect */


/*
 * called only if we get canceled in our_completion_thread()
 * when waiting to get the mutex, which is explicitly NOT a cancelation point!
 */
void
our_completion_cancel_handler(void *ptr)
{
	struct exs_conn *cb = ptr;
	TRACE_DBG("[%lu] leave", pthread_self());

	OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
	cb = our_socket_unref(cb);
}	/* our_completion_cancel_handler */


static int
our_repost_recv_block(struct exs_conn *cb, struct transaction_block *block)
{
	int ret = 0;

	if(cb->conn_flags & EXS_NEED_DISCONNECT) {
		ret = our_try_to_post_recv_block(cb, block);
	} else {
		TRACE_DBG("our_try_to_post_recv_block not called due to connection disconnected, state %s, conn_flags %#x",
				our_conn_state_str(cb->conn_state),
				cb->conn_flags);
	}

	return ret;
}	/* our_repost_recv_block */

static int
our_handle_recv_advert_seqpacket(struct exs_conn *cb,
				struct transaction_block *block)
{
	struct our_advert		*ad_ptr;
	struct transaction_block	*this;
	int				ret;

	ret = 0;

	/* just got an advert */
	STATISTICS_INCREMENT(recv_advert_completions);
	ad_ptr = (struct our_advert *)block->recv_buf;

	this = cb->next_send_primed;
	this->advert_buf->ad_buf_addr = ntohll(ad_ptr->ad_buf_addr);
	this->advert_buf->ad_buf_key = ntohl(ad_ptr->ad_buf_key);
	this->advert_buf->ad_buf_len = ntohl(ad_ptr->ad_buf_len);

	ret = our_repost_recv_block(cb, block);

	if (atomic_set_bits(ADVERT_READY_BIT, &this->send_state)
					& SEND_READY_BIT) {
		if (cb->ops->xmit_if_match(cb, this) < 0) {
			ret = -1;
		}
	}

	cb->next_send_primed = this->next;

	return ret;
}	/* our_handle_recv_advert_seqpacket */

static int
our_handle_recv_advert_stream(struct exs_conn *cb,
				struct transaction_block *block)
{
	struct our_advert		*ad_ptr;
	struct stream_advert_block	*this;
	uint64_t			ad_seqno;
	unsigned int			ad_phase;
	int				ret;

	ret = 0;

	/* just got an advert */
	STATISTICS_INCREMENT(recv_advert_completions);
	ad_ptr = (struct our_advert *)block->recv_buf;

	/* Acknowledge this advert when sending next WWI */
	atomic_add(1, &cb->next_recv_advert_msg_seqno);

	ad_seqno = ntohll(ad_ptr->ad_buf_seqno);
	ad_phase = ntohl(ad_ptr->ad_buf_phase);
	TRACE_DBG("got advert with seqno=%" PRIu64 "; len=%" PRIu32 "; current seqno=%" PRIu64,
		ad_seqno,
		ntohl(ad_ptr->ad_buf_len),
		cb->send_seqno);
	this = exs_p_list_del_head(&cb->recved_advert_empty_list);
	if (!this) {
		TRACE_INFO("creating new advert block for cb %p",
				(void *)cb);
		this = malloc(sizeof(*this));
		if (!this) {
			OUR_PERROR(errno, "could not allocate new advert block");

			/* Really ugly hack: throw away advert if we
			 * cannot allocate a new block. */
			OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
			TRACE_DBG("current advert seqno %" PRIu64 " phase %" PRIu32,
				ad_seqno,
				ntohl(ad_ptr->ad_buf_phase));
			TRACE_DBG("sender seqno %" PRIu64 " phase %u",
				cb->send_seqno, cb->send_phase);
			assert(PHASE_IS_DIRECT(ad_phase));
			cb->send_phase = ad_phase + 1;
			OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
			ret = our_repost_recv_block(cb, block);
			return ret;
		}
		STATISTICS_INCREMENT(extra_advert_blocks);
	}
	this->advert_buf.ad_buf_seqno = ad_seqno;
	this->advert_buf.ad_buf_addr = ntohll(ad_ptr->ad_buf_addr);
	this->advert_buf.ad_buf_key = ntohl(ad_ptr->ad_buf_key);
	this->advert_buf.ad_buf_len = ntohl(ad_ptr->ad_buf_len);
	this->advert_buf.ad_buf_flags = ntohl(ad_ptr->ad_buf_flags);
	this->advert_buf.ad_buf_phase = ad_phase;
	this->offset = 0;
	exs_p_list_add_tail(&cb->recved_advert_queue, this);

	ret = our_repost_recv_block(cb, block);

	/* Process piggybacked ACK */
	OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
	cb->remote_buf_n_empty += ntohl(ad_ptr->ad_ack_empty);
	cb->stream_wwi_counter += ntohl(ad_ptr->ad_ack_count);
	OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);

	if (cb->next_send_primed->send_state & SEND_READY_BIT) {
		TRACE_DBG("advert and send ready");
		if (cb->ops->xmit_if_match(cb, cb->next_send_primed) < 0) {
			ret = -1;
		}
	} else {
		TRACE_DBG("advert ready only");
	}


	return ret;
}	/* our_handle_recv_advert_stream */

static int
our_handle_send_write_seqpacket(struct exs_conn *cb,
		struct transaction_block *block)
{
	STATISTICS_INCREMENT(send_done_after_completion);
	return our_all_done_with_block(cb, block);
}	/* our_handle_send_write_seqpacket */

static int
our_handle_send_write_stream(struct exs_conn *cb,
		struct transaction_block *block)
{
	unsigned int new_outstanding_completion_count;
	int ret;

	OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
	new_outstanding_completion_count =
		--block->outstanding_completion_count;
	if (new_outstanding_completion_count == 0
			&& block->buf_offset == block->user_length) {
		/* Post the semaphore on this block to open it up for
		 * another send if and only if there are no remaining
		 * outstanding completions for this block and we have
		 * transmitted all data for this block. */
		STATISTICS_INCREMENT(send_done_after_completion);
		atomic_set_bits(SEND_DONE_BIT, &block->send_state);
	}
	OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);

	if (block->send_state & SEND_DONE_BIT) {
		atomic_clear_bits(SEND_DONE_BIT, &block->send_state);
		ret = our_all_done_with_block(cb, block);
	} else {
		ret = 0;
	}

	return ret;
}	/* our_handle_send_write_stream */


/* Sends an ACK.  Returns 0 on success and -1 on failure. */
static int
our_send_ack(struct exs_conn *cb, int *send_flags)
{
	struct transaction_block *ablock;
	unsigned int unsignaled_count;

	/* This test-and-set is nonatomic, but that is ok since this function is
	 * under stream_sync_mutex and the only other consumer of this value can
	 * only increase the stream_ack_counter. */
	if (cb->stream_ack_counter == 0) {
		/* Delay the ACK until we have credit to send it */
		STATISTICS_INCREMENT(no_credits_on_send_ack);
		return 0;
	}
	--cb->stream_ack_counter;
	STATISTICS_SET_MAX(max_used_ack_credits,
				cb->posted_credits - cb->stream_ack_counter);
	ablock = exs_p_list_del_head(&cb->send_ack_empty_list);
	if (!ablock) {
		STATISTICS_INCREMENT(no_tblock_on_send_ack);
		return 0;
	}

	TRACE_DBG("send ACK seqno %u n_empty %u/%u n_unacked_wwi %u/%u",
		cb->ack_seqno + cb->recv_buf.n_empty,
		cb->recv_buf.n_empty,
		cb->recv_buf.n_empty_threshold,
		cb->n_unacked_wwi,
		cb->n_unacked_wwi_threshold);

	cb->ack_seqno += cb->recv_buf.n_empty;

	ablock->send_header->hd_type = htonl(OUR_ACK);
	ablock->send_header->hd_seqno = htonl(cb->ack_seqno);
	ablock->ack_buf->recv_buf_empty = htonl(cb->recv_buf.n_empty);
	ablock->ack_buf->recv_buf_ack_count = htonl(cb->n_unacked_wwi);

	ablock->send_wr.send_flags = *send_flags;

	if ((sizeof(ablock->send_header) + sizeof(ablock->ack_buf))
						<= cb->inline_limit) {
		ablock->send_wr.send_flags |= IBV_SEND_INLINE;
		unsignaled_count = ++cb->ack_signal_count;
		if ((ablock->send_wr.send_flags & IBV_SEND_SIGNALED)
				|| unsignaled_count >= cb->signal_limit) {
			/* time to force a periodic signal */
			TRACE_DBG("force a signal, ack_signal_count %d, signal_limit %d",
				unsignaled_count, cb->signal_limit);
			ablock->send_wr.send_flags |= IBV_SEND_SIGNALED;
			ablock->unsignaled_count = unsignaled_count;
			/* not atomic, but ok due to stream_sync_mutex */
			cb->ack_signal_count = 0;
		} else {
			ablock->unsignaled_count = 0;
		}
	} else {
		ablock->unsignaled_count = cb->ack_signal_count + 1;
		cb->ack_signal_count = 0;
		ablock->send_wr.send_flags |= IBV_SEND_SIGNALED;
	}

	/* now post send of this advert, then leave */
	STATISTICS_INCREMENT(ack_blocks_sent);
	if (our_post_send(cb, &ablock->send_wr) < 0) {
		/* got an input/output error, so ACK was NOT SENT.
		 * Post the event for this block anyways. */
		return -1;
	}

	*send_flags = ablock->send_wr.send_flags;

	if (!(*send_flags & IBV_SEND_SIGNALED)) {
		exs_p_list_add_tail(&cb->send_ack_empty_list, ablock);
	}

	cb->recv_buf.n_empty = 0;
	cb->n_unacked_wwi = 0;

	return 0;
}	/* our_send_ack */


/* Sends an EOF notification.  Returns 0 on success; -1 on failure. */
static int
our_send_eof(struct exs_conn *cb, struct transaction_block *block,
		int *send_flags, int shutdown_how)
{
	unsigned int bits_to_clear;
	if (atomic_set_bits(SEND_DONE_BIT, &block->send_state)
							& SEND_DONE_BIT) {
		/* Already done */
		TRACE_DBG("cb %p block %p already sent",
				(void *)cb, (void *)block);
		errno = EBUSY;
		return -EBUSY;
	}

	switch (shutdown_how) {
	case SHUT_RD:
		bits_to_clear = EXS_NEED_EOF_RECV;
		break;
	case SHUT_WR:
		bits_to_clear = EXS_NEED_EOF_SEND;
		break;
	case SHUT_RDWR:
		bits_to_clear = EXS_NEED_EOF;
		break;
	default:
		assert(0);
	}

	if (!(our_clear_conn_flags(cb, bits_to_clear) & bits_to_clear)) {
		/* Already done */
		errno = EBUSY;
		return -EBUSY;
	}

	TRACE_DBG("cb %p block %p send EOF conn_state %s how %d",
			(void *)cb, (void *)block,
			our_conn_state_str(cb->conn_state), shutdown_how);

	block->send_header->hd_type = htonl(OUR_EOF);
	block->send_header->hd_seqno = htonl(cb->send_seqno);
	block->eof_buf->flags = htonl(shutdown_how);

	/* The EOF message MUST be sent signaled in order to flush the send
	 * completion queue. */
	block->send_wr.send_flags = IBV_SEND_SIGNALED;
	block->send_wr.send_flags |= (send_flags ? *send_flags : 0);
	if ((sizeof(block->send_header) + sizeof(block->eof_buf))
						<= cb->inline_limit) {
		block->send_wr.send_flags |= IBV_SEND_INLINE;
	}

	/* now post send of this advert, then leave */
	STATISTICS_INCREMENT(eof_blocks_sent);
	if (our_post_send(cb, &block->send_wr) < 0) {
		/* got an input/output error, so EOF was NOT SENT. */
		return -1;
	}

	if (send_flags) {
		*send_flags = block->send_wr.send_flags;
	}

	return 0;
}	/* our_send_eof */


#define our_memcpy(dst_buf, dst_offset, src_buf, src_offset, length) \
	do { \
		TRACE_DBG(\
			"memcpy %zu bytes from %p+%#lx to %p+%#lx", \
			(length), (src_buf), (unsigned long)(src_offset), \
			(dst_buf), (unsigned long)(dst_offset)); \
		VALGRIND_MAKE_MEM_DEFINED((src_buf) + (src_offset), (length)); \
		(void)memcpy((dst_buf) + (dst_offset), \
				(src_buf) + (src_offset), \
				(length)); \
	} while (0)


/* Copies up to length bytes from the receive buffer into the given user buffer.
 * Meant to be called directly from exs_recv().  Returns the number of bytes
 * copied, which may be 0, in which case nothing was copied because no data was
 * available.
 *
 * This function expects stream_sync_mutex to be locked, and ***UNLOCKs*** the
 * mutex while it does the copy.  RECV_READY_BIT should therefore *not* be set
 * while this function is being called to prevent another thread from trying to
 * manipulate this block or any subsequent ones. */
static ssize_t
our_stream_copy_recv(struct exs_conn *cb, void *buffer, size_t offset,
		size_t length)
{
	size_t tmp_take_offset;
	size_t copy_length;
	size_t chunk_length;
	size_t padded_length;

	if (cb->recv_buf.n_full == 0) {
		return 0;
	}

	copy_length = length - offset;
	if (copy_length > *cb->recv_buf.next_entry_primed) {
		TRACE_DBG("copy_length=%zu > next_entry_primed=%u",
			copy_length, *cb->recv_buf.next_entry_primed);
		copy_length = *cb->recv_buf.next_entry_primed;
	}
	tmp_take_offset = cb->recv_buf.take_offset;
	chunk_length = cb->recv_buf.buf.size - tmp_take_offset;
	OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
	if (copy_length <= chunk_length) {
		our_memcpy(buffer, offset, cb->recv_buf.buf.ptr,
				tmp_take_offset, copy_length);
	} else {
		/* We know that all of the requested data is present, but
		 * we must copy it in two chunks since we must "wrap around" the
		 * end of the buffer.  chunk_length is the length of the first
		 * chunk. */
		our_memcpy(buffer, offset, cb->recv_buf.buf.ptr,
				tmp_take_offset, chunk_length);
		our_memcpy(buffer, offset + chunk_length,
				cb->recv_buf.buf.ptr, 0,
				copy_length - chunk_length);
	}
	OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);

	/* Update loop/buffer variables */
	cb->recv_buf.n_full -= copy_length;
	padded_length = (cb->fd_flags & EXS_FD_STREAM_BUF_NOALIGN) ? copy_length
		: our_aligned_size(copy_length, our_cache_line_size);
	cb->recv_buf.n_empty += padded_length;
	cb->recv_buf.take_offset += padded_length;
	if (cb->recv_buf.take_offset >= cb->recv_buf.buf.size) {
		/* May be greater in case of wrap-around */
		cb->recv_buf.take_offset -= cb->recv_buf.buf.size;
	}

	cb->recv_buf.next_entry_primed++;
	if (cb->recv_buf.next_entry_primed
			>= cb->recv_buf.full_bytes_array_end) {
		cb->recv_buf.next_entry_primed
			= cb->recv_buf.full_bytes_array;
	}

	return copy_length;
}	/* our_stream_copy_recv */


/* Satisfies all pending recv blocks which may be satisfied by data in the
 * receive buffer.  The stream_sync_mutex for the connection MUST be locked
 * before calling this function. */
static int
our_stream_recv_loop(struct exs_conn *cb)
{
	struct transaction_block *first, *ablock;
	ssize_t copied_length;
	int flags;
	int contiguous_event_posts;
	int errcode;
	int ret = 0;

	/* Satisfy any receives we can with this new chunk of data */
	first = ablock = cb->next_recv_primed;

	contiguous_event_posts = 1;
	while (ablock->send_state & RECV_DONE_BIT) {
		if (contiguous_event_posts && (!(ablock->send_state & ADVERT_SENDING_BIT)
				|| (ablock->send_state & SEND_DONE_BIT))) {
			ablock->send_state = 0;

			our_exs_recv_done(cb, ablock);
			cb->next_recv_primed = ablock->next;
		} else {
			contiguous_event_posts = 0;
		}

		ablock = ablock->next;
		if (ablock == first) {
			goto ack;
		}
	}

	while (cb->recv_buf.n_full > 0 && (ablock->send_state & RECV_READY_BIT)) {
		assert(!(ablock->send_state & RECV_DONE_BIT));

		/* our_stream_copy_recv ***UNLOCKS*** stream_sync_mutex!!!
		 * We clear RECV_READY_BIT to ensure that another thread
		 * entering this function will not process this block or any
		 * subsequent blocks. */
		atomic_clear_bits(RECV_READY_BIT, &ablock->send_state);
		copied_length = our_stream_copy_recv(cb,
				ablock->user_buffer,
				ablock->recv_byte_len,
				ablock->user_length);
		atomic_set_bits(RECV_READY_BIT, &ablock->send_state);
		if (copied_length <= 0) {
			/* Nothing was copied either because nothing was
			 * available or MSG_WAITALL was set */
			goto ack;
		}

		/* Update the received length */
		ablock->recv_byte_len += copied_length;

		if ((ablock->flags & MSG_WAITALL)
				&& (cb->conn_state == EXS_CONN_RUNNING)
				&& (ablock->recv_byte_len
					< ablock->user_length)) {
			/* We were only able to partially fulfill the request
			 * because the request was larger than the stream buffer
			 * size. */
			continue;
		}

		/* Update the block state since the block was satisfied */
		if ((ablock->send_state & ADVERT_SENDING_BIT)) {
			assert(ntohll(ablock->advert_buf->ad_buf_seqno)
							<= cb->recv_seqno);
			if (!(ablock->flags & MSG_WAITALL)) {
				cb->source_seqno += ablock->recv_byte_len - 1;
			}
		} else {
			cb->source_seqno += ablock->recv_byte_len;
		}
		cb->recv_seqno += ablock->recv_byte_len;
		flags = atomic_set_bits(RECV_DONE_BIT, &ablock->send_state);
		if (contiguous_event_posts && (!(flags & ADVERT_SENDING_BIT)
						|| (flags & SEND_DONE_BIT))) {
			ablock->send_state = 0;

			our_exs_recv_done(cb, ablock);
			cb->next_recv_primed = ablock->next;
		} else {
			contiguous_event_posts = 0;
		}

		ablock = ablock->next;
		if (ablock == first) {
			goto ack;
		}
	}

	if (cb->recv_buf.n_full > 0 || (cb->fd_flags & EXS_FD_NO_STREAM_ADVERT)) {
		goto ack;
	} else if (cb->conn_state != EXS_CONN_RUNNING
						&& cb->recv_buf.n_full == 0) {
		/* We are in the process of shutting down and have just drained
		 * the buffer, so prevent issuing any new exs_recv() operations
		 * now. */
		TRACE_DBG("this_conn %p conn_state %s unset EXS_OK_TO_ISSUE_RECV",
				(void *)cb, our_conn_state_str(cb->conn_state));
		atomic_clear_bits(EXS_OK_TO_ISSUE_RECV, &cb->conn_flags);
		goto ack;
	}

	while (ablock->send_state & RECV_READY_BIT) {
		assert(!(ablock->send_state & RECV_DONE_BIT));
		if ((ablock->send_state & ADVERT_SENDING_BIT)
				&& (serial_less(ntohl(
					ablock->advert_buf->ad_buf_phase),
					cb->recv_phase))) {
			goto ack;
		} else if (!(ablock->send_state & ADVERT_SENDING_BIT)) {
			errcode = our_send_advert(cb, ablock);
			if (errcode < 0 && errno != ECONNRESET) {
				OUR_PERROR(errno, "Could not send ADVERT");
				ret = -1;
				goto ack;
			} else if (!(ablock->send_state & ADVERT_SENDING_BIT)) {
				goto ack;
			}
		}
		ablock = ablock->next;
		if (ablock == first) {
			goto ack;
		}
	}

ack:
	/* Send an ACK if we need to */
	flags = 0;
	if ((cb->recv_buf.n_empty > cb->recv_buf.n_empty_threshold)
			|| (cb->n_unacked_wwi
				> cb->n_unacked_wwi_threshold)) {
		errcode = our_send_ack(cb, &flags);
		if (errcode < 0 && errno != ECONNRESET) {
			OUR_PERROR(errno, "Could not send ACK");
			ret = -1;
		}
	}

	return ret;
}	/* our_stream_recv_loop */

static int
our_handle_recv_write_notify_seqpacket(struct exs_conn *cb,
		uint32_t byte_len, uint32_t imm_data)
{
	struct transaction_block *this;
	(void)imm_data;

	this = cb->next_recv_primed;
	assert(!(this->send_state & RECV_DONE_BIT));
	this->recv_byte_len = byte_len;
	VALGRIND_MAKE_MEM_DEFINED(this->user_buffer, byte_len);
	if (atomic_set_bits(RECV_DONE_BIT, &this->send_state) & SEND_DONE_BIT) {
		if (!cb->stream_advert_imbalance) {
			/* TODO: fix error reporting */
			our_exs_recv_done(cb, this);
			cb->next_recv_primed = this->next;
			return 0;
		}
	} else {
		/* Mark that we have a block that has received data but not the
		 * SEND ADVERT completion, so that we do not post future RECV
		 * completions before this one.  This is decremented by the send
		 * ADVERT completion handler.
		 *
		 * nonatomic increment is ok since this variable is only ever
		 * adjusted/used from the completion thread */
		++cb->stream_advert_imbalance;
	}
	cb->next_recv_primed = this->next;
	return 0;
}	/* our_handle_recv_write_notify_seqpacket */

static int
our_handle_recv_write_notify_stream(struct exs_conn *cb,
		uint32_t byte_len, uint32_t imm_data)
{
	struct transaction_block *this, *ptr;
	unsigned int imm_flags, wwi_phase, advert_ack_seqno;
	int ret, send_flags, diff;

	imm_flags = (imm_data & IMM_FLAG_MASK);
	wwi_phase = (imm_data & IMM_PHASE_MASK) >> IMM_PHASE_SHIFT;
	advert_ack_seqno = (imm_data & IMM_SEQNO_MASK) >> IMM_SEQNO_SHIFT;

	TRACE_DBG("got WWI phase %" PRIu32 " (recv phase %u) recv_seqno %" PRIu64 " direct=%" PRIx32,
			wwi_phase, cb->recv_phase,
			cb->recv_seqno, imm_flags & IMM_DIRECT_BIT);

	diff = (advert_ack_seqno - cb->last_unacked_advert_msg_seqno)
					& (IMM_SEQNO_MASK >> IMM_SEQNO_SHIFT);
	if (diff > 0) {
		cb->last_unacked_advert_msg_seqno = advert_ack_seqno;
		/* FIXME: Ensure greater than 0 */
		if (cb->stream_advert_imbalance < -diff) {
			atomic_add(diff, &cb->stream_advert_counter);
		} else if (cb->stream_advert_imbalance < 0) {
			atomic_sub(cb->stream_advert_imbalance,
						&cb->stream_advert_counter);
		}
		assert(cb->stream_advert_counter <= cb->posted_credits);
		cb->stream_advert_imbalance += diff;
	}

	if (imm_flags & IMM_DIRECT_BIT) {
		this = ptr = cb->next_recv_primed;
		/* Skip already completed transaction blocks */
		while (this->send_state & RECV_DONE_BIT) {
			this = this->next;
			assert(this != ptr);
		}
		assert(cb->recv_buf.n_full == 0);
		assert(wwi_phase == (cb->recv_phase
					& (IMM_PHASE_MASK >> IMM_PHASE_SHIFT)));
		VALGRIND_MAKE_MEM_DEFINED((uint8_t *)(this->user_buffer)
				+ this->recv_byte_len, this->recv_byte_len);
		this->recv_byte_len += byte_len;
		send_flags = 0;

		OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
		assert(this->send_state & ADVERT_SENDING_BIT);
		assert(ntohll(this->advert_buf->ad_buf_seqno) <= cb->recv_seqno);
		cb->recv_seqno += byte_len;
		if (!(this->flags & MSG_WAITALL)) {
			cb->source_seqno += byte_len - 1;
		}
		++cb->n_unacked_wwi;
		if (cb->n_unacked_wwi > cb->n_unacked_wwi_threshold) {
			/* We will free the transaction block after we
			 * get the SEND completion for this ACK */
			int ret = our_send_ack(cb, &send_flags);
			if (ret < 0 && errno != ECONNRESET) {
				OUR_PERROR(errno, "Could not send ACK");
			}
		}
		OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);

		if (!(this->flags & MSG_WAITALL)
				|| this->recv_byte_len >= this->user_length) {
			OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
			atomic_set_bits(RECV_DONE_BIT, &this->send_state);
			ret = our_stream_recv_loop(cb);
			OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
			return 0;
		} else {
			return 0;
		}
	} else {
		OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
		if (PHASE_IS_DIRECT(cb->recv_phase)) {
			++cb->recv_phase;
			TRACE_DBG("got indirect WWI set recv_phase to %u",
				cb->recv_phase);
		}
		cb->recv_buf.n_full += byte_len;
		STATISTICS_SET_MAX(max_recv_buf_n_full, cb->recv_buf.n_full);
		*cb->recv_buf.next_entry = byte_len;
		cb->recv_buf.next_entry++;
		if (cb->recv_buf.next_entry
				>= cb->recv_buf.full_bytes_array_end) {
			cb->recv_buf.next_entry
				= cb->recv_buf.full_bytes_array;
		}
		++cb->n_unacked_wwi;
		ret = our_stream_recv_loop(cb);
		OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);

		return ret;
	}
}	/* our_handle_recv_write_notify_stream */


/* Clean up recved ACKs. */
static int
our_handle_recv_ack(struct exs_conn *cb, uint64_t seqno,
		struct our_stream_ack *recv_ack,
		struct transaction_block *recved)
{
	int ret;
	uint32_t new_n_empty = ntohl(recv_ack->recv_buf_empty);
	unsigned int n_acks = ntohl(recv_ack->recv_buf_ack_count);
	unsigned int old_remote_buf_n_empty, old_wwi_counter;
	(void)seqno;

	TRACE_DBG("received ACK seqno %" PRIu64 " empty %" PRIu32 " ack_count %u",
		seqno, new_n_empty, n_acks);

	ret = our_repost_recv_block(cb, recved);

	/* Process data buffer ACK */
	OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
	old_remote_buf_n_empty = cb->remote_buf_n_empty;
	cb->remote_buf_n_empty += new_n_empty;
	old_wwi_counter = cb->stream_wwi_counter;
	cb->stream_wwi_counter += n_acks;
	if (n_acks) {
		STATISTICS_INCREMENT(recv_ack_buf_completions);
	}
	OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
	if ((old_remote_buf_n_empty == 0 && new_n_empty > 0)
			|| (old_wwi_counter == 0 && n_acks > 0)) {
		if (cb->ops->xmit_if_match(cb, cb->next_send_primed) < 0) {
			ret = -1;
		}
	}

	return ret;
}	/* our_handle_recv_ack */


static int
our_handle_send_eof(struct exs_conn *this_conn,
			struct transaction_block *block,
			struct our_eof *eof_buf)
{
	(void)this_conn;
	(void)block;
	(void)eof_buf;

	return 0;
}	/* our_handle_send_eof */


static int
our_shutdown_finished(struct exs_conn *this_conn,
			struct transaction_block *block,
			int how)
{
	struct exs_event *evt;
	int ret;
	(void)this_conn;
	(void)how;

	assert(!(block->flags & EXS_NOTIFY_ALREADY_DONE));

	evt = block->user_event_ptr;
	block->user_event_ptr = NULL;
	/* ensure block->user_event_ptr is NULL prior to event posting */
	memory_barrier();
	ret = _put_full_evt_queue_event(block->qhandle, evt);

	/* mark that this block has notified the user */
	block->flags |= EXS_NOTIFY_ALREADY_DONE;
	block->in_progress = 0;

	return ret;
}	/* our_shutdown_finished */


static int
our_handle_recv_eof(struct exs_conn *cb,
			struct transaction_block *block,
			struct our_eof *eof_buf)
{
	int ret;
	unsigned int flags;
	int send_flags;
	int old_conn_flags;

	flags = ntohl(eof_buf->flags);
	TRACE_DBG("start cb %p flags %#x", (void *)cb, flags);

	OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);
	switch (flags) {
	case SHUT_RDWR:
		old_conn_flags = our_clear_conn_flags(cb, EXS_OK_TO_ISSUE_IO);
		if (!(old_conn_flags & EXS_NEED_EOF)) {
			/* If we previously sent an EOF message, fire off the
			 * disconnect now */
			if (our_clear_conn_flags(cb, EXS_NEED_DISCONNECT)
							& EXS_NEED_DISCONNECT) {
				ret = our_rdma_disconnect(cb);
			} else {
				ret = 0;
			}
		} else {
			ret = our_repost_recv_block(cb, block);
			/* Send reply, since we believe that we have not sent
			 * EOF yet. */
			send_flags = IBV_SEND_SIGNALED;
			ret = our_send_eof(cb, &cb->eof_block_array[1],
							&send_flags, flags);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
		}
		if (!ret && cb->eof_block_array[0].in_progress) {
			ret = our_shutdown_finished(cb,
						&cb->eof_block_array[0],
						SHUT_RD);
		} else if (!ret && cb->eof_block_array[1].in_progress) {
			ret = our_shutdown_finished(cb,
						&cb->eof_block_array[1],
						SHUT_RD);
		}
		break;
	case SHUT_RD:
		/* other side shutdown for reading, so we must shutdown for
		 * writing */
		old_conn_flags = our_clear_conn_flags(cb, EXS_OK_TO_ISSUE_SEND);
		ret = our_repost_recv_block(cb, block);
		if ((old_conn_flags & EXS_NEED_EOF) == EXS_NEED_EOF) {
			send_flags = IBV_SEND_SIGNALED;
			ret = our_send_eof(cb, &cb->eof_block_array[0],
							&send_flags, SHUT_WR);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
		} else if ((old_conn_flags & EXS_NEED_EOF)
							== EXS_NEED_EOF_SEND) {
			send_flags = IBV_SEND_SIGNALED;
			ret = our_send_eof(cb, &cb->eof_block_array[1],
							&send_flags, SHUT_WR);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
		} else {
			if (!(cb->conn_flags & (EXS_NEED_EOF))
					&& (our_clear_conn_flags(cb,
						EXS_NEED_DISCONNECT)
						& EXS_NEED_DISCONNECT)) {
				our_rdma_disconnect(cb);
			}
			if (cb->eof_block_array[0].in_progress) {
				ret = our_shutdown_finished(cb,
							&cb->eof_block_array[0],
							SHUT_WR);
			} else if (cb->eof_block_array[1].in_progress) {
				ret = our_shutdown_finished(cb,
							&cb->eof_block_array[1],
							SHUT_WR);
			} else {
				abort();
			}
		}
		break;
	case SHUT_WR:
		/* other side shutdown for writing, so we must shutdown for
		 * reading */
		old_conn_flags = our_clear_conn_flags(cb, EXS_OK_TO_ISSUE_RECV);
		ret = our_repost_recv_block(cb, block);
		if (cb->pending_recv_count > 0) {
			our_flush_all_stranded_recv_blocks(cb);
		}
		if ((old_conn_flags & EXS_NEED_EOF) == EXS_NEED_EOF) {
			send_flags = IBV_SEND_SIGNALED;
			ret = our_send_eof(cb, &cb->eof_block_array[0],
							&send_flags, SHUT_RD);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
		} else if ((old_conn_flags & EXS_NEED_EOF)
							== EXS_NEED_EOF_RECV) {
			send_flags = IBV_SEND_SIGNALED;
			ret = our_send_eof(cb, &cb->eof_block_array[1],
							&send_flags, SHUT_RD);
			if (ret < 0 && errno == EBUSY) {
				ret = 0;
			}
		} else {
			if (!(cb->conn_flags & (EXS_NEED_EOF))
					&& (our_clear_conn_flags(cb,
						EXS_NEED_DISCONNECT)
						& EXS_NEED_DISCONNECT)) {
				our_rdma_disconnect(cb);
			}
			if (cb->eof_block_array[0].in_progress) {
				ret = our_shutdown_finished(cb,
							&cb->eof_block_array[0],
							SHUT_RD);
			} else if (cb->eof_block_array[1].in_progress) {
				ret = our_shutdown_finished(cb,
							&cb->eof_block_array[1],
							SHUT_RD);
			} else {
				abort();
			}
		}
		break;
	default:
		ret = our_repost_recv_block(cb, block);
		TRACE_ERR("Got unexpected flags for remote exs_shutdown: %#x\n",
				flags);
		break;
	}
	OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);

	return ret;
}	/* our_handle_recv_eof */


static int
our_handle_send_ack(struct exs_conn *cb, struct transaction_block *block)
{
	int flags;
	int ret;

	ret = 0;
	if (block->unsignaled_count > 0) {
		OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
		assert(cb->stream_ack_counter + block->unsignaled_count
							<= cb->posted_credits);
		cb->stream_ack_counter = block->unsignaled_count;
		if ((cb->recv_buf.n_empty > cb->recv_buf.n_empty_threshold)
				|| (cb->n_unacked_wwi
					> cb->n_unacked_wwi_threshold)) {
			flags = 0;
			if (our_send_ack(cb, &flags) < 0
							&& errno != ECONNRESET) {
				OUR_PERROR(errno, "Could not send ACK");
				ret = -1;
			}
		}
		OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);
	}

	exs_p_list_add_tail(&cb->send_ack_empty_list, block);

	return ret;
}	/* our_handle_send_ack */


static int
our_handle_send_advert_seqpacket(struct exs_conn *cb,
			struct transaction_block *block)
{
	int ret;
	assert(!(block->send_state & SEND_DONE_BIT));
	ret = 0;
	if (atomic_set_bits(SEND_DONE_BIT, &block->send_state) & RECV_DONE_BIT) {
		if (cb->stream_advert_imbalance) {
			--cb->stream_advert_imbalance;
		}
		do {
			/* TODO: fix error reporting */
			ret = our_exs_recv_done(cb, block);
			block = block->next;
		} while (block != cb->next_recv_primed
				&& (block->send_state & (SEND_DONE_BIT|RECV_DONE_BIT))
					== (SEND_DONE_BIT|RECV_DONE_BIT));
	}
	return ret;
}	/* our_handle_send_advert_seqpacket */

static int
our_handle_send_advert_stream(struct exs_conn *cb,
			struct transaction_block *block)
{
	int ret;

	TRACE_DBG("cb %p stream_advert_counter %u got advert block %p unsignaled_count %u",
			(void *)cb, cb->stream_advert_counter, (void *)block,
			block->unsignaled_count);

	OUR_MEMBER_MUTEX_LOCK(cb, stream_sync_mutex);
	if (block->unsignaled_count > 0) {
		/* FIXME: Ensure atomic increment value is always positive */
		if (cb->stream_advert_imbalance > (int)block->unsignaled_count) {
			atomic_add(block->unsignaled_count,
						&cb->stream_advert_counter);
		} else if (cb->stream_advert_imbalance > 0) {
			atomic_add(cb->stream_advert_imbalance,
						&cb->stream_advert_counter);
		}
		assert(cb->stream_advert_counter <= cb->posted_credits);
		cb->stream_advert_imbalance -= block->unsignaled_count;

	}
	/* This bit will be checked later while holding stream_sync_mutex mutex,
	 * so no need to atomically test the result */
	atomic_set_bits(SEND_DONE_BIT, &block->send_state);
	ret = our_stream_recv_loop(cb);
	OUR_MEMBER_MUTEX_UNLOCK(cb, stream_sync_mutex);

	return ret;
}	/* our_handle_send_advert_stream */


static int
our_dispatch_send_control_completion(struct exs_conn *cb,
		__attribute__((unused)) const struct ibv_wc *wc,
		struct transaction_block *block)
{
	enum our_message_header_types rtype;
	int err;

	/* a previously posted send just completed */
	rtype = ntohl(block->send_header->hd_type) & HEADER_TYPE_MASK;
	TRACE_DBG("wc opcode IBV_WC_SEND seqno %" PRIu32 " type %" PRIu32,
			ntohl(block->send_header->hd_seqno), rtype);

	switch (rtype) {
	case OUR_ADVERTISEMENT:
		STATISTICS_INCREMENT(send_advert_completions);
		if (cb->ops->handle_send_advert(cb, block) < 0) {
			return -1;
		}
		break;

	case OUR_WRITE_NOTIFY:
		/* this is an iWARP WRITE notification */
		TRACE_DBG("writeack_header.hd_seqno = %d",
				ntohl(block->send_header->hd_seqno));

		cb->ops->handle_send_write(cb, block);
		break;

	case OUR_SETUP_REQUEST:
		/* should only happen on client */
		assert(cb->conn_state == EXS_CONN_CLIENT_SETUP
				&& cb->nego_out_ptr != NULL);
		STATISTICS_INCREMENT(total_setup_completions);
		if ((err = our_client_finishing(cb, SEND_DONE_BIT)) != 0) {
			OUR_PERROR(err, "negotiation failed");
			return -1;
		}
		break;

	case OUR_SETUP_REPLY:
		/* should only happen on agent */
		assert(cb->conn_state == EXS_CONN_AGENT_SETUP
						&& cb->nego_out_ptr != NULL);
		STATISTICS_INCREMENT(total_setup_completions);
		if ((err = our_pending_accept_done(cb)) != 0) {
			OUR_PERROR(err, "negotiation failed");
			return -1;
		}
		break;

	case OUR_ACK:
		TRACE_DBG("cb %p stream_ack_counter %u got ack block %p unsignaled_count %u",
				(void *)cb, cb->stream_ack_counter,
				(void *)block, block->unsignaled_count);
		STATISTICS_INCREMENT(send_ack_completions);
		if (our_handle_send_ack(cb, block) < 0) {
			return -1;
		}
		break;

	case OUR_EOF:
		our_handle_send_eof(cb, block, block->eof_buf);
		break;

#if defined(ALLOW_PARANOID_CHECKING)
	default:
		/* should never happen */
		TRACE_ERR("block %p has illegal type %d",
				(void *)block, rtype);
		return -1;
#endif
	}	/* switch(rtype) */

	return 0;
}	/* our_dispatch_send_control_completion */


static int
our_dispatch_send_data_completion(struct exs_conn *cb,
		__attribute__((unused)) const struct ibv_wc *wc,
		struct transaction_block *block)
{
	STATISTICS_INCREMENT(send_rdma_write_completions);
	TRACE_DBG("IBV_WC_RDMA_WRITE imm_data %" PRIu32 " wc_flags %" PRIx32,
		ntohl(wc->imm_data), wc->wc_flags);
	return cb->ops->handle_send_write(cb, block);
}	/* our_dispatch_send_data_completion */


/* block is *not* the block affected by the received immediate data!  It is just
 * the receive WR that was consumed by receiving the WWI, and we need it so we
 * can repost this receive WR. */
static int
our_dispatch_recv_data_completion(struct exs_conn *cb, const struct ibv_wc *wc,
		struct transaction_block *block)
{
	uint32_t byte_len = wc->byte_len;
	uint32_t imm_data = ntohl(wc->imm_data);
	int ret;

	STATISTICS_INCREMENT(recv_rdma_with_imm_completions);
	STATISTICS_ADD(recv_rdma_with_imm_bytes, byte_len);

	/* imm_data is in network byte order
	 * all other work completion fields are in host byte order */
	TRACE_DBG("IBV_WC_RECV_RDMA_WITH_IMM byte_len %" PRIu32 " imm_data %" PRIu32,
		byte_len, imm_data);

	ret = cb->ops->handle_recv_write_notify(cb, byte_len, imm_data);
	if (ret < 0) {
		return ret;
	}

	ret = our_repost_recv_block(cb, block);
	if (ret) {
		errno = ret;
		return -1;
	} else {
		return 0;
	}
}	/* our_dispatch_recv_data_completion */


static inline int
our_check_recv_size(size_t actual, size_t expected, const char *message)
{
	if (actual != expected) {
		TRACE_ERR("%s: received %zu bytes, expected %zu bytes",
				message, actual, expected);
		return -1;
	} else {
		return 0;
	}
}	/* check_msg_size */


static int
our_dispatch_recv_completion(struct exs_conn *cb, const struct ibv_wc *wc,
		struct transaction_block *block)
{
	enum our_message_header_types rtype;
	struct our_notify *notify_ptr;
	size_t byte_len = wc->byte_len;
	int err;

	/* send_header gets newly recvd block's hd_seqno
	 *	in network byte order
	 * recv_header gets newly recvd block's hd_seqno
	 *	in host byte order
	 */
	VALGRIND_MAKE_MEM_DEFINED(block->recv_wr.sg_list[0].addr, byte_len);
	block->recv_header->hd_seqno = ntohl(block->recv_header->hd_seqno);
	block->recv_header->hd_type = ntohl(block->recv_header->hd_type);
	rtype = (enum our_message_header_types)
		(block->recv_header->hd_type & HEADER_TYPE_MASK);
	TRACE_DBG("IBV_WC_RECV byte_len %zu hd_seqno %" PRIu32 " hd_type %" PRIu32,
			byte_len, block->recv_header->hd_seqno, rtype);

	/* We must copy data out of this block *before* it gets reposted, and we
	 * also must defensively check received messages for protocol errors. */
	switch(rtype) {
	case OUR_ADVERTISEMENT:
		if (our_check_recv_size(byte_len, sizeof(block->send_header)
				+ sizeof(*block->advert_buf),
				"recv ADVERT completion")) {
			return -1;
		}
		if (cb->ops->handle_recv_advert(cb, block) < 0) {
			return -1;
		}
		break;

	case OUR_SETUP_REQUEST:
		/* should only happen on agent */
		if (our_check_recv_size(byte_len, sizeof(block->send_header)
				+ sizeof(struct our_nego_mess),
				"recv SETUP_REQUEST completion")) {
			return -1;
		}
		if (cb->conn_state != EXS_CONN_AGENT_SETUP
						|| cb->nego_out_ptr == NULL) {
			/* should never happen */
			TRACE_ERR("block %p has type %d, but state is %s, expected %s",
				(void *)block, rtype,
				our_conn_state_str(cb->conn_state),
				our_conn_state_str(EXS_CONN_AGENT_SETUP));
			return -1;
		}
		STATISTICS_INCREMENT(total_setup_completions);
		cb->nego_in_ptr = block;
		if ((err = our_agent_finishing(cb)) != 0) {
			OUR_PERROR(err, "negotiation failure");
			return -1;
		}
		our_repost_recv_block(cb, block);
		break;

	case OUR_SETUP_REPLY:
		/* should only happen on client */
		if (our_check_recv_size(byte_len, sizeof(block->send_header)
				+ sizeof(struct our_nego_mess),
				"recv SETUP_REPLY completion")) {
			return -1;
		}
		if (cb->conn_state != EXS_CONN_CLIENT_SETUP
						|| cb->nego_out_ptr == NULL) {
			/* should never happen */
			TRACE_ERR("block %p has type %d, but state is %s, expected %s",
				(void *)block, rtype,
				our_conn_state_str(cb->conn_state),
				our_conn_state_str(EXS_CONN_CLIENT_SETUP));
			return -1;
		}
		STATISTICS_INCREMENT(total_setup_completions);
		cb->nego_in_ptr = block;
		if ((err = our_client_finishing(cb, ACK_DONE_BIT)) != 0) {
			OUR_PERROR(err, "negotiation failure");
			return -1;
		}
		our_repost_recv_block(cb, block);
		break;

	case OUR_WRITE_NOTIFY:
		if (our_check_recv_size(byte_len, sizeof(*block->send_header)
				+ sizeof(*block->writeack_buf),
				"recv WRITE_NOTIFY completion")) {
			return -1;
		}
		notify_ptr = (struct our_notify *)block->recv_buf;
		TRACE_DBG("Got WRITE_NOTIFY index=%" PRIu32,
				ntohl(notify_ptr->write_index));
		cb->ops->handle_recv_write_notify(cb,
				ntohl(notify_ptr->write_len),
				(ntohl(notify_ptr->write_index) & ~IMM_FLAG_MASK) |
				(ntohl(notify_ptr->write_flags) & IMM_FLAG_MASK));
		our_repost_recv_block(cb, block);
		break;

	case OUR_ACK:
		if (our_check_recv_size(byte_len, sizeof(block->send_header)
				+ sizeof(*block->ack_buf),
				"recv ACK completion")) {
			return -1;
		}
		STATISTICS_INCREMENT(recv_ack_completions);

		if (our_handle_recv_ack(cb, block->recv_header->hd_seqno,
				(struct our_stream_ack *)block->recv_buf,
				block) < 0) {
			return -1;
		}
		break;

	case OUR_EOF:
		if (our_check_recv_size(byte_len, sizeof(block->send_header)
				+ sizeof(*block->eof_buf),
				"recv EOF completion")) {
			return -1;
		}
		STATISTICS_INCREMENT(recv_eof_completions);
		our_handle_recv_eof(cb, block,
				(struct our_eof *)block->recv_buf);
		break;

	default:
		/* should never happen */
		TRACE_ERR("received block %p has illegal type %d",
				(void *)block, rtype);
		return -1;
	}

	return 0;
}	/* our_dispatch_recv_completion */


static int
our_dispatch_completion(const struct ibv_wc *wc, struct exs_conn *cb)
{
	struct transaction_block *block;
	int			ret = -1;	/* assume failure */
	int			err;

	/* once in the critical section, do not allow
	 * this thread to be cancelled
	 */
	err = pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
	if (err) {
		OUR_PERROR(err, "pthread_setcancelstate(PTHREAD_CANCEL_DISABLE)");
	}

	/* get transaction block for this completion */
	block = (void *)(uintptr_t)wc->wr_id;
	assert(block != NULL);
	TRACE_DBG("wc->wr_id is block %p, event %p",
			(void *)block, (void *)block->user_event_ptr);

	TRACE_DBG("got ibv_poll_cq opcode %d, expected opcode %d",
			wc->opcode, block->expected_comp);
	STATISTICS_INCREMENT(good_completions);

	assert(wc->status == IBV_WC_SUCCESS);
	switch (wc->opcode)
	{
	case IBV_WC_SEND:	/* 0 */
		ret = our_dispatch_send_control_completion(cb, wc, block);
		break;

	case IBV_WC_RECV:	/* 128 */
		ret = our_dispatch_recv_completion(cb, wc, block);
		break;

	case IBV_WC_RECV_RDMA_WITH_IMM:
		ret = our_dispatch_recv_data_completion(cb, wc, block);
		break;

	case IBV_WC_RDMA_WRITE:		/* 1 */
		ret = our_dispatch_send_data_completion(cb, wc, block);
		break;

	default:
		/* should never happen */
		TRACE_ERR("Got unknown wc->opcode = %d", wc->opcode);
		ret = -1;
		break;
	}	/* switch expected_comp */

	/* end of critical section, allow cancellation again */
	err=pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
	if (err) {
		OUR_PERROR(err, "pthread_setcancelstate(PTHREAD_CANCEL_ENABLE)");
	}

	return ret;
}	/* our_dispatch_completion */


static void
our_handle_bad_completion(struct ibv_wc *wc, struct exs_conn *cb)
{
	struct transaction_block *block;

	block = (wc->wr_id == 0) ? NULL
		: ((struct transaction_block *)(uintptr_t)block);
	if (wc->status == IBV_WC_WR_FLUSH_ERR) {
		TRACE_DBG("fd %d IBV_WC_WR_FLUSH_ERR", cb->sockno);
	} else {
		TRACE_ERR("ibv_poll_cq returned: %s",
				ibv_wc_status_str(wc->status));
		if (block) {
			TRACE_ERR("expected completion type %d",
				block->expected_comp);
			TRACE_ERR("send header type %" PRIu32,
				block->send_header->hd_type);
		}
		if (!cb->is_seqpacket) {
			TRACE_ERR("stream_wwi_counter %u",
				cb->stream_wwi_counter);
			TRACE_ERR("stream_advert_counter %u",
				cb->stream_advert_counter);
			TRACE_ERR("stream_ack_counter %u",
				cb->stream_ack_counter);
			TRACE_ERR("n_unacked_wwi %u",
				cb->n_unacked_wwi);
		}
	}
}	/* our_handle_bad_completion */


/* Waits for something to arrive on the completion queue for the given
 * connection.  Expects that the completion channel for the connection has
 * already been armed.  Will rearm the completion channel prior to exiting.
 * Returns the value returned by ibv_poll_cq() on success, or the negative value
 * of errno on failure.  Returns -EINVAL if the completion channel returns a
 * completion event for the wrong CQ. */
static int
our_poll_cq_wait_notify(struct exs_conn *cb)
{
	struct ibv_cq		*event_cq;
	void			*event_ctx;
	int			ret;

	do {
		/* wait for cq event */
		if ((ret = ibv_get_cq_event(cb->comp_channel,
						&event_cq, &event_ctx))) {
			OUR_PERROR(ret, "ibv_get_cq_event(%p) on cb %p failed",
					(void *)cb->comp_channel, (void *)cb);
			return -ret;
		}
		STATISTICS_INCREMENT(good_ibv_get_cq_events);

		if(event_cq != cb->cq)
		{
			TRACE_ERR("got event_cq = %p on cb %p, but expected cb->cq = %p",
					(void *)event_cq, (void *)cb,
					(void *)cb->cq);
			return -EINVAL;
		}
		TRACE_DBG("got event_cq = cb->cq = %p on channel %p",
				(void *)cb->cq, (void *)cb->comp_channel);

		/* accumulate number of cq events that need to
		 * be acked, and periodically ack them
		 */
		cb->n_cq_events_that_need_ack++;
		if (cb->n_cq_events_that_need_ack == UINT_MAX) {
			ibv_ack_cq_events(cb->cq, UINT_MAX);
			cb->n_cq_events_that_need_ack = 0;
		}
		/* immediately set up completion queue
		 * notificaton again so that when next entry of
		 * any type is added to the completion queue, an
		 * event will be triggered on the completion
		 * channel
		 */
		ret = ibv_req_notify_cq(cb->cq, 0);
		if(ret)
		{
			OUR_PERROR(ret, "failed ibv_req_notify_cq on cb %p",
					(void *)cb);
			return -ret;
		}
	} while ((ret = ibv_poll_cq(cb->cq, cb->max_n_wc_slots,
					cb->wc_array)) == 0);

	if (ret < 0) {
		OUR_PERROR(-ret, "failed ibv_poll_cq on cb %p",
				(void *)cb);
	}

	return ret;
}	/* our_poll_cq_wait_notify */


/* Busy polls for something to arrive on the completion queue for the given
 * connection.  Returns the nonzero value returned by ibv_poll_cq() (if it
 * returns 0 indefinitely, this function will not return). */
static int
our_poll_cq_busy_poll(struct exs_conn *cb)
{
	int			ret;

	while ((ret = ibv_poll_cq(cb->cq, cb->max_n_wc_slots,
					cb->wc_array)) == 0)
		/* do nothing */;

	if (ret < 0) {
		OUR_PERROR(-ret, "failed ibv_poll_cq on cb %p",
				(void *)cb);
	}

	return ret;
}	/* our_poll_cq_busy_poll */


static void
our_completion_thread_wait_active(struct exs_conn *cb)
{
	pthread_cleanup_push(our_completion_cancel_handler, cb);
	OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);

	while( !(cb->conn_flags & EXS_NEED_DISCONNECT)) {
		pthread_cond_wait(&cb->transmit_ok_notify, &cb->event_mutex);
	}

	OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
	pthread_cleanup_pop(0);		/* i.e., do NOT do the unlock */
}	/* our_completion_thread_wait_active */


/* thread to catch completion events on connection "arg"
 *
 * while waiting or polling for the next completion event on this connection,
 *	no mutexes are held
 *
 * this thread initially can be cancelled by our_cancel_completion_thread(),
 * which will first lock event_mutex to ensure no race with this thread

 *	The arg passed in at creation is a pointer to the cb of
 *	either a client connection or an agent connection.
 *	It is never the cb of a listener connection.
 *
 */
static void *
our_completion_thread(void *arg)
{
	struct exs_conn		*cb = arg;
	struct ibv_wc		*wc;
	struct ibv_wc		*wc_limit;
	int			count, do_busy_poll;

	TRACE_DBG("cb %p enter, cb->fd_flags %04x",
		(void *)cb, cb->fd_flags);

	/* don't let the completion thread start to reap completions
	 * until the connection manager thread has completely finished
	 * establishing this connection -- otherwise we get a race
	 * between these 2 threads
	 */
	our_completion_thread_wait_active(cb);

	TRACE_DBG("active, cb->fd_flags %#x", cb->fd_flags);

	do_busy_poll = cb->fd_flags & EXS_FD_BUSYPOLL;
	for ( ; ; ) {
		STATISTICS_INCREMENT(completion_loops);

		/* loop to wait/poll for at least 1 new work completion */
		if (do_busy_poll) {
			count = our_poll_cq_busy_poll(cb);
		} else {
			count = our_poll_cq_wait_notify(cb);
		}
		if (count < 0) {
			goto out0;
		}

		/* when get out of loop, count is positive ibv_poll_cq()
		 * result */
		STATISTICS_INCREMENT(good_ibv_poll_cqs);

		/* now process all the work completions we just polled */
		wc_limit = &cb->wc_array[count];

		/* once around this loop for each work completion just polled */
		for (wc = cb->wc_array; wc != wc_limit; wc++) {
			PRINT_STATUS("our_completion_thread ibv_poll_cq",
					wc->status);
			if (wc->status != IBV_WC_SUCCESS) {
				our_handle_bad_completion(wc, cb);
				goto out0;
			} else if (our_dispatch_completion(wc, cb) < 0) {
				goto out0;
			}
		}	/* for each work completion */

		STATISTICS_SET_MAX(max_ibv_poll_cq_completions, count);
	}	/* for ever */

out0:
	OUR_MEMBER_MUTEX_LOCK(cb, event_mutex);
	if (our_clear_conn_flags(cb, EXS_NEED_DISCONNECT | EXS_OK_TO_ISSUE_IO)
							& EXS_NEED_DISCONNECT) {
		our_rdma_disconnect(cb);
	}

	our_flush_stranded_send_blocks(cb);
	our_flush_all_stranded_recv_blocks(cb);

	if (cb->eof_block_array[0].in_progress
			&& cb->eof_block_array[0].qhandle) {
		our_shutdown_finished(cb, &cb->eof_block_array[0], SHUT_RDWR);
	}
	if (cb->eof_block_array[1].in_progress
			&& cb->eof_block_array[1].qhandle) {
		our_shutdown_finished(cb, &cb->eof_block_array[1], SHUT_RDWR);
	}

	TRACE_DBG("leave conn_state %s", our_conn_state_str(cb->conn_state));
	OUR_MEMBER_MUTEX_UNLOCK(cb, event_mutex);
	cb = our_socket_unref(cb);
	pthread_exit(NULL);
}	/* our_completion_thread */


/* create a new our_completion_thread
 * to collect ibv completion events on this new connection
 *
 * called by client from our_connect_route_resolved()
 *	when our_con_man thread gets RDMA_CM_EVENT_ROUTE_RESOLVED
 *
 * called by server from our_merge_pending_accept()
 *	when our_con_man thread gets RDMA_CM_EVENT_CONNECT_REQUEST
 *
 * this probably should be done later in the state machine
 *	by the client in our_connect_established()
 *	by the server in our_accept_established()
 * after they set EXS_NEED_DISCONNECT in cb->conn_flags
 *	(so our_completion_thread can start reaping completion events
 *	 immediately, without waiting for the transmit_ok_notify signal)
 *
 * When called, event_mutex is locked
 */
static int
our_create_completion_thread(struct exs_conn *cb)
{
	sigset_t newsigset, oldsigset;
	int	ret;

	TRACE_DBG("for cb %p, signal_limit %d",
		(void *)cb, cb->signal_limit);

	/* Set state bit so we know that we must cancel this thread later if
	 * we abruptly terminate the connection. */
	atomic_set_bits(EXS_NEED_COMP_CANCEL, &cb->conn_flags);

	/* Temporarily block all signals in this thread while we create the new
	 * thread, so that it inherits our full set of blocked signals.  Then
	 * restore the caller's original blocked signal set. */
	sigfillset(&newsigset);
	pthread_sigmask(SIG_BLOCK, &newsigset, &oldsigset);
	ret = pthread_create(&cb->comp_thread, NULL, our_completion_thread,
							our_socket_ref(cb));
	pthread_sigmask(SIG_BLOCK, &oldsigset, NULL);
	if(ret)
	{
		OUR_PERROR(ret, "pthread_create of our_completion_thread failed");
		atomic_clear_bits(EXS_NEED_COMP_CANCEL, &cb->conn_flags);
		cb = our_socket_unref(cb);
		goto out;
	}
	/* FIXME: printing pthread_t as unsigned long is unportable */
	TRACE_DBG("pthread_create created new our_completion_thread with id %lu",
		cb->comp_thread);

	pthread_detach(cb->comp_thread);

	if (cb->comp_thread_cpu != INT_MAX) {
		if ((ret = _exs_set_thread_affinity(
				cb->comp_thread,
				cb->comp_thread_cpu, NULL)) < 0) {
			OUR_PERROR(errno, "set completion thread affinity failed");
		}
	}

out:
	return ret;
}	/* our_create_completion_thread */


/* destroy the stuff set up in our_init_completion_synch() above */
static void
our_destroy_completion_synch(struct exs_conn *this_conn)
{
	if (atomic_clear_bits(EXS_NEED_DESTROY_COMP_SYNC,
			&this_conn->conn_flags) & EXS_NEED_DESTROY_COMP_SYNC) {
		pthread_cond_destroy(&this_conn->transmit_ok_notify);
		pthread_mutex_destroy(&this_conn->event_mutex);
	}
}	/* our_destroy_completion_synch */


/*
 * Called from either our_merge_pending_accept (in a con_man context)
 *		   or exs_accept (in a user context)
 *
 * When called, listen_conn's event_mutex MUST be locked
 *		(prevents accepted_conn from processing completions until
 *		 we are finished setting everything up here)
 *		and listen_conn->pending_accept_list MUST NOT be empty
 *
 * Returns 0 if all ok, -1 on error.
 */
static int
our_continue_merge_pending_accept(struct exs_conn *listen_conn,
			 const struct rdma_cm_event *cm_event,
			 const struct our_nego_mess *nego_recv)
{
	int				ret = 0;
	enum conn_states		old_state;
	struct pending_accept		*new_pending;
	struct rdma_conn_param		conn_param;
	struct exs_conn			*accepted_conn;

	/* remove first item from pending_accept_list */
	new_pending = exs_p_list_del_head(&listen_conn->pending_accept_list);
	if (new_pending == NULL) {
		errno = EINVAL;
		return -1;
	}

	TRACE_DBG("got new_pending %p", (void *)new_pending);
	/* Acquire a reference onto the accepted_conn so that it can't get
	 * destroyed by another thread until we are done with it.  This is
	 * needed because we have to do our own cleanup if the accept fails so
	 * that we can restore the state of the control block and try again with
	 * a new connection. */
	accepted_conn = our_socket_ref(new_pending->accepted_conn);
	old_state = accepted_conn->conn_state;

	TRACE_DBG("about to fill in pending accept, accepted_conn %p",
			(void *)accepted_conn);

	our_setup_accepted_conn(listen_conn, accepted_conn, nego_recv);

	/* initialize completion condition variables */
	ret = our_init_completion_synch(accepted_conn);
	if(ret)
	{
		new_pending->err_numb = ret;
		goto err1;
	}

	accepted_conn->event_channel = rdma_create_event_channel();
	if (accepted_conn->event_channel == NULL) {
		new_pending->err_numb = errno;
		OUR_PERROR(new_pending->err_numb,
				"rdma_create_event_channel failed");
		goto err1_5;
	}
	/* save the user's fd in accepted_conn */
	accepted_conn->sockno = accepted_conn->event_channel->fd;

	/* agent is given a new cm_id by the system, so migrate it from the
	 * listener's event channel to the new event channel just created
	 */
	TRACE_DBG("merge cm_id %p (from con_man event) to accepted_conn %p as its cm_id",
		(void *)cm_event->id, (void *)accepted_conn);
	accepted_conn->cm_id = cm_event->id;
	if ((ret = rdma_migrate_id(cm_event->id, accepted_conn->event_channel)) != 0) {
		ret = errno;
		new_pending->err_numb = ret;
		OUR_PERROR(ret, "rdma_migrate_id failed");
		/* need to unwind stuff we just did above */
		goto err2;
	}

	/* create our_con_man_thread to pick up agent's asynch cm events */
	ret = create_our_con_man_thread(accepted_conn);
	if(ret) {
		/* error message already given,
		 * but need to unwind stuff just done above
		 */
		goto err3;
	}
	ret = our_setup_qp(accepted_conn, accepted_conn->cm_id);
	if(ret)
	{
		new_pending->err_numb = ret;
		OUR_PERROR(ret, "our_setup_qp failed");
		/* need to unwind stuff we just did above */
		goto err4;
	}
	TRACE_DBG("got queue pair %p for pending accept",
		(void *)accepted_conn->qp);

	TRACE_DBG("accepted on source port %u",
		ntohs(rdma_get_src_port(accepted_conn->cm_id)));
	TRACE_DBG("accepted destination port %u",
		ntohs(rdma_get_dst_port(accepted_conn->cm_id)));

	TRACE_DBG("Dump of src_addr");
	DUMP_BUF((void *)&accepted_conn->cm_id->route.addr.
		src_addr, sizeof(struct sockaddr_storage));

	TRACE_DBG("Dump of dst_addr:");
	DUMP_BUF((void *)&accepted_conn->cm_id->route.addr.
		dst_addr, sizeof(struct sockaddr_storage));

	/* create a thread to collect ibv completion events on
	 * this newly accepted connection
	 *
	 * this probably should be done later in the state machine
	 * in our_accept_established()
	 * when EXS_NEED_DISCONNECT is set in conn_flags
	 */

	accepted_conn->conn_state = EXS_CONN_ACCEPTING;

	 /*** !!! listener's event_mutex must be locked !!! ***/
	if((ret = our_create_completion_thread(accepted_conn)) != 0) {
		/* need to unwind stuff we just did above */
		new_pending->err_numb = ret;
		goto err5;
	}

	if (!accepted_conn->is_seqpacket && accepted_conn->stream_buf_size > 0){
		if (our_create_stream_recv_buf(accepted_conn) < 0) {
			new_pending->err_numb = ENOMEM;
			goto err6;
		}
	}

	our_setup_agent_nego_mess(accepted_conn, &new_pending->nego_send);

	/* create all the empty transaction blocks, post some of them */
	ret = our_create_and_post_blocks(accepted_conn);
	if(ret)
	{
		OUR_PERROR(ret, "failed create and post blocks");
		new_pending->err_numb = ret;
		goto err7;
	}

	memset(&conn_param, 0, sizeof(conn_param));

	conn_param.private_data = &new_pending->nego_send;
	conn_param.private_data_len = sizeof(new_pending->nego_send);
	conn_param.responder_resources = RESPONDER_RESOURCES;
	conn_param.initiator_depth = INITIATOR_DEPTH;
	conn_param.retry_count = RETRY_COUNT;
	conn_param.rnr_retry_count = RNR_RETRY_COUNT;

	/* actually accept the next client connection
	 * the accepted_conn->cm_id was set with the newly created
	 * id taken from the listening event we waited for above
	 * to tell us about the connection request.
	 */
	TRACE_DBG("accepting the pending accept");

	ret = rdma_accept(accepted_conn->cm_id, &conn_param);
	if(ret)
	{
		ret = errno;
		new_pending->err_numb = ret;
		OUR_PERROR(ret, "rdma_accept failed");
		/* need to unwind stuff we just did above */
		goto err7;
	}

	accepted_conn = our_socket_unref(accepted_conn);
	return 0;

err7:
	exs_buf_free(&accepted_conn->recv_buf.buf);
err6:
	our_cancel_completion_thread(accepted_conn);
err5:
	our_free_qp(accepted_conn);
err4:
	our_terminate_con_man_thread(accepted_conn);
err3:
	rdma_reject(accepted_conn->cm_id, &conn_param, sizeof(conn_param));
	rdma_destroy_id(accepted_conn->cm_id);
	accepted_conn->cm_id = NULL;
err2:
	rdma_destroy_event_channel(accepted_conn->event_channel);
	accepted_conn->event_channel = NULL;
err1_5:
	/* destroy the stuff set up in our_init_completion_synch() above */
	our_destroy_completion_synch(accepted_conn);
err1:
	/* restore connection state */
	accepted_conn->conn_state = old_state;
	/* add item back to pending_accept_list */
	exs_p_list_add_tail(&listen_conn->pending_accept_list, new_pending);
	accepted_conn = our_socket_unref(accepted_conn);
	return -1;
}	/* our_continue_merge_pending_accept */


/* called only by exs_accept() for a listening connection
 * called only in a user thread context
 *
 * appends a new non-empty list of "struct pending_accept" items
 * to the existing (possibly empty) accept_list of listen_conn
 */
static inline void
append_list_to_pending_accept_list(struct our_list_head *new_list,
				struct exs_conn *listen_conn)
{
	exs_p_list_splice(&listen_conn->pending_accept_list, new_list);
}	/* append_list_to_pending_accept_list */


/* called by exs_accept() and our_free_all_pending_lists()
 * for a listening connection
 * called only in a user thread context
 *
 * frees up all "struct pending_accept" items on new_list
 */
static void
our_free_new_accept_list(struct exs_conn *this_conn,
		struct our_list_head *new_list, int post_events)
{
	int			j;
	struct pending_accept	*new_pending;
	struct exs_conn		*accepted_conn;

	TRACE_DBG("free all pending accepts on new_list %p",
		 (void *)new_list);

	j = 0;
	while ((new_pending = exs_p_list_del_head(new_list))) {
		/* free "struct pending_connect" in "struct pending_accept" */
		accepted_conn = new_pending->accepted_conn;
		if (post_events) {
			new_pending->err_numb = ECONNABORTED;
			our_fill_in_accept_event(new_pending, this_conn->sockno);
		} else {
			/* free "struct pending_accept" itself */
			EXS_FREE(new_pending);
		}

		/* free "struct exs_conn" in "struct pending_accept" */
		accepted_conn->accept_pending = NULL;
		accepted_conn = our_socket_unref(accepted_conn);

		j++;
	}
	exs_p_list_destroy(new_list);

	TRACE_DBG("freed %d pending accepts from new_list %p",
		j, (void *)new_list);
}	/* our_free_new_accept_list */


/* Internal function that actually sets up accept. */
static int
our_start_accept(struct exs_conn *listen_conn, struct exs_acceptaddr *addrvec,
			int addrvec_cnt, struct evt_queue *qptr)
{
	int			i, ret;
	struct pending_accept	*new_pending;
	struct our_list_head	new_list;
	struct conn_requested	*old_requested;

	exs_p_list_init(&new_list, 0, addrvec_cnt);

	TRACE_DBG("listen_conn %p, addrvec %p, addrvec_cnt %d",
			(void *)listen_conn, (void *)addrvec, addrvec_cnt);
	TRACE_DBG("exs_accept on source port %u",
			ntohs(rdma_get_src_port(listen_conn->cm_id)));

	TRACE_DBG("Listening destination port %u",
			ntohs(rdma_get_dst_port(listen_conn->cm_id)));

	/* verify that this socket is currently listening */
	if(listen_conn->conn_state != EXS_CONN_LISTENING) {
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
		goto err0;
	}

	TRACE_DBG("exs_accept on source port %u",
			ntohs(rdma_get_src_port(listen_conn->cm_id)));
	TRACE_DBG("Listening destination port %u",
			ntohs(rdma_get_dst_port(listen_conn->cm_id)));

	/* we lock event_mutex.
	 * At first glance this seems excessive, but at this time
	 * nothing else should be happening on this listener socket,
	 * so there should be no contention for this lock --
	 * we lock it to explicitly prevent anything else from happening!
	 */

	/* need to lock event_mutex to protect listener's conn_state */
	OUR_MEMBER_MUTEX_LOCK(listen_conn, event_mutex);

	/* check again that this socket is currently listening */
	if(listen_conn->conn_state != EXS_CONN_LISTENING) {
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
		goto err0;
	}

	/* allocate memory for new accepted connections and events.
	 * A failure here is returned synchronously.
	 */
	ret = 0;
	for(i = 0; i < addrvec_cnt; i++) {
		TRACE_DBG("allocating memory for pending accept %d", i);

		new_pending = (struct pending_accept*)
				EXS_CALLOC(1, sizeof(*new_pending));
		if(new_pending == NULL) {
			OUR_PERROR(errno, "calloc of %zu bytes for new_pending %d on listener socket %d failed",
					sizeof(struct pending_accept),
					i, listen_conn->sockno);
			goto errmalloc1;
		}

		TRACE_DBG("Pending new_pending[%d] = %p",
			i, (void *)new_pending);

		/* allocates a new struct exs_conn, and initializes its
		 * state field to EXS_CONN_UNOPENED
		 */
		new_pending->accepted_conn = our_socket_new();
		if(new_pending->accepted_conn == NULL) {
			/* can't come up with enough memory */
			goto errmalloc2;
		}

		/* set pending back pointer */
		new_pending->accepted_conn->accept_pending=new_pending;

		/* copy exs_fcntl settings */
		new_pending->accepted_conn->comp_thread_cpu =
					listen_conn->comp_thread_cpu;
		new_pending->accepted_conn->inline_limit =
					listen_conn->inline_limit;

		if (addrvec) {
			memcpy(&new_pending->addrvec, &addrvec[i],
					sizeof(new_pending->addrvec));
		}

		ret = _get_empty_evt_queue_event(qptr, &new_pending->evt_item);
		if (ret < 0) {
			errno = -ret;
			ret = -1;
			goto errmalloc4;
		}
		/* Impossible to have unsignaled accept since the whole point is
		 * to get back the fd of the new connection */
		assert(new_pending->evt_item != NULL);

		new_pending->qhandle = qptr;
		exs_p_list_add_tail(&new_list, new_pending);
	}
	TRACE_DBG("done allocating memory for all %d pending accepts", i);
	append_list_to_pending_accept_list(&new_list, listen_conn);
	/* new_list is invalidated after the last call to
	 * append_list_to_pending_accept_list */

	while(!exs_p_list_empty(&listen_conn->pending_accept_list)
		&& listen_conn->conn_requested_backlog > 0
		&& !exs_p_list_empty(&listen_conn->conn_requested_list)) {
		/* a connect request from remote client already arrived
		 * and was waiting for these accepts, satisfy it now.
		 */
		old_requested = exs_p_list_del_head(
				&listen_conn->conn_requested_list);
		if(our_continue_merge_pending_accept(listen_conn,
					&old_requested->cm_event,
					&old_requested->nego_recv)) {
			/* merge failed */
			break;
		}
		/* merge worked ok, recyle old_requested */
		exs_p_list_add_tail(
				&listen_conn->conn_requested_free_list,
				old_requested);
		listen_conn->conn_requested_backlog--;
	}	/* while */
	goto exit;

	/* cleanup from failed accept initialization */

errmalloc4:
	/* free connection in incomplete pending_accept */
	new_pending->accepted_conn
		= our_socket_unref(new_pending->accepted_conn);

errmalloc2: /* free incomplete pending_accept */
	EXS_FREE(new_pending);

errmalloc1: /* free any completed pending_accept structs */
	our_free_new_accept_list(listen_conn, &new_list, 0);

	errno = ENOMEM;			/* Out of memory */
	ret = -1;

err0:
	exs_p_list_destroy(&new_list);

exit:
	OUR_MEMBER_MUTEX_UNLOCK(listen_conn, event_mutex);
	return ret;
}	/* our_start_accept */


/*
 *	exs_accept - page 11 of Interconnect Software Consortium Issue 1.0
 *		asynchronously accept incoming connections on a socket
 *
 *	returns 0 if async op initiated ok,
 *		else -1 with errno set to error value
 *
 * If the EXS_BLOCK flag is set in flags, this function will block until the
 * operation completes (i.e., the operation will be synchronous) and the
 * supplied addrvec_cnt must be 1.  In this case, the qhandle argument is
 * ignored.  No other flags are supported.
 */
EXS_EXPORT int
exs_accept(int sockno, exs_acceptaddr_t *addrvec, int addrvec_cnt,
		int flags, exs_qhandle_t qhandle)
{
	int			ret;
	int			local_errno;
	struct evt_queue	*qptr;
	struct evt_queue	*local_qhandle;
	struct exs_conn		*listen_conn;
	struct exs_event	local_vector;

	TRACE_DBG("start, sockno %d qhandle %p", sockno, (void *)qhandle);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that the user's flags are supported */
	if (flags & ~(EXS_BLOCK)) {
		errno = EINVAL;
		return -1;
	}

	/* verify that the caller is asking for at least 1 connection */
	if(addrvec_cnt <= 0)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	TRACE_DBG("socket %d, addrvec %p, addrvec_cnt %d, flags %#x",
			sockno, (void *)addrvec, addrvec_cnt, flags);


	/* verify that this socket has been created */
	if((listen_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* see if user gave us an event queue or wants a blocking accept */
	if (flags & EXS_BLOCK) {
		/* caller can ask for only 1 connection when blocking */
		if (addrvec_cnt != 1) {
			errno = EINVAL;		/* Invalid argument */
			ret = -1;
			goto out_now;
		}

		/* get qhandle from cache, dynamically creating one if needed */
		local_qhandle = our_get_from_queue_cache(listen_conn);
		if (local_qhandle == NULL) {
			/* couldn't create it, errno already set */
			ret = -1;
			goto out_now;
		}
		qptr = local_qhandle;	/* use our local qhandle */
	} else if ((qptr = _cnv_qnumb_to_ptr(qhandle, addrvec_cnt,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* caller has given us an invalid qhandle */
		ret = -1;
		goto out_now;
	} else {
		/* user supplied his own event queue, use it, no local one */
		local_qhandle = NULL;
	}

	ret = our_start_accept(listen_conn, addrvec, addrvec_cnt, qptr);
	if (ret) {
		/* always return any locally created queue to conn's cache */
		if (local_qhandle != NULL) {
			our_put_to_queue_cache(listen_conn, local_qhandle);
		} else if (qptr) {
			qptr = _evt_queue_unref(qptr, 1, EVT_QUEUE_REF_PENDING);
		}
		goto out_now;
	}

	/* synchronous part of accept started ok, rest happens asynchly */
	if(local_qhandle == NULL)
		goto out_now;

	/* caller wants a blocking (synchronous) accept */

	TRACE_DBG("blocking accept, local_qhandle %p",
			(void *)local_qhandle);

	/* accept was started ok, now wait for it to complete */
	ret = _evt_queue_dequeue_vector(local_qhandle,
					&local_vector, 1, NULL);

	TRACE_DBG("back from dequeue, local_qhandle %p, ret %d, exs_evt_type %d, exs_evt_errno %d",
		(void *)local_qhandle, ret,
		local_vector.exs_evt_type,
		local_vector.exs_evt_errno);

	if(ret != -1)
	{	/* no error on dequeue itself check completion */
		if((local_errno = local_vector.exs_evt_errno) != 0) {
			ret = -1;
		}
#if defined(ALLOW_PARANOID_CHECKING)
		else if(local_vector.exs_evt_type != EXS_EVT_ACCEPT)
		{	/* should never happen */
			TRACE_ERR("_evt_queue_dequeue_vector got item type %d, expected %d (EXS_EVT_ACCEPT)",
				local_vector.exs_evt_type, EXS_EVT_ACCEPT);
			errno = EBADSLT;	/* Invalid slot !!! */
			ret = -1;
		}
#endif
		else
		{	/* no error on accept, return fd of new connection */
			if (addrvec)
				addrvec->exs_addrlen = local_vector.exs_evt_union
					.exs_evt_accept.exs_evt_addrlen;
			ret = local_vector.exs_evt_union.exs_evt_accept
				.exs_evt_new_socket;
		}
	} else {
		local_errno = errno;
	}

	if (local_errno != ECONNABORTED) {
		/* return the locally created queue to the connection's cache */
		our_put_to_queue_cache(listen_conn, local_qhandle);
	} else {
		local_qhandle = _evt_queue_unref(local_qhandle, 1, 0);
	}
	errno = local_errno;

out_now:
	TRACE_DBG("finished, listen_conn %p, state %s, ret %d",
			(void *)listen_conn,
			our_conn_state_str(listen_conn->conn_state),
			ret);
	listen_conn = our_socket_unref(listen_conn);
	return ret;

}	/* exs_accept */


/*
 *	exs_blocking_accept()
 *			- not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns >= 0  if accepted ok
 *		== -1 if error (with errno set to error value)
 */
EXS_EXPORT int
exs_blocking_accept(int sockno, struct sockaddr *addr, socklen_t *addrlen)
{
	int ret;

	exs_acceptaddr_t addrvec;
	addrvec.exs_addr = addr;
	addrvec.exs_addrlen = addrlen ? *addrlen : 0;
	addrvec.exs_ahandle = NULL;

	ret = exs_accept(sockno, &addrvec, 1, EXS_BLOCK, NULL);
	if (addrlen) {
		*addrlen = addrvec.exs_addrlen;
	}
	return ret;
}	/* exs_blocking_accept */


/*
 * Called only in our_con_man_thread() context for a listening connection
 *
 * called directly from our_con_man_thread()
 * when an RDMA_CM_EVENT_CONNECT_REQUEST has been received on listen_conn
 *
 * When called, event_mutex MUST be locked.
 *
 * Returns 0 if all ok, -1 on error.
 */
static int
our_merge_pending_accept(struct exs_conn *listen_conn,
			 struct rdma_cm_event *cm_event)
{
	struct conn_requested		*new_requested;
	const struct our_nego_mess	*nego_recv;

	/* note: client may send more than the length the client actually sent,
	 *	 since the amount transfered is transport dependent
	 */
	if ((nego_recv = cm_event->param.conn.private_data) == NULL
		|| cm_event->param.conn.private_data_len < sizeof(*nego_recv)){
		/* something wrong with client's private data */
		if (nego_recv == NULL) {
			TRACE_ERR("client sent no private_data");
		} else {
			TRACE_ERR("client sent private_data_len %u, expected %zu",
				cm_event->param.conn.private_data_len,
				sizeof(*nego_recv));
		}
		/* Operation not supported on transport endpoint */
		errno = EOPNOTSUPP;
		goto reject;
	}

	if (our_check_nego_recv(listen_conn, nego_recv)) {
		goto reject;
	}

	if(!exs_p_list_empty(&listen_conn->pending_accept_list))
	{	/* a remote client wants to connect and this listener
		 * has at least 1 exs_accept() pending to match it
		 */
		return our_continue_merge_pending_accept(listen_conn, cm_event,
				nego_recv);
	}

	/* a remote client wants to connect, but this listener
	 * has no exs_accept() pending to match it
	 */
	TRACE_DBG("cb %p got event type CONNECT_REQUEST but pending_accept_list is empty",
		(void *)listen_conn);
	if(!(new_requested = exs_p_list_del_head(
				&listen_conn->conn_requested_free_list))) {
		/* no free structures, try to create a new one */
		new_requested = EXS_CALLOC(1, sizeof(*new_requested));
		if(new_requested == NULL)
		{	/* not enough memory for a new structure */
			OUR_PERROR(errno, "calloc of %zu bytes for struct conn_requested failed",
					sizeof(*new_requested));
			goto reject;
		}
	}
	listen_conn->conn_requested_backlog++;
	if(listen_conn->conn_requested_backlog
					> listen_conn->backlog_limit)
	{
		TRACE_ERR("current backlog %d exceeds limit %d on listening socket %d",
			listen_conn->conn_requested_backlog,
			listen_conn->backlog_limit,
			listen_conn->sockno);
	}
	TRACE_DBG("copy cm_id %p to new_requested %p as its cm_id, current backlog %d",
		(void *)cm_event->id, (void *)new_requested,
		listen_conn->conn_requested_backlog);
	memcpy(&new_requested->cm_event,cm_event,sizeof(struct rdma_cm_event));
	memcpy(&new_requested->nego_recv,nego_recv,sizeof(*nego_recv));
	exs_p_list_add_tail(&listen_conn->conn_requested_list, new_requested);
	return 0;

reject:
	if (rdma_reject(cm_event->id, NULL, 0)) {
		OUR_PERROR(errno, "rdma_reject CONNECT_REQUEST");
	}
	return -1;
}	/* our_merge_pending_accept */


/*
 * Called only in our_con_man_thread() context for an agent connection
 * called only from our_con_man_thread()
 *
 * when an RDMA_CM_EVENT_ESTABLISHED has been received on listen_conn
 *
 * When called, event_mutex MUST be locked.
 *
 * Returns 0 if all ok, -1 on error.
 */
static int
our_accept_established(struct exs_conn *accepted_conn)
{
	/* turn on all possible conn_flags */
	atomic_set_bits(EXS_NEED_LOCAL_CLOSE | EXS_NEED_DISCONNECT
					| EXS_NEED_EOF | EXS_OK_TO_ISSUE_IO,
						&accepted_conn->conn_flags);

	accepted_conn->conn_state = EXS_CONN_AGENT_SETUP;

	/* add accepted_conn to end of conn_list */
	our_add_socket_to_list(accepted_conn);
	return 0;
}	/* our_accept_established */

static void
our_uninit_transaction_block(struct transaction_block *block)
{
	if (block->user_event_ptr) {
		EXS_FREE(block->user_event_ptr);
	}
	/*
	exs_buf_free(&block->send_buf);
	*/
	exs_buf_free(&block->recv_rdma_buf);
}	/* our_uninit_transaction_block */

/*
 * called from our_setup_qp(),
 *	twice from our_free_qp(), our_connect_done()
 *	our_pending_accept_done()
 *
 * destroy all the things created in our_create_transaction_block()
 */
static void
our_free_transaction_block(struct transaction_block *block)
{
	our_uninit_transaction_block(block);
	EXS_FREE(block);
}	/* our_free_transaction_block */


/*	destroy the queue pair (QP) on connection cb
 *
 *	"Users must destroy any QP associated with an rdma_cm_id
 *	 before destroying the ID."
 */
static void
our_destroy_qp(struct exs_conn *cb)
{
	TRACE_DBG("call rdma_destroy_qp(%p) on cb %p",
		(void *)cb->cm_id, (void *)cb);
	rdma_destroy_qp(cb->cm_id);
	cb->qp = NULL;
}	 /* our_destroy_qp */


/*	destroy the completion queue (CQ) on connection cb
 *
 *	Problematic:
 *	"To avoid races, destroying a CQ will wait for all
 *	 completion events to be acknowledged; this guarantees a
 *	 one-to_one correspondence between acks and successful gets."
 */
static void
our_destroy_cq(struct exs_conn *cb)
{
	int	ret;

	/* acknowledge all accumulated successful ibv_get_cq_event() calls */
	if (cb->n_cq_events_that_need_ack != 0) {
		ibv_ack_cq_events(cb->cq, cb->n_cq_events_that_need_ack);
		cb->n_cq_events_that_need_ack = 0;
	}

	TRACE_DBG("call ibv_destroy_cq(%p) on cb %p", (void *)cb->cq, (void *)cb);
	if((ret = ibv_destroy_cq(cb->cq)) != 0)
	{	/* fails if any queue pair is still associated with this CQ */
		OUR_PERROR(ret, "ibv_destroy_cq(%p) on cb %p failed",
				(void *)cb->cq, (void *)cb);
	}
	cb->cq = NULL;
	if (cb->wc_array != NULL)
		EXS_FREE(cb->wc_array);
}	 /* our_destroy_cq */


/*	destroy the completion event channel on connection cb */
static void
our_destroy_comp_channel(struct exs_conn *cb)
{
	int	ret;

	if (cb->fd_flags & EXS_FD_BUSYPOLL) {
		return;
	}

	TRACE_DBG("call ibv_destroy_comp_channel(%p) on cb %p",
		(void *)cb->comp_channel, (void *)cb);
	if((ret = ibv_destroy_comp_channel(cb->comp_channel)) != 0)
	{	/* this fails if any CQs are still associated with the
		 * completion event channel being destroyed
		 */
		OUR_PERROR(ret, "ibv_destroy_comp_channel(%p) on cb %p failed",
			(void *)cb->comp_channel, (void *)cb);
	}
	cb->comp_channel = NULL;
}	/* our_destroy_comp_channel */

/* Attempt to create a queue pair with the given inline limit.  Returns the
 * value of errno on error and 0 on success. */
int
our_try_rdma_create_qp(struct exs_conn *cb, struct ibv_qp_init_attr *init_attr,
		int inline_limit)
{
	int ret;

	init_attr->cap.max_inline_data =
		cb->requested_cap.max_inline_data = inline_limit;
	if((ret = rdma_create_qp(cb->cm_id, cb->pd, init_attr))) {
		ret = errno;
		TRACE_DBG("rdma_create_qp failed requesting max_inline_data %u, errno %d",
			inline_limit, ret);
	} else {
		ret = 0;
		TRACE_DBG("rdma_create_qp ok requesting max_inline_data %u",
			inline_limit);
	}

	return ret;
}	/* our_try_rdma_create_qp */

/*
 * Called only in our_con_man_thread() context
 * called only from our_setup_qp()
 *
 * to create a queue pair
 *
 * no locks needed for this function
 *
 * Returns 0 if all ok, errno on error.
 */
static int
our_create_qp(struct exs_conn *cb)
{
	struct ibv_qp_init_attr	init_attr;
	struct ibv_qp_attr	attr;
	int			ret, i;

	TRACE_DBG("enter cb %p", (void *)cb);

	/* initialize the queue pair capabilities we will request */
	cb->requested_cap.max_send_wr = SQ_DEPTH(cb);
	cb->requested_cap.max_recv_wr = RQ_DEPTH(cb);
	cb->requested_cap.max_recv_sge = 4;
	cb->requested_cap.max_send_sge = 4;
	cb->requested_cap.max_inline_data = inline_data_size[0];

	memset(&init_attr, 0, sizeof(init_attr));
	memcpy(&init_attr.cap, &cb->requested_cap, sizeof(struct ibv_qp_cap));

	init_attr.qp_type = IBV_QPT_RC;
	init_attr.send_cq = cb->cq;
	init_attr.recv_cq = cb->cq;

	ret = -1;

	/* allocate a queue pair associated with the cm_id for this connection
	 * and transition it for sending and receiving.  On return, it is ready
	 * to handle posting of sends and receives.
	 *
	 * Try the user-specified inline limit first */
	if(cb->inline_limit != INT_MAX && our_try_rdma_create_qp(cb, &init_attr,
						cb->inline_limit) == 0) {
		goto got_qp;
	}

	/* Now try some known inline limits */
	for (i = 0; i < KNOWN_INLINE_DATA_SIZE_LIMIT; ++i) {
		if (our_try_rdma_create_qp(cb, &init_attr,
						inline_data_size[i]) == 0) {
			goto got_qp;
		}
	}

	/* If we get here, then we failed to create a queue pair! */
	goto err;

got_qp:
	ret = 0;
	/* queue pair created ok, save the new cm_id and the
	 * capabilities we obtained */
	cb->qp = cb->cm_id->qp;
	ibv_query_qp(cb->qp, &attr, -1, &init_attr);
	memcpy(&cb->obtained_cap, &init_attr.cap,
					sizeof(struct ibv_qp_cap));
	if (attr.path_mtu > 0 && attr.path_mtu < (sizeof(path_size)
							/ sizeof(*path_size))) {
		cb->obtained_path_mtu_size = path_size[attr.path_mtu];
	}

	/* Work around a bug in some adapters which do not respect their maximum
	 * inline size */
	if (cb->obtained_cap.max_inline_data
					> cb->requested_cap.max_inline_data) {
		cb->obtained_cap.max_inline_data
					= cb->requested_cap.max_inline_data;
	}

	/* Decrease the user-specified limit down to the obtained size */
	if (cb->inline_limit > cb->obtained_cap.max_inline_data) {
		cb->inline_limit = cb->obtained_cap.max_inline_data;
	}

	switch (cb->qp->context->device->transport_type) {
	case IBV_TRANSPORT_IB:
		TRACE_DBG("InfiniBand/RoCE interface supports RDMA_WRITE_WITH_IMM");
		break;
	case IBV_TRANSPORT_IWARP:
		TRACE_DBG("iWARP does not support RDMA_WRITE_WITH_IMM");
		cb->fd_flags |= EXS_FD_NO_WRITE_WITH_IMM;
		break;
	default:
		TRACE_DBG("Unknown transport type");
		cb->fd_flags |= EXS_FD_NO_WRITE_WITH_IMM;
		break;
	}	/* switch */

err:
	TRACE_DBG("finished, returning %d", ret);

	return ret;
}	/* our_create_qp */


/*
 * Called only in our_con_man_thread() context
 * called from our_merge_pending_accept() and our_connect_route_resolved()
 *
 * to set up a complete queue-pair
 *
 * no locks needed for this function
 *
 * returns 0 if all ok, else the errno if things go bad
 */
static int
our_setup_qp(struct exs_conn *cb, struct rdma_cm_id *cm_id)
{
	int			ret;

	TRACE_DBG("enter cb %p, cm_id %p",
		(void *)cb, (void *)cm_id);

	/* get a global protection domain for this cm_id's device */
	if ((cb->pd = our_get_pd(cm_id->verbs)) == NULL) {
		ret = errno;
		goto err0;
	}

	/* create a completion channel for this cm_id */
	if (cb->fd_flags & EXS_FD_BUSYPOLL) {
		cb->comp_channel = NULL;
	} else {
		cb->comp_channel = ibv_create_comp_channel(cm_id->verbs);
		if(!cb->comp_channel)
		{
			ret = errno;
			OUR_PERROR(ret, "ibv_create_comp_channel(%p) on cb %p failed",
				(void *)cm_id->verbs, (void *)cb);
			goto err1;
		}
		TRACE_DBG("created ibv completion channel %p, fd %d",
			(void *)cb->comp_channel, cb->comp_channel->fd);
	}

	/* create a completion queue */
	cb->cq = ibv_create_cq(cm_id->verbs, CQ_DEPTH(cb), cb,
				cb->comp_channel, 0);
	if(!cb->cq)
	{
		ret = errno;
		OUR_PERROR(ret, "ibv_create_cq(%p, %d, %p, %p) failed",
			(void *)cm_id->verbs, CQ_DEPTH(cb),
			(void *)cb, (void *)cb->comp_channel);
		goto err2;
	}
	TRACE_DBG("created ibv cq %p, cqe %d",
		(void *)cb->cq, cb->cq->cqe);

	cb->requested_cq_nelements = CQ_DEPTH(cb);
	cb->obtained_cq_nelements = cb->cq->cqe;

	/* allocate an array to hold polled work completions that is
	 * big enough to never fill up, even if cq is full when polled
	 */
	cb->max_n_wc_slots = cb->cq->cqe + 1;
	if ((cb->wc_array = (struct ibv_wc*)EXS_CALLOC(cb->max_n_wc_slots,
					sizeof(struct ibv_wc))) == NULL) {
		/* not enough memory for the work completion array */
		ret = errno;
		OUR_PERROR(ret, "calloc of %zu bytes for wc_array failed",
			sizeof(struct ibv_wc) * cb->max_n_wc_slots);
		goto err3;
	}

	TRACE_DBG("allocated wc_array %p, max_n_wc_slots %d",
		(void *)cb->wc_array, cb->max_n_wc_slots);

	/* create a queue pair */
	ret = our_create_qp(cb);
	if(ret)
	{
		OUR_PERROR(ret, "our_create_qp failed");
		goto err3;
	}
	TRACE_DBG("created qp %p", (void *)cb->qp);

	if ((ret = pthread_mutex_init(&cb->advert_sequence_mutex, NULL)) != 0) {
		goto err4;
	}
	if ((ret = pthread_mutex_init(&cb->data_sequence_mutex, NULL)) != 0) {
		goto err5;
	}

	if (!cb->is_seqpacket) {
		if ((ret = pthread_mutex_init(&cb->stream_sync_mutex,
								NULL)) != 0) {
			goto err6;
		}
	}

	cb->pending_send_count = 0;		/* number currently posted */
	cb->pending_recv_count = 0;		/* number currently posted */

	ret = our_nego_init(cb, &cb->nego_out_ptr);
	if(ret)
	{
		goto err9;
		/*goto err10;*/
	}

	/* to avoid races, prime the cq notification mechanism BEFORE
	 * we connect or accept the connection
	 */
	ret = ibv_req_notify_cq(cb->cq, 0);
	if (ret) {
		OUR_PERROR(ret, "failed ibv_req_notify_cq on cb %p",
				(void *)cb);
		goto err10;
	}
	TRACE_DBG("finished ok");
	goto err0;

err10:
	cb->nego_in_ptr = NULL;
err9:
	if (!cb->is_seqpacket) {
		pthread_mutex_destroy(&cb->stream_sync_mutex);
	}
err6:
	pthread_mutex_destroy(&cb->data_sequence_mutex);
err5:
	pthread_mutex_destroy(&cb->advert_sequence_mutex);
err4:
	our_destroy_qp(cb);
err3:
	our_destroy_cq(cb);
err2:
	our_destroy_comp_channel(cb);
err1:
	/* FIXME: What failed? */
	OUR_PERROR(ret, "failed");
err0:
	return ret;
}	/* our_setup_qp */


/*
 * called from our_merge_pending_accept(), our_connect_returning()
 *	our_close_accepting(), our_close_connecting, our_start_close()
 *
 * to destroy all the things created in our_setup_qp()
 *
 * when called, our_completion_thread MUST be canceled or this must be called
 * from our_completion_thread AFTER the event thread has exited and exs_close()
 * has been called
 *	(so it doesn't try to mess with cb's blocks and block lists)
 */
static void
our_free_qp(struct exs_conn *cb)
{
	struct ibv_wc		wc;
	int			ncompletes;

	TRACE_DBG("enter cb %p cm_id %p",
		(void *)cb, (void *)cb->cm_id);

	our_free_all_blocks(cb);

	/* try to pull "final" work completion(s) off completion queue */
	while ((ncompletes = ibv_poll_cq(cb->cq, 1, &wc)) > 0)
		{
		TRACE_DBG("ibv_poll_cq() returned %d entries",
			ncompletes);
		PRINT_STATUS(__func__, wc.status);
		}
	if (ncompletes < 0)
		{
		OUR_PERROR(-ncompletes, "ibv_poll_cq() returned error");
		}


	if (!cb->is_seqpacket) {
		pthread_mutex_destroy(&cb->stream_sync_mutex);
	}
	pthread_mutex_destroy(&cb->data_sequence_mutex);
	pthread_mutex_destroy(&cb->advert_sequence_mutex);

	TRACE_DBG("call our_destroy_qp");
	our_destroy_qp(cb);

	TRACE_DBG("call our_destroy_cq");
	our_destroy_cq(cb);

	TRACE_DBG("call our_destroy_comp_channel");
	our_destroy_comp_channel(cb);

	TRACE_DBG("finished ok");
}	/* our_free_qp */


/*
 * called from our_connect_addr_resolved(), our_connect_done(),
 *	our_client_finishing(), our_deal_with_bad_event()
 *	and three times by our_connect_route_resolved()
 *
 * when called, event_mutex MAY or MAY NOT be locked
 *
 * asynchronous error exits for various stages of exs_connect()
 *	- to unwind the partial series of allocations
 *
 * Returns whatever value is in ret (should be 0 if all ok, else errno)
 */
static int
our_connect_returning(struct exs_conn *new_conn, int err_level, int ret)
{
	OUR_PERROR(ret, "enter level %d, state %s, qhandle %p",
		err_level,
		our_conn_state_str(new_conn->conn_state),
		(void *)new_conn->conn_pending->qhandle);

	switch(err_level)
	{
	case 5:
		TRACE_DBG("calling rdma_disconnect for cb %p, cm_id %p",
			(void *)new_conn, (void *)new_conn->cm_id);

		if (our_clear_conn_flags(new_conn, EXS_NEED_DISCONNECT
							| EXS_OK_TO_ISSUE_IO)
							& EXS_NEED_DISCONNECT) {
			our_rdma_disconnect(new_conn);
		}
	case 45:
		exs_buf_free(&new_conn->recv_buf.buf);
	case 4:
		our_cancel_completion_thread(new_conn);
	case 3:
	case 2:
		/* destroy all the things created in our_setup_qp() */
		our_free_qp(new_conn);
	case 1:
		new_conn->conn_state = new_conn->conn_pending->old_state;
	case 0:
		break;
#if defined(ALLOW_PARANOID_CHECKING)
	default:
		/* should never happen */
		TRACE_ERR("unknown level %d", err_level);
		break;
#endif
	}	/* switch */

	new_conn->conn_pending->err_numb = ret;
	if(new_conn->conn_pending->qhandle != NULL)
	{
		_post_event_to_full_evt_queue(new_conn->conn_pending);
	}

	OUR_PERROR(ret, "leave, state %s, qhandle %p",
		our_conn_state_str(new_conn->conn_state),
		(void *)new_conn->conn_pending->qhandle);

	return ret;
}	/* our_connect_returning */


/*
 * Called only in our_con_man_thread() context for a client connection
 * called only from our_con_man_thread()
 *	when it receives an RDMA_CM_EVENT_ESTABLISHED
 *	and new_conn->conn_state == EXS_CONN_CONNECTING
 *
 * Starts the negotiation procedure.
 *
 * When called, event_mutex MUST be locked.
 *
 * Returns 0 if all ok, else the error number
 */
static int
our_connect_established(struct exs_conn *new_conn, struct rdma_cm_event *cm_event)
{
	const struct our_nego_mess	*nego_recv;
	uint32_t			value;

	/* we have connected, so a future rdma_disconnect is needed */
	atomic_set_bits(EXS_NEED_DISCONNECT, &new_conn->conn_flags);
	new_conn->conn_state = EXS_CONN_CLIENT_SETUP;

	TRACE_DBG("Connected on source port %u",
		ntohs(rdma_get_src_port(new_conn->cm_id)));
	TRACE_DBG("Connected destination port %u",
		ntohs(rdma_get_dst_port(new_conn->cm_id)));
	TRACE_DBG("Dump of src_addr:");
	DUMP_BUF((void *)&new_conn->cm_id->route.addr.src_addr,
		sizeof(struct sockaddr_storage));
	TRACE_DBG("Dump of dst_addr:");
	DUMP_BUF((void *)&new_conn->cm_id->route.addr.dst_addr,
		sizeof(struct sockaddr_storage));

	/* agent's version matched ours, so it returned parameter values
	 * by keeping the minimum of client's and agent's values
	 */
	nego_recv = cm_event->param.conn.private_data;

	TRACE_DBG("%21s %u.%u.%u",
		"agent's exs version",
		 nego_recv->version, nego_recv->subversion,
		 nego_recv->subsubversion);

	value = ntohl(nego_recv->client_credits);
	if (value > new_conn->posted_credits) {
		value = new_conn->posted_credits;
	}
	our_set_send_credit_limits(new_conn, value);
	TRACE_DBG("%21s %u",
			"posted credits",
			new_conn->posted_credits);

	value = ntohl(nego_recv->sp_max_size);
	if (value > new_conn->small_packet_max_size) {
		value = new_conn->small_packet_max_size;
	}
	new_conn->small_packet_max_size = value;
	TRACE_DBG("%21s %u",
			"small packet max size", value);

	if (!new_conn->is_seqpacket
			&& ntohl(nego_recv->recv_buf_len) > 0) {
		new_conn->remote_buf_n_empty
			= ntohl(nego_recv->recv_buf_len);
		new_conn->remote_recv_buf.ad_buf_addr
			= ntohll(nego_recv->recv_buf_addr);
		new_conn->remote_recv_buf.ad_buf_len
			= new_conn->remote_buf_n_empty;
		new_conn->remote_recv_buf.ad_buf_key
			= ntohl(nego_recv->recv_buf_rkey);
		new_conn->remote_buf_align
			= ntohl(nego_recv->recv_buf_align);
		TRACE_DBG("remote_recv_buf is %#.16" PRIx64 " rkey %#.8" PRIx32 " len %" PRIu32,
				new_conn->remote_recv_buf.ad_buf_addr,
				new_conn->remote_recv_buf.ad_buf_key,
				new_conn->remote_recv_buf.ad_buf_len);
	}

	/* Rather than finishing up at this point, we now
	 * build a SETUP_REQUEST message and send it to the server.
	 * This message contains a standard header and 4 fields:
	 *		a - the current version
	 *		b - the maximum number of buffers we want to be able
	 *		    to send to the server.  This is our current
	 *		    this_conn->posted_remote_credits, and is
	 *		    "What I want to be able to send to you".
	 *		c - the number of buffers we are reserving
	 *		    for the server to send to us.  This is
	 *		    our current cb->posted_local_credits
	 *		    and is "What I am able to receive from you".
	 *		d - the small_packet_max_size we want to use
	 */

	/* now send the SETUP_REQUEST to the agent */
	return our_client_nego_launch(new_conn);
}	/* our_connect_established */


/*
 * called only from our_client_finishing()
 */
static void
our_connect_done(struct exs_conn *new_conn)
{

	atomic_set_bits(EXS_OK_TO_ISSUE_IO|EXS_NEED_EOF, &new_conn->conn_flags);
	new_conn->conn_state = EXS_CONN_RUNNING;

	TRACE_DBG("Connected on source port %u",
		ntohs(rdma_get_src_port(new_conn->cm_id)));
	TRACE_DBG("Connected destination port %u",
		ntohs(rdma_get_dst_port(new_conn->cm_id)));
	TRACE_DBG("Dump of src_addr:");
	DUMP_BUF((void *)&new_conn->cm_id->route.addr.src_addr,
		sizeof(struct sockaddr_storage));
	TRACE_DBG("Dump of dst_addr:");
	DUMP_BUF((void *)&new_conn->cm_id->route.addr.dst_addr,
		sizeof(struct sockaddr_storage));

	TRACE_DBG("done ok");

	if(new_conn->nego_out_ptr != NULL)
	{	/* need to free nego block */
		TRACE_DBG("free nego out transaction block %p",
			(void *)new_conn->nego_out_ptr);
		/* FIXME: hack to get rid of memory leak -- need a more
		 * permanent solution */
		ibv_dereg_mr(new_conn->nego_out_ptr->block_mr);
		EXS_FREE(new_conn->nego_out_ptr->send_header);
		our_free_transaction_block(new_conn->nego_out_ptr);
		new_conn->nego_out_ptr = NULL;	/* so we don't free it again */
	}

	/* post a "connect complete" event to user's "qhandle" on the return */
	our_connect_returning(new_conn, 0, 0);
}	/* our_connect_done */


/*
 * Called only in our_con_man_thread() context for a client connection
 *
 * called only from our_con_man_thread()
 *	when it receives an RDMA_CM_EVENT_ROUTE_RESOLVED
 *	and new_conn->conn_state == EXS_CONN_ROUTING
 *
 * the networking part of the connection has been completed,
 * so we call our_setup_qp() to set up our queue-pair,
 * post some receive buffers,
 * start up the completion thread for this connection,
 * then mark the state EXS_CONN_CONNECTING
 * and call rdma_connect() before departing asynchronously.
 *
 * When rdma_connect() completes,
 * we should get control back in our_connect_established().
 *
 * When called, event_mutex MUST be locked.
 *
 * Returns 0 if all ok, else errno if something goes wrong
 */
static int
our_connect_route_resolved(struct exs_conn *new_conn)
{
	int			ret;
	struct rdma_conn_param	conn_param;

	TRACE_DBG("calling our_setup_qp");

	ret = our_setup_qp(new_conn, new_conn->cm_id);
	if(ret)
	{
		TRACE_DBG("our_setup_qp failed, ret = %d", ret);
		return our_connect_returning(new_conn,1,ret);
	}
	TRACE_DBG("back from our_setup_qp, new qp %p",
		(void *)new_conn->cm_id->qp);

	/* create a thread to collect ibv completion events on this connection
	 *
	 * this probably should be done later in the state machine
	 * in our_connect_established()
	 * when EXS_NEED_DISCONNECT is set in conn_flags
	 */
	if((ret = our_create_completion_thread(new_conn)) != 0) {
		/* need to unwind stuff we just did above */
		return our_connect_returning(new_conn,3,ret);
	}


	if (!new_conn->is_seqpacket && new_conn->stream_buf_size > 0) {
		if ((ret = our_create_stream_recv_buf(new_conn)) < 0) {
			return our_connect_returning(new_conn,4,ret);
		}
	}

	our_fill_client_nego_mess(new_conn, &new_conn->nego_send);

	/* see "man rdma_connect" for values of these parameter fields */
	memset(&conn_param, 0, sizeof(conn_param));

	conn_param.private_data = &new_conn->nego_send;
	conn_param.private_data_len = sizeof(new_conn->nego_send);
	conn_param.responder_resources = RESPONDER_RESOURCES;
	conn_param.initiator_depth = INITIATOR_DEPTH;
	conn_param.retry_count = RETRY_COUNT;
	conn_param.rnr_retry_count = RNR_RETRY_COUNT;

	TRACE_DBG("calling rdma_connect");

	new_conn->conn_state = EXS_CONN_CONNECTING;

	if((ret = rdma_connect(new_conn->cm_id, &conn_param)))
	{
		OUR_PERROR(errno, "rdma_connect failed");
		return our_connect_returning(new_conn,45,ret);
	}
	TRACE_DBG("rdma_connect started ok");

	return ret;
}	/* our_connect_route_resolved */


/*
 * Called only in our_con_man_thread() context for a client connection
 *
 * called only from our_con_man_thread()
 *	when it receives an RDMA_CM_EVENT_ADDR_RESOLVED
 *	and new_conn->conn_state == EXS_CONN_ADDRESSING
 *
 * The addressing part of the connection has been completed,
 * so we mark the state EXS_CONN_ROUTING,
 * and call rdma_resolve_route() before departing asynchronously.
 *
 * When rdma_resolve_route() completes,
 * we should get control back in our_connect_route_resolved().
 *
 * When called, event_mutex MUST be locked.
 *
 * Returns 0 if all ok, else the error number
 */
static int
our_connect_addr_resolved(struct exs_conn *new_conn)
{
	int			ret;

	TRACE_DBG("calling rdma_resolve_route");

	new_conn->conn_state = EXS_CONN_ROUTING;

	/*---=-=--==--=--=Resolve route time-=--==-=-=-=-=-*/
	if((ret = rdma_resolve_route(new_conn->cm_id,
						2000/*timeout->tv_sec*1000*/)))
	{
		ret = errno;
		OUR_PERROR(ret, "rdma resolve route failed");
		return our_connect_returning(new_conn,1,ret);
	}
	TRACE_DBG("rdma_resolve_route started ok");
	return ret;

}	/* our_connect_addr_resolved */


static int
our_start_connect(struct exs_conn *new_conn, const struct sockaddr *address,
		int flags, struct timeval *timeout,
		struct evt_queue *qptr, void *ahandle)
{
	int			ret = 0;
	(void)timeout;
	(void)flags;

	/* we lock event_mutex.
	 * At first glance this seems excessive, but at this time
	 * nothing else should be happening on this client socket,
	 * so there should be no contention for either of these locks --
	 * we lock them to explicitly prevent anything else from happening!
	 */

	/* need to lock event_mutex to protect conn_state */
	OUR_MEMBER_MUTEX_LOCK(new_conn, event_mutex);

	/* check again that this socket has not already been connected */
	if(new_conn->conn_state != EXS_CONN_UNBOUND
				&& new_conn->conn_state != EXS_CONN_BOUND)
	{
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
		goto out;
	}

	TRACE_DBG("begin connection process new_conn %p, state %s",
			(void *)new_conn, our_conn_state_str(new_conn->conn_state));

	/* fill in ALL fields of struct pending_connect for this conn */
	new_conn->conn_pending->qhandle = qptr;
	new_conn->conn_pending->ahandle = ahandle;
	new_conn->conn_pending->err_numb = 0;
	new_conn->conn_pending->sockno = new_conn->sockno;
	new_conn->conn_pending->evt_type = EXS_EVT_CONNECT;
	new_conn->conn_pending->old_state = new_conn->conn_state;

	new_conn->conn_state = EXS_CONN_ADDRESSING;

	/*-=-=-=-=--=--=-=Resolve addr-==-=-=--=-==-=-*/
	ret = rdma_resolve_addr(new_conn->cm_id, NULL,
		(struct sockaddr *)address, 2000);
	/* FIXME: timeout parameter not respected because there must be
	 * multiple RDMA operations in order to do an exs_connect(). */
	if (ret) {
		/* rdma_resolve_addr() itself failed */
		OUR_PERROR(errno, "rdma_resolve_addr failed");
		new_conn->conn_state = new_conn->conn_pending->old_state;
	} else {
		TRACE_DBG("rdma_resolve_addr started ok new_conn %p",
							(void *)new_conn);
	}

out:
	OUR_MEMBER_MUTEX_UNLOCK(new_conn, event_mutex);

	return ret;
}	/* our_start_connect */

/*
 *	exs_connect - page 17 of Interconnect Software Consortium Issue 1.0
 *		asynchronously connect a socket
 *
 *	returns 0 if async op initiated ok,
 *		else -1 with errno set to error value
 *
 *	This routine starts a sequence of asynchronous actions that
 *	require the following structures to have already been created:
 *		new_conn
 *		new_conn->event_channel
 *		new_conn->cm_id
 *		new_conn->pd
 *		new_conn->comp_channel
 *		new_conn->cq
 *		new_conn->cm_id->qp == new_conn->qp
 *
 * In this routine, we first verify that these have been created
 * and that we have not already gone beyond the state EXS_CONN_BOUND.
 * If this is all in order, we create
 *		new_conn->conn_pending
 * to hold the working parameters of this exs_connect.
 *
 * We then set the conn_state to EXS_CONN_ADDRESSING,
 * and call rdma_resolve_addr() before departing asynchronously.
 *
 * When rdma_resolve_addr() completes,
 *	our_con_man_thread() should get an RDMA_CM_EVENT_ADDR_RESOLVED event
 *	and transfer control in our_connect_addr_resolved().
 *
 * If the EXS_BLOCK flag is set in flags, this function will block until the
 * operation completes (i.e., the operation will be synchronous).  In this case,
 * the qhandle and ahandle arguments are ignored.  No other flags are supported.
 */
EXS_EXPORT int
exs_connect(int sockno, const struct sockaddr *address,
		socklen_t address_len, int flags, struct timeval *timeout,
		exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	int			ret = 0;
	struct evt_queue	*qptr;
	struct evt_queue	*local_qhandle;
	struct exs_conn		*new_conn;
	exs_event_t		local_vector;

	TRACE_DBG("enter fd %d, qhandle %p",
		sockno, (void *)qhandle);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that the user's flags are supported */
	if (flags & ~(EXS_BLOCK)) {
		errno = EINVAL;
		return -1;
	}

	/* verify that this socket has been created */
	if((new_conn = our_lookup_socket(sockno)) == NULL) {
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has not already been connected or listened */
	if(new_conn->conn_state != EXS_CONN_UNBOUND
				&& new_conn->conn_state != EXS_CONN_BOUND) {
		ret = errno = EINVAL;   /* Invalid argument */
		goto err0;
	}

	/* see if user gave us an event queue or wants a blocking connect */
	if (flags & EXS_BLOCK) {
		/* get qhandle from cache, dynamically creating one if needed */
		local_qhandle = our_get_from_queue_cache(new_conn);
		if(local_qhandle == NULL) {
			/* couldn't create it, errno already set */
			ret = errno;
			goto err0;
		}
		qptr = local_qhandle;	/* use our local qhandle */
	} else if((qptr = _cnv_qnumb_to_ptr(qhandle, 1,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* Invalid argument */
		ret = errno;
		goto err0;
	} else {
		/* user supplied his own event queue, use it, no local one */
		local_qhandle = NULL;
	}

	(void)address_len;
	ret = our_start_connect(new_conn, address, flags, timeout, qptr, ahandle);

	if (ret) {
		goto err1;
	}

	if (local_qhandle == NULL) {
		goto outok;
	}

	/* caller wants a blocking (synchronous) connect */

	TRACE_DBG("blocking connect, local_qhandle %p, ahandle %p",
					(void *)local_qhandle, (void *)ahandle);

	/* connect was started ok, now wait for it to complete */
	ret = _evt_queue_dequeue_vector(local_qhandle, &local_vector, 1, NULL);

	TRACE_DBG("back from dequeue, local_qhandle %p, ahandle %p, ret %d, exs_evt_type %d, exs_evt_errno %d, exs_evt_ahandle %p",
		(void *)local_qhandle, (void *)ahandle, ret,
		local_vector.exs_evt_type,
		local_vector.exs_evt_errno,
		(void *)local_vector.exs_evt_ahandle);

	/* always return the locally created queue to the connection's cache */
	our_put_to_queue_cache(new_conn, local_qhandle);

	if (ret < 0 || (errno = local_vector.exs_evt_errno) != 0) {
		ret = errno;
		goto err0;
	}

outok:
	TRACE_DBG("finished ok, new_conn %p, state %s",
			(void *)new_conn, our_conn_state_str(new_conn->conn_state));
	new_conn = our_socket_unref(new_conn);

	return 0;

err1:
	ret = errno;
	if (qptr) {
		qptr = _evt_queue_unref(qptr, 1, EVT_QUEUE_REF_PENDING);
	}
	if (local_qhandle) {
		/* caller wanted a blocking (synchronous) connect,
		 * delete the locally created queue
		 */
		our_put_to_queue_cache(new_conn, local_qhandle);
	}
err0:
	OUR_PERROR(ret, "failed, new_conn %p, state %s error",
			(void *)new_conn, our_conn_state_str(new_conn->conn_state));
	new_conn = our_socket_unref(new_conn);
	return -1;
}	/* exs_connect */


/*
 *	exs_blocking_connect()
 *			- not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns == 0  if connected ok
 *		== -1 if error (with errno set to error value)
 */
EXS_EXPORT int
exs_blocking_connect(int sockno, const struct sockaddr *address,
		socklen_t address_len)
{
	return exs_connect(sockno, address, address_len, EXS_BLOCK,
					NULL, NULL, NULL);
}	/* exs_blocking_connect */


/*
 * to send an RDMA_WRITE with remote notification
 * local side will write this buffer as part of the send
 *
 * This posts an IBV_WR_RDMA_WRITE_WITH_IMM if the operation is supported by the
 * underlying transport (all except for iWARP). On iWARP, this function posts an
 * IBV_WR_RDMA_WRITE followed by an IBV_WR_SEND.
 *
 * send_unreg_mr will be set to a ibv_mr that will need to be deregistered after
 * the WRITE operation completes, or NULL if nothing needs to be deregistered.
 *
 * On input, if IBV_SEND_SIGNALED is set in send_flags, a signal will be forced
 * for this WRITE whether or not it is needed.  On output, send_flags will
 * contain all flags that were set in the work request for the RDMA WRITE
 * request (plus the IBV_SEND_SIGNALED flag if the transport is iWARP and the
 * IBV_WR_SEND request needed to be signaled).
 *
 * Neither send_unreg_mr nor send_flags should be NULL.
 *
 * returns number of bytes actually sent if our_post_send() worked
 * successfully, -1 on any error
 */
static ssize_t
our_send_rdma_write_notify(struct exs_conn *this_conn,
		struct transaction_block *block,
		const void *buffer, size_t length, exs_mhandle_t mhandle,
		struct our_advert *ad_ptr,
		int *send_flags, struct ibv_mr **send_unreg_mr)
{
	int			ret;
	struct ibv_mr		*use_this_mr;
	int			force_signaled;
	uint64_t		remote_addr;
	uint32_t		remote_key;
	uint32_t		imm_data;
	uint32_t		imm_flags;
	unsigned int		new_data_signal_count;

	TRACE_DBG("started this_conn %p block %p buffer %p length %zd ad_ptr %p send_flags %8x",
				(void *)this_conn,
				(void *)block, (void *)buffer, length,
				(void *)ad_ptr, (unsigned)*send_flags);

	/* Assume this will fail */
	ret = -1;

	if (ad_ptr) {
		remote_addr = ad_ptr->ad_buf_addr;
		remote_key = ad_ptr->ad_buf_key;
		TRACE_DBG("addr %#" PRIx64 " rkey %#" PRIx32,
			remote_addr, remote_key);
	} else {
		remote_addr = this_conn->remote_recv_buf.ad_buf_addr
			+ this_conn->remote_buf_offset;
		remote_key = this_conn->remote_recv_buf.ad_buf_key;
	}

	/* output arguments must not be NULL */
	if (send_unreg_mr == NULL) {
		TRACE_DBG("send_unreg_mr output parameter is NULL");
		errno = EINVAL;
		return -1;
	}
	if (send_flags == NULL) {
		TRACE_DBG("send_flags output parameter is NULL");
		errno = EINVAL;
		return -1;
	}

	force_signaled = (*send_flags & IBV_SEND_SIGNALED);

	if (length < this_conn->inline_limit) {
		/* Send WRITE inline */
		new_data_signal_count = ++this_conn->data_signal_count;
		if (new_data_signal_count <= this_conn->signal_limit
							&& !force_signaled) {
			block->send_wr.send_flags = IBV_SEND_INLINE;
			STATISTICS_INCREMENT(unsignaled_rdma_write_blocks_sent);
		} else {
			/* time to force a periodic signal */
			this_conn->data_signal_count = 0;
			TRACE_DBG("force a signal, data_signal_count %d, signal_limit %d",
				new_data_signal_count, this_conn->signal_limit);
			block->send_wr.send_flags
					= IBV_SEND_INLINE|IBV_SEND_SIGNALED;
			STATISTICS_INCREMENT(signaled_rdma_write_blocks_sent);
		}
		use_this_mr = NULL;
		*send_unreg_mr = NULL;
	} else {
		/* Cannot send WRITE inline; must send signaled */
		block->send_wr.send_flags = IBV_SEND_SIGNALED;
		if (block->data_mr != NULL) {
			/* We already have a registration for this buffer */
			use_this_mr = block->data_mr;
			*send_unreg_mr = NULL;
			TRACE_DBG("using already-created temporary memory registration %p; addr=%p lkey=%" PRIu32,
				use_this_mr, use_this_mr->addr, use_this_mr->lkey);
		} else if(mhandle == EXS_MHANDLE_UNREGISTERED) {
			/* this data was not registered by the user */
			if ((this_conn->fd_flags & EXS_FD_SP_COPY_UNREG_ON_SEND)
					&& (length
					<= this_conn->small_packet_max_size)) {
				/* copy the data */
				TRACE_DBG("this_conn %p block %p copying %zu bytes",
						(void *)this_conn,
						(void *)block, length);
				memcpy(block->send_buf.ptr, buffer, length);
				buffer = block->send_buf.ptr;
				use_this_mr = block->send_buf.mr;
				*send_unreg_mr = NULL;
			} else {
				/* dynamically register user's data */
				*send_unreg_mr = our_call_ibv_reg_mr(
							this_conn->pd,
							block->user_buffer,
							block->user_length,
							IBV_ACCESS_REMOTE_READ,
							"user send buffer");
				if(*send_unreg_mr == NULL)
				{
					goto err1;
				}
				use_this_mr = *send_unreg_mr;
				TRACE_DBG("dynamically registered mr %p addr=%p; lkey=%" PRIu32,
					(void *)use_this_mr,
					(void *)use_this_mr->addr,
					use_this_mr->lkey);
			}
		} else {
			/* this data was already registered by the user */
			TRACE_DBG("mhandle %p is registered", (void *)mhandle);
			*send_unreg_mr = NULL;
			use_this_mr = our_ensure_buffer_registered(this_conn,
					mhandle, "user send buffer");
			if (use_this_mr == NULL) {
				/* should be able to register user buffer
				 * but couldn't */
				goto err1;
			}
		}
	}

	if (this_conn->is_seqpacket) {
		imm_data = 0;
		imm_flags = 0;
	} else {
		imm_flags = (ad_ptr ? IMM_DIRECT_BIT : 0);
		imm_data = ((this_conn->send_phase << IMM_PHASE_SHIFT)
							& IMM_PHASE_MASK)
			| ((this_conn->next_recv_advert_msg_seqno
					<< IMM_SEQNO_SHIFT) & IMM_SEQNO_MASK);
		this_conn->send_seqno += length;
	}

	if (!(this_conn->fd_flags & EXS_FD_NO_WRITE_WITH_IMM)) {
		/* set expected completion code to be RDMA_WRITE */
		block->send_wr.imm_data = htonl((imm_data & ~IMM_FLAG_MASK)
						| (imm_flags & IMM_FLAG_MASK));
	} else {
		block->send_wr.send_flags &= ~IBV_SEND_SIGNALED;
		block->writeack_wr.send_flags |= IBV_SEND_SIGNALED;
		block->writeack_buf->write_len = htonl(length);
		block->writeack_buf->write_index = htonl(imm_data);
		block->writeack_buf->write_flags = htonl(imm_flags);
	}

	block->buf_offset += length;
	block->send_total_bytes += length;

	/* send an RDMA_WRITE_WITH_IMM with the data */
	block->send_wr.wr.rdma.rkey = remote_key;
	block->send_wr.wr.rdma.remote_addr = remote_addr;
	block->send_wr.sg_list->length = length;
	block->send_wr.sg_list->addr
			= (uint64_t)(unsigned long)buffer;
	if (use_this_mr) {
		block->send_wr.sg_list->lkey = use_this_mr->lkey;
	}

	assert(block->send_wr.num_sge == 1);
	assert(block->send_wr.sg_list->length > 0);
	if (our_post_send(this_conn, &block->send_wr)) {
		goto err2;
	}
	if (block->send_wr.send_flags & IBV_SEND_SIGNALED) {
		this_conn->data_signal_count = 0;
	}

	STATISTICS_INCREMENT(rdma_write_blocks_sent);
	STATISTICS_ADD(rdma_write_bytes_sent, length);

	*send_flags = block->send_wr.send_flags;
	if ((this_conn->fd_flags & EXS_FD_NO_WRITE_WITH_IMM)
			&& (block->writeack_wr.send_flags & IBV_SEND_SIGNALED)){
		*send_flags |= IBV_SEND_SIGNALED;
	}

	ret = length;
	goto err1;
err2:
	if(*send_unreg_mr != NULL) {
		STATISTICS_INCREMENT(ibv_dereg_mr_calls);
		ibv_dereg_mr(*send_unreg_mr);
		*send_unreg_mr = NULL;
	}
err1:

	TRACE_DBG("returning %d this_conn %p block %p buffer %p length %zd ad_ptr %p send_flags %8x send_unreg_mr %p\n",
				ret,
				(void *)this_conn, (void *)block,
				(void *)buffer, length,
				(void *)ad_ptr, (unsigned)*send_flags,
				(void *)*send_unreg_mr);
	return ret;
}	/* our_send_rdma_write_notify */

/*
 * called only from exs_recv()
 *
 * to set up and send a buffer advertisement so the
 * remote side can write directly to this buffer
 *
 * returns 0 if advertisement was successfully posted; else -1 on any error
 */
static int
our_send_advert(struct exs_conn *this_conn, struct transaction_block *block)
{
	unsigned int unsignaled_count;
	const void *buffer;
	size_t length;

	if (!this_conn->is_seqpacket) {
		/* This test-and-set is nonatomic, but that is ok since this
		 * function is under stream_sync_mutex when called in
		 * SOCK_STREAM mode and the only other consumer of this value
		 * can only increase the counter. */
		if (this_conn->stream_advert_counter == 0) {
			STATISTICS_INCREMENT(no_credits_on_send_advert);
			return 0;
		}
		atomic_sub(1, &this_conn->stream_advert_counter);
	}

	buffer = block->user_buffer;
	length = block->user_length;

	TRACE_DBG("conn %p, block %p, advert_buf %p, advert_wr %p, advert_sg_list %p, data_mr %p index %" PRIu32 " flags %" PRIu32,
		(void *)this_conn, (void *)block, (void *)&block->advert_buf,
		(void *)&block->send_wr, (void *)block->send_wr.sg_list,
		(void *)block->data_mr, block->index, block->flags);

	if(block->mhandle == EXS_MHANDLE_UNREGISTERED) {
		/* this data was not registered by the user */
		if ((this_conn->fd_flags & EXS_FD_SP_COPY_UNREG_ON_RECV)
				&& length <= this_conn->small_packet_max_size) {
			block->flags |= EXS_COPY_UNREG;
			block->data_mr = NULL;
		} else {
			/* dynamically register user's data */
			block->data_mr = our_call_ibv_reg_mr(this_conn->pd,
						(void *)buffer, length,
						IBV_ACCESS_LOCAL_WRITE
						|IBV_ACCESS_REMOTE_WRITE,
						"user recv buffer");
			if(block->data_mr == NULL) {
				return -errno;
			}
			block->flags |= EXS_MR_DYNAMIC;
		}
	} else {
		/* this data was already registered by the user */
		TRACE_DBG("mhandle %p is registered",
			(void *)block->mhandle);
		block->data_mr = our_ensure_buffer_registered(this_conn,
					block->mhandle, "user recv buffer");
		if (block->data_mr == NULL) {
			/* should be able to register user buffer
			 * but couldn't */
			errno = EINVAL;
			return -EINVAL;
		}
	}

	if (!this_conn->is_seqpacket) {
		assert(block->data_mr != NULL);
	}

	/* Do some setup for the header for this message */
	block->send_header->hd_seqno = htonl(this_conn->next_send_advert_msg_seqno++);
	TRACE_DBG("assigned next advert hd_seqno %u",
		this_conn->next_send_advert_msg_seqno);

	if (!this_conn->is_seqpacket
			&& PHASE_IS_INDIRECT(this_conn->recv_phase)) {
		assert(this_conn->recv_buf.n_full == 0);
		assert(this_conn->recv_seqno == this_conn->source_seqno);
		++this_conn->recv_phase;
		TRACE_DBG("increment recv_phase to %u",
			this_conn->recv_phase);
	}

	/* Increment source_seqno.  For SOCK_SEQPACKET, we increment it per
	 * message.  For SOCK_STREAM, we increment it per byte, except that
	 * non-WAITALL messages are considered to be one byte. */
	block->advert_buf->ad_buf_seqno = htonll(this_conn->source_seqno);
	if (this_conn->is_seqpacket) {
		++this_conn->source_seqno;
	} else if (block->flags & MSG_WAITALL) {
		this_conn->source_seqno += length;
	} else {
		++this_conn->source_seqno;
	}

	/* fill in advert_buf with description of user's buffer
	 */
	if (block->flags & EXS_COPY_UNREG) {
		block->advert_buf->ad_buf_addr
			= htonll((uint64_t)(uintptr_t)block->recv_rdma_buf.ptr);
		block->advert_buf->ad_buf_key
			= htonl(block->recv_rdma_buf.mr->rkey);
	} else {
		block->advert_buf->ad_buf_addr
				= htonll((uint64_t)(unsigned long)buffer);
		block->advert_buf->ad_buf_key
				= htonl(block->data_mr->rkey);
	}
	block->advert_buf->ad_buf_phase = htonl(this_conn->recv_phase);
	block->advert_buf->ad_buf_len
			= htonl(length);
	if (this_conn->is_seqpacket) {
		block->advert_buf->ad_buf_flags = 0;
	} else {
		block->advert_buf->ad_buf_flags = htonl(
			block->flags & MSG_WAITALL);
		block->advert_buf->ad_ack_count = htonl(
			this_conn->n_unacked_wwi);
		block->advert_buf->ad_ack_empty = htonl(
			this_conn->recv_buf.n_empty);
	}

	TRACE_DBG("remote key 0x%08x, remote addr %p, remote length %zu",
				ntohl(block->advert_buf->ad_buf_key),
				(void *)buffer, length);

	block->send_wr.send_flags = 0;

	atomic_set_bits(ADVERT_SENDING_BIT, &block->send_state);

	if ((sizeof(block->send_header) + sizeof(block->advert_buf))
						<= this_conn->inline_limit) {
		block->send_wr.send_flags |= IBV_SEND_INLINE;

		unsignaled_count = ++this_conn->advert_signal_count;
		if (unsignaled_count >= this_conn->signal_limit) {
			/* time to force a periodic signal */
			this_conn->advert_signal_count = 0;
			TRACE_DBG("force a signal, advert_signal_count %d, signal_limit %d",
				this_conn->advert_signal_count,
				this_conn->signal_limit);
			block->send_wr.send_flags |= IBV_SEND_SIGNALED;
			block->unsignaled_count = unsignaled_count;
			STATISTICS_INCREMENT(signaled_advert_blocks_sent);
		} else {
			block->unsignaled_count = 0;
			atomic_set_bits(SEND_DONE_BIT, &block->send_state);
			STATISTICS_INCREMENT(unsignaled_advert_blocks_sent);
		}
	} else {
		block->send_wr.send_flags |= IBV_SEND_SIGNALED;
		block->unsignaled_count = this_conn->advert_signal_count + 1;
		this_conn->advert_signal_count = 0;
		STATISTICS_INCREMENT(signaled_advert_blocks_sent);
	}

	/* now post send of this advert, then leave */
	TRACE_DBG("send advert: seqno %" PRIu64 " n_empty %" PRIu32,
			ntohll(block->advert_buf->ad_buf_seqno),
			this_conn->recv_buf.n_empty);
	++this_conn->next_send_advert_msg_seqno;
	if (our_post_send(this_conn, &block->send_wr)) {
		/* got an input/output error, so advert was NOT SENT */
		atomic_clear_bits(ADVERT_SENDING_BIT|SEND_DONE_BIT,
							&block->send_state);
		OUR_PERROR(errno, "Could not post send of ADVERT");
		return -1;
	}
	STATISTICS_INCREMENT(advert_blocks_sent);

	if (!this_conn->is_seqpacket) {
		this_conn->recv_buf.n_empty = 0;
		this_conn->n_unacked_wwi = 0;
	}

	TRACE_DBG("finished ok");

	return 0;
}	/* our_send_advert */


/*
 * called from exs_send() and exs_sendfile(), and our_deal_with_completion()
 *
 * sets up and posts the send of
 *	the WWI message for 1 registered or large unregistered data buffer
 *
 * If the WWI can be sent inline and unsignalled, the user event will be posted.
 *
 * returns 0 if all ok, else -1 with errno set on any error
 */
static int
our_xmit_if_match_seqpacket(struct exs_conn *this_conn,
		struct transaction_block *this)
{
	int		err;
	int		send_flags;
	uint32_t	wwi_length;
	uint32_t	advert_length;
	exs_mhandle_t	mhandle;
	struct ibv_mr	*send_unreg_mr;
	const void *buffer;

	/* Set length */
	wwi_length = this->user_length;
	advert_length = this->advert_buf->ad_buf_len;
	assert(advert_length > 0);
	if (wwi_length > advert_length) {
		wwi_length = advert_length;
	}

	/* Clear the bits */
	atomic_clear_bits(ADVERT_READY_BIT | SEND_READY_BIT,
						&this->send_state);

	buffer = this->user_buffer;
	mhandle = this->mhandle;

	send_flags = IBV_SEND_SIGNALED;
	if (our_send_rdma_write_notify(this_conn, this, buffer, wwi_length,
					mhandle, this->advert_buf,
					&send_flags, &send_unreg_mr) < 0) {
		err = errno;
		goto err1;
	}
	STATISTICS_INCREMENT(direct_rdma_write_blocks_sent);

	if (send_unreg_mr) {
		this->data_mr = send_unreg_mr;
		this->flags |= EXS_MR_DYNAMIC;
	}

	if (!(send_flags & IBV_SEND_SIGNALED)) {
		TRACE_DBG("unsignalled write and last");
		STATISTICS_INCREMENT(send_done_after_sending);
		our_all_done_with_block(this_conn, this);
	}

	TRACE_DBG("finished ok");
	return 0;

err1:
	err = errno;
	OUR_PERROR(err, "failed, this_conn %p, state %s error",
		(void *)this_conn,
		our_conn_state_str(this_conn->conn_state));
	errno = err;
	return -1;
}	/* our_xmit_if_match_seqpacket */


/* Returns 1 if the send was successful and completes the exs_send() operation,
 * 0 if the send was successful but we still must wait to send more data or for
 * a completion, and -1 if there was an error (check errno). */
static int
our_handle_match_stream(struct exs_conn *this_conn,
		struct transaction_block *this,
		struct stream_advert_block *advert)
{
	uint32_t	wwi_length;
	uint32_t	advert_length;
	exs_mhandle_t	mhandle;
	int done_with_send, recv_flags, send_flags;
	struct ibv_mr *send_unreg_mr;
	const void *buffer;

	memcpy(this->advert_buf, &advert->advert_buf,
						sizeof(*this->advert_buf));
	recv_flags = this->advert_buf->ad_buf_flags;
	if (advert->offset > 0) {
		this->advert_buf->ad_buf_addr += advert->offset;
		this->advert_buf->ad_buf_len -= advert->offset;
	}

	/* Set length */
	wwi_length = this->user_length - this->buf_offset;
	advert_length = this->advert_buf->ad_buf_len;

	if ((done_with_send = (wwi_length <= advert_length))) {
		atomic_clear_bits(SEND_READY_BIT, &this->send_state);
		this_conn->next_send_primed = this->next;
	} else {
		wwi_length = advert_length;
	}

	advert->offset += wwi_length;
	if (advert_length <= wwi_length || !(recv_flags & MSG_WAITALL)) {
		assert(exs_p_list_peek_head(&this_conn->recved_advert_queue)
								== advert);
		exs_p_list_del_head(&this_conn->recved_advert_queue);
		exs_p_list_add_tail(&this_conn->recved_advert_empty_list, advert);
	}

	TRACE_DBG("seqno %"PRIu64" (advert seqno %" PRIu64 ") matched send_block %p offset %zu length %u advert_block %p offset %u length %u",
		this_conn->send_seqno,
		advert->advert_buf.ad_buf_seqno,
		(void *)this, this->buf_offset, wwi_length,
		(void *)advert, advert->offset - wwi_length,
		advert_length);

	buffer = this->user_buffer + this->buf_offset;
	mhandle = this->mhandle;

	++this->outstanding_completion_count;
	send_flags = IBV_SEND_SIGNALED;
	if (our_send_rdma_write_notify(this_conn, this, buffer,
					wwi_length, mhandle,
					this->advert_buf,
					&send_flags,
					&send_unreg_mr) < 0) {
		++this_conn->stream_wwi_counter;
		--this->outstanding_completion_count;
		return -1;
	}

	if (send_unreg_mr) {
		this->data_mr = send_unreg_mr;
		this->flags |= EXS_MR_DYNAMIC;
	}

	STATISTICS_INCREMENT(direct_rdma_write_blocks_sent);
	if (!(send_flags & IBV_SEND_SIGNALED)) {
		--this->outstanding_completion_count;
	}

	return (done_with_send && !(send_flags & IBV_SEND_SIGNALED)) ? 1 : 0;
}	/* our_handle_match_stream */


/* Returns nonzero if we can match the given advertisement, and zero if we
 * cannot match the given advertisement.  Called only from
 * our_xmit_if_match_stream().
 *
 * When called, stream_sync_mutex MUST be locked. */
static inline int
our_can_match_advert(struct exs_conn *this_conn,
		struct stream_advert_block *advert)
{
	int ret;

	ret = 1;
	if (PHASE_IS_INDIRECT(this_conn->send_phase)
			&& (!serial_less(advert->advert_buf.ad_buf_phase,
				this_conn->send_phase))
			&& (serial_greater_64(advert->advert_buf.ad_buf_seqno,
					this_conn->send_seqno))) {
		TRACE_DBG("got advert_seqno %lu > send_seqno %lu unexpectedly",
				advert->advert_buf.ad_buf_seqno,
				this_conn->send_seqno);
	}
	if (PHASE_IS_INDIRECT(this_conn->send_phase)
			&& (serial_less(advert->advert_buf.ad_buf_phase,
							this_conn->send_phase)
					|| (serial_less_64(
						advert->advert_buf.ad_buf_seqno,
						this_conn->send_seqno)))) {
		/* We already sent at least part of this
		 * data, so throw away the advert */
		TRACE_DBG("drop advert with seqno %" PRIu64 " phase=%" PRIu32 "; send_seqno=%" PRIu64 " phase=%u",
			advert->advert_buf.ad_buf_seqno,
			advert->advert_buf.ad_buf_phase,
			this_conn->send_seqno,
			this_conn->send_phase);
		if (serial_less(this_conn->send_phase,
				advert->advert_buf.ad_buf_phase)) {
			this_conn->send_phase
				= advert->advert_buf.ad_buf_phase + 1;
		}
		assert(PHASE_IS_INDIRECT(this_conn->send_phase));

		assert(exs_p_list_peek_head(&this_conn->recved_advert_queue)
								== advert);
		exs_p_list_del_head(&this_conn->recved_advert_queue);
		exs_p_list_add_tail(&this_conn->recved_advert_empty_list, advert);
		ret = 0;
	}

	return ret;
}	/* our_can_match_advert */

static int
our_xmit_if_match_stream(struct exs_conn *this_conn,
		struct transaction_block *this)
{
	struct stream_advert_block *advert;
	size_t		total_length;
	int		ret;

	ret = 0;		/* assume we will succeed */

	OUR_MEMBER_MUTEX_LOCK(this_conn, stream_sync_mutex);
	while (this_conn->stream_wwi_counter > 0) {
		TRACE_DBG("remote_buf_n_empty %u",
				this_conn->remote_buf_n_empty);
		this = this_conn->next_send_primed;
		if (!(this->send_state & SEND_READY_BIT)) {
			break;
		}

		advert = exs_p_list_peek_head(&this_conn->recved_advert_queue);
		if (advert) {
			assert(PHASE_IS_DIRECT(advert->advert_buf.ad_buf_phase));
			if (!our_can_match_advert(this_conn, advert)) {
				continue;
			}
			if (PHASE_IS_INDIRECT(this_conn->send_phase)) {
				STATISTICS_INCREMENT(direct_indirect_switches);
				this_conn->send_phase
						= advert->advert_buf.ad_buf_phase;
				TRACE_DBG("increment send_phase to %u on send direct",
					this_conn->send_phase);
			} else {
				assert(advert->advert_buf.ad_buf_phase
						== this_conn->send_phase);
			}
			--this_conn->stream_wwi_counter;
			STATISTICS_SET_MAX(max_used_wwi_credits,
					this_conn->posted_credits
					- this_conn->stream_wwi_counter);
			ret = our_handle_match_stream(this_conn, this, advert);
			if (ret < 0) {
				OUR_PERROR(errno, "could not send WWI");
				break;
			} else if (ret > 0) {
				TRACE_DBG("completed write operation for block %p",
					(void *)this);
				STATISTICS_INCREMENT(send_done_after_sending);
				atomic_set_bits(SEND_DONE_BIT, &this->send_state);
				our_all_done_with_block(this_conn, this);
			}
		} else if (this_conn->remote_buf_n_empty > 0) {
			if (PHASE_IS_DIRECT(this_conn->send_phase)) {
				STATISTICS_INCREMENT(direct_indirect_switches);
				++this_conn->send_phase;
				TRACE_DBG("increment send_phase to %u on send indirect",
					this_conn->send_phase);
			}
			total_length = this->user_length;
			--this_conn->stream_wwi_counter;
			STATISTICS_SET_MAX(max_used_wwi_credits,
					this_conn->posted_credits
					- this_conn->stream_wwi_counter);
			ret = our_send_indirect(this_conn, this);
			if (ret < 0) {
				OUR_PERROR(errno, "could not send WWI");
				break;
			} else if (this->buf_offset >= total_length) {
				atomic_clear_bits(SEND_READY_BIT,
							&this->send_state);
				this_conn->next_send_primed = this->next;
			}
		} else {
			STATISTICS_INCREMENT(recv_buf_full_on_send);
			break;
		}
	}
	if (this_conn->stream_wwi_counter == 0) {
		STATISTICS_INCREMENT(no_credits_on_send_wwi);
	}
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, stream_sync_mutex);

	TRACE_DBG("finished ok");
	return ret;
}	/* our_xmit_if_match_stream */


/* Get next transaction block if we can get a lock on the semaphore for that
 * block, indicating that we have credits to use it.  The appropriate mutex
 * (data_sequence_mutex or advert_sequence_mutex) must be locked prior to
 * calling this function.  Returns 0 if we got the semaphore on the transaction
 * block, or a negative errno value if we did not.  The only currently defined
 * error is -EBUSY if the semaphore is held by another thread. */
static int
our_try_get_next_block(struct exs_conn *this_conn,
				struct transaction_block *block, int flags)
{
	int ret = 0;
#if !defined(ALLOW_EXS_TRACE)
	(void)this_conn;
#endif

	if (flags & (EXS_CREDIT_WAIT|EXS_BLOCK)) {
		OUR_MEMBER_SEM_WAIT(block, sem);
	} else {
		/* get a flow-control send slot so we don't overrun the receiver */
		/* but don't block if all empty send slots are currently used up */
		TRACE_DBG("TRYWAIT block %p sem %p",
				(void *)block, (void *)&block->sem);
		if (sem_trywait(&block->sem) < 0) {
			TRACE_DBG("no free send slots on cb %p",
							(void *)this_conn);
			ret = (errno != EAGAIN) ? -errno : -EBUSY;
		}
		TRACE_DBG("PAST block %p sem %p",
				(void *)block, (void *)&block->sem);
	}
	return ret;
}	/* our_try_get_next_block */


static ssize_t
our_start_send(struct exs_conn *this_conn, const void *buffer, size_t length,
			int flags, struct evt_queue *qptr, void *ahandle,
			exs_mhandle_t mhandle)
{
	struct exs_event		*event_ptr;
	struct transaction_block	*block;
	ssize_t				ret;

	TRACE_DBG("socket %d this_conn %p state %s qptr %p flags %#08x length %zu",
		this_conn->sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state),
		(void *)qptr, flags, length);

	OUR_MEMBER_MUTEX_LOCK(this_conn, data_sequence_mutex);

	if ((ret = our_try_get_next_block(this_conn, this_conn->next_send,
								flags)) < 0) {
		errno = -ret;
		ret = -1;
		goto out0;
	}

	/* Verify that we are not in the middle of disconnecting */
	if (!(this_conn->conn_flags & EXS_OK_TO_ISSUE_SEND)) {
		if ((this_conn->conn_state == EXS_CONN_TERMINATED
				|| this_conn->conn_state == EXS_CONN_TERMINATING)
				&& (this_conn->conn_flags & EXS_NEED_EOF_SEND)) {
			errno = ECONNRESET;
		} else {
			errno = EPIPE;
		}
		block = NULL;
		ret = -1;
		goto err1;
	}

	ret = _get_empty_evt_queue_event(qptr, &event_ptr);
	if (ret < 0) {
		errno = -ret;
		ret = -1;
		goto err1;
	}
	assert(event_ptr != NULL || (flags & EXS_UNSIGNALED));

	atomic_add(1, &this_conn->pending_send_count);

	/* get an unused transaction block for sending */
	block = this_conn->next_send;
	this_conn->next_send = block->next;

	/* set block's flags field with flags from the exs_send */
	block->flags = flags;

	/* fill in transaction's fields with fixed parameters from exs_send */
	block->in_progress = 1;
	block->send_total_bytes = 0;
	block->buf_offset = 0;
	block->mhandle = mhandle;
	block->user_buffer = (void *)buffer;
	block->user_length = length;
	block->qhandle = qptr;
	block->outstanding_completion_count = 0;
	if (event_ptr) {
		event_ptr->exs_evt_type = EXS_EVT_SEND;
		event_ptr->exs_evt_errno = 0;
		event_ptr->exs_evt_ahandle = ahandle;
		event_ptr->exs_evt_socket = this_conn->sockno;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_buffer
							= (void *)buffer;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_length = length;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_mhandle = mhandle;
		block->user_event_ptr = event_ptr;
	}

	/* This will reset qptr to NULL or set it to a meaningful value. */
	ret = this_conn->ops->enqueue_send(this_conn, block, buffer, length);
	goto out0;

err1:
	OUR_MEMBER_SEM_POST(this_conn->next_send, sem);

out0:
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, data_sequence_mutex);

	/* FIXME: Figure out what is setting ret nonzero */
	return (ret < 0) ? ret : 0;
}	/* our_start_send */


/*
 *	exs_send - page 51 of Interconnect Software Consortium Issue 1.0
 *		asynchronously send a message on a socket
 *
 *	returns == 0  if async op initiated ok,
 *		>= 0  if sync op finished ok (value is no. of bytes written),
 *		== -1 if error ( errno set to error value)
 *
 *	only flags values accepted:
 *
 *		EXS_DONTWAIT		(nonstandard)
 *			without this flag, this function may block until a
 *				posted_send_slot becomes available (happens when
 *				user has too many outstanding exs_send calls)
 *			with this flag, this function will return EAGAIN error
 *				if no posted_send_slot is available
 *			with or without this flag, if this function returns 0
 *				then the data has not been sent until the
 *				completion event has been posted to qhandle.
 *		EXS_BLOCK		(nonstandard)
 *			force this send to block until it completes.  The
 *			qhandle and ahandle parameters are ignored.
 *		EXS_UNSIGNALED		(nonstandard)
 *			don't cause a completion event when this send completes.
 *			The qhandle and ahandle parameters are ignored.
 *
 *	qhandle values accepted:
 *		NULL
 *			don't cause a completion event when this send completes
 *		EXS_QHANDLE_INVALID
 *			left over from a previous error, cause a new error
 *		anything else
 *			assume it is a user-created queue
 *
 *	this may cause the completion event to be posted before returning
 */
EXS_EXPORT ssize_t
exs_send(int sockno, const void *buffer, size_t length, int flags,
		exs_qhandle_t qhandle, exs_ahandle_t ahandle,
		exs_mhandle_t mhandle)
{
	ssize_t				ret;
	struct evt_queue		*qptr;
	struct evt_queue		*local_qhandle;
	struct exs_conn			*this_conn;
	exs_event_t			local_vector;

	TRACE_DBG("qhandle %p, length %zu, flags 0x%x",
		(void *)qhandle, length, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if (_exs_global_flags & EXS_GLOBAL_BYPASS_PARAM_CHECKING) {
		goto no_parameter_checks;
	}

	/* verify that caller is either giving us a valid qhandle or NULL */
	if(qhandle == EXS_QHANDLE_INVALID) {
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* verify that flags are supported for this socket type or protocol */
	if((flags & ~(EXS_DONTWAIT|EXS_CREDIT_WAIT|EXS_BLOCK|EXS_UNSIGNALED))!=0)
	{
		errno = EOPNOTSUPP;	/* Operation not supported */
		return -1;
	}

	/* if user specified a NULL qhandle, then ensure that either EXS_BLOCK
	 * or EXS_UNSIGNALED flag was specified. */
	if (qhandle == NULL && !(flags & (EXS_BLOCK|EXS_UNSIGNALED))) {
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* verify that length is at least 1 and less than 1 Gigabyte*/
	if (length <= 0 || length > MAX_EXS_MESSAGE_SIZE) {
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* ensure that buffer is not NULL,
	 * TO-DO -- and is not an outstanding recycle buffer,
	 * since that leads to a race condition
	 */
	if(buffer == NULL)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

no_parameter_checks:
	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has already been connected */
	if ((this_conn->conn_state > EXS_CONN_FINISHED
			&& this_conn->conn_state < EXS_CONN_RUNNING)
			|| this_conn->conn_state == EXS_CONN_LISTENING) {
		/* Transport endpoint is not connected */
		errno = ENOTCONN;
		ret = -1;
		goto out0;
	} else if ((this_conn->conn_state == EXS_CONN_TERMINATED
				|| this_conn->conn_state == EXS_CONN_TERMINATING)
				&& (this_conn->conn_flags & EXS_NEED_EOF_SEND)) {
		errno = ECONNRESET;
		ret = -1;
		goto out0;
	} else if (this_conn->conn_state != EXS_CONN_RUNNING) {
		errno = EPIPE;
		ret = -1;
		goto out0;
	}

	/* assume this is not created locally */
	local_qhandle = NULL;

	/* see if user gave us an event queue or wants a blocking send.  After
	 * this, qptr will be set or this function will have returned an error
	 * condition. */
	if (flags & EXS_BLOCK) {
		/* get qhandle from cache, dynamically creating one if needed */
		local_qhandle = our_get_from_queue_cache(this_conn);
		if(local_qhandle == NULL)
		{	/* couldn't create it, errno already set */
			ret = -1;
			goto out1;
		}
		qptr = local_qhandle;
	} else if (qhandle == NULL) {
		/* caller gave us no qhandle, so don't use one */
		qptr = NULL;
	} else if ((qptr = _cnv_qnumb_to_ptr(qhandle, 1,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* Invalid argument */
		errno = EINVAL;
		ret = -1;
		goto out1;
	}

	ret = our_start_send(this_conn, buffer, length, flags, qptr,
							ahandle, mhandle);
	if (ret < 0) {
		if (qptr && !local_qhandle) {
			qptr = _evt_queue_unref(qptr, 1, EVT_QUEUE_REF_PENDING);
		}
		goto out1;
	}

	if (local_qhandle) {
		/* caller wants a blocking (synchronous) send */

		TRACE_DBG("back from common_send, local_qhandle %p, ahandle %p, buffer %p, ret %ld",
			(void *)local_qhandle, ahandle, buffer, ret);

		/* send was started ok, now wait for it to complete */
		ret = _evt_queue_dequeue_vector(local_qhandle,
				&local_vector, 1, NULL);

		TRACE_DBG("back from dequeue, local qhandle %p, ahandle %p, buffer %p, ret %ld, exs_evt_type %d, exs_evt_errno %d, exs_evt_ahandle %p, exs_evt_length %ld, exs_evt_buffer %p",
			(void *)local_qhandle, ahandle, buffer, ret,
			local_vector.exs_evt_type,
			local_vector.exs_evt_errno,
			local_vector.exs_evt_ahandle,
			local_vector.exs_evt_union.exs_evt_xfer
						.exs_evt_length,
			local_vector.exs_evt_union.exs_evt_xfer
						.exs_evt_buffer);

		if(ret != -1)
		{	/* no error on dequeue itself check completion*/
			assert(local_vector.exs_evt_type == EXS_EVT_SEND);

			if((errno = local_vector.exs_evt_errno) != 0)
				ret = -1;
			else
			{	/* no error on send, get bytes written*/
				ret = local_vector.exs_evt_union
					.exs_evt_xfer.exs_evt_length;
			}
		}
	}

out1:
	if (local_qhandle) {
		our_put_to_queue_cache(this_conn, local_qhandle);
	}
out0:
	/* return < 0 on error, >= 0 if started (or completed) ok */
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_send */

static int
our_enqueue_send_seqpacket(struct exs_conn *this_conn, struct transaction_block *this,
		const void *buffer, size_t length)
{
	int ret;
	(void)buffer;
	(void)length;

	if (atomic_set_bits(SEND_READY_BIT, &this->send_state)
							& ADVERT_READY_BIT) {
		TRACE_DBG("send and advert ready on exs_send");
		STATISTICS_INCREMENT(matched_recv_await);

		ret = this_conn->ops->xmit_if_match(this_conn, this);
	} else {
		TRACE_DBG("send ready only on exs_send");
		STATISTICS_INCREMENT(unmatched_recv_await);
		ret = 0;
	}

	return ret;
}	/* our_enqueue_send_seqpacket */


/* when called, we know that:
 *	nbytes > 0
 *	length >= options->direct_minimum
 *	client_conn->send_offset == client_conn->take_offset
 *	client_conn->credit_count > 0
 *	client_conn->n_remote > 0
 *	stream_sync_mutex is locked
 *
 * returns < 0 if anything fails
 *	   >=0 number of bytes successfully sent directly
 */
static int
our_send_indirect(struct exs_conn *cb, struct transaction_block *block)
{
	struct ibv_mr		*send_unreg_mr;
	exs_mhandle_t		block_mhandle;
	void			*source;
	size_t			length;
	ssize_t			nbytes;
	size_t			padded_length;
	int			old_errno, offset, local_send_flags;

	source = block->user_buffer;
	length = block->user_length;
	offset = block->buf_offset;

	nbytes = length - offset;
	/* but don't send more than receiver can handle */
	if (nbytes > cb->remote_buf_n_empty) {
		TRACE_DBG("nbytes %zu > remote_buf_n_empty %u",
			nbytes, cb->remote_buf_n_empty);
		nbytes = cb->remote_buf_n_empty;
	}
	/* and don't go over the edge of the buffer */
	if (nbytes > (cb->remote_recv_buf.ad_buf_len
				- cb->remote_buf_offset)) {
		TRACE_DBG("nbytes %zu > remaining buffer space %u (offset %u)",
			nbytes, cb->remote_recv_buf.ad_buf_len
				- cb->remote_buf_offset,
			cb->remote_buf_offset);
		nbytes = cb->remote_recv_buf.ad_buf_len
			- cb->remote_buf_offset;
	}

	local_send_flags = IBV_SEND_SIGNALED;

	/* Must do this ahead of time */
	++block->outstanding_completion_count;
	if ((size_t)(nbytes + offset) >= length) {
		atomic_clear_bits(SEND_READY_BIT, &block->send_state);
	}

	TRACE_DBG("rdma_write %zd bytes, length %zu, local offset %u, remote offset %u, n_remote %u, seqno %" PRIu64,
		nbytes, length, offset, cb->remote_buf_offset, cb->remote_buf_n_empty,
		cb->send_seqno);
	block_mhandle = block->user_event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_mhandle;
	if ((nbytes = our_send_rdma_write_notify(cb, block,
				source + offset, nbytes,
				block_mhandle, NULL,
				&local_send_flags,
				&send_unreg_mr)) < 0) {
		--block->outstanding_completion_count;
		++cb->stream_wwi_counter;
		block->user_event_ptr->exs_evt_errno = EIO;
		atomic_set_bits(SEND_DONE_BIT, &block->send_state);
		old_errno = errno;
		our_all_done_with_block(cb, block);
		errno = old_errno;
		return -1;
	}
	if (send_unreg_mr) {
		block->data_mr = send_unreg_mr;
		block->flags |= EXS_MR_DYNAMIC;
	}
	if (!(local_send_flags & IBV_SEND_SIGNALED)) {
		if ((--block->outstanding_completion_count) == 0
				&& block->buf_offset >= length) {
			atomic_set_bits(SEND_DONE_BIT, &block->send_state);
			our_all_done_with_block(cb, block);
		}
	}
	STATISTICS_INCREMENT(indirect_rdma_write_blocks_sent);

	/* send directly from user source to remote buffer worked ok */
	TRACE_DBG("direct post_send %zd bytes", nbytes);

	padded_length = (cb->remote_buf_align == 0)
		? (size_t)nbytes
		: our_aligned_size(nbytes, cb->remote_buf_align);
	cb->remote_buf_offset += padded_length;
	if (cb->remote_buf_offset >= cb->remote_recv_buf.ad_buf_len) {
		cb->remote_buf_offset = 0;
	}
	cb->remote_buf_n_empty -= padded_length;
	STATISTICS_SET_MAX(max_remote_recv_buf_n_full,
		cb->remote_recv_buf.ad_buf_len - cb->remote_buf_n_empty);

	return nbytes;
}	/* our_send_indirect */


static int
our_enqueue_send_stream(struct exs_conn *cb, struct transaction_block *this,
		const void *buffer, size_t length)
{
	(void)buffer;
	(void)length;
	atomic_set_bits(SEND_READY_BIT, &this->send_state);
	return cb->ops->xmit_if_match(cb, this);
}	/* our_enqueue_send_stream */


/*
 *	exs_sendmsg - page 60 of Interconnect Software Consortium Issue 1.0
 *		asynchronously send a message on a socket
 *
 *	returns 0 if async op initiated ok,
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_sendmsg(int sockno, const exs_msghdr_t *message, int flags,
		exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	ssize_t		ret;
	struct evt_queue*qptr;
	struct exs_conn	*this_conn;

	TRACE_DBG("enter qhandle %p, flags 0x%x", (void *)qhandle, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if (_exs_global_flags & EXS_GLOBAL_BYPASS_PARAM_CHECKING) {
		goto no_parameter_checks;
	}

no_parameter_checks:
	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has already been connected */
	if(this_conn->conn_state != EXS_CONN_RUNNING)
	{
		/* Transport endpoint is not connected */
		errno = ENOTCONN;
		goto out;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	/* verify that caller is giving us a valid qhandle */
	if((qptr = _cnv_qnumb_to_ptr(qhandle, 0, 0)) == NULL)
	{	/* Invalid argument */
		goto out;
	}

	/* verify that flags are supported for this socket type or protocol */
	if(flags != 0)
	{
		errno = EOPNOTSUPP;	/* Operation not supported */
		goto out;
	}

out:
	ret = -1;
	errno = ENOSYS;		/* Function not implemented */
	(void)message;
	(void)ahandle;

	TRACE_DBG("finished, returning %zd",
		ret);

	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_sendmsg */


static size_t
our_immediate_recv(struct exs_conn *this_conn, void *buffer, size_t length,
								int flags)
{
	ssize_t ret;
	int send_flags;

	if (this_conn->is_seqpacket
			|| (flags & (EXS_BLOCK|MSG_WAITALL)) != EXS_BLOCK
			|| this_conn->stream_buf_size == 0) {
		return 0;
	}

	/* For stream copy mode, we may be able to avoid creating a
	 * transaction block altogether. */
	OUR_MEMBER_MUTEX_LOCK(this_conn, stream_sync_mutex);
	TRACE_DBG("this_conn %p stream_buf n_full %u",
				(void *)this_conn, this_conn->recv_buf.n_full);
	assert(this_conn->next_recv == this_conn->next_recv_primed);
	/* this will have to be put in a loop if we allow requests with
	 * MSG_WAITALL to use this */
	ret = our_stream_copy_recv(this_conn, buffer, 0, length);
	if (ret > 0) {
		this_conn->recv_seqno += ret;
		this_conn->source_seqno += ret;
		if ((this_conn->recv_buf.n_empty
					> this_conn->recv_buf.n_empty_threshold)
				|| (this_conn->n_unacked_wwi
					> this_conn->n_unacked_wwi_threshold)) {
			send_flags = 0;
			if (our_send_ack(this_conn, &send_flags) < 0) {
				if (errno != ECONNRESET) {
					OUR_PERROR(errno, "Could not post send of ACK");
				}
			}
		}
	}
	if (this_conn->conn_state != EXS_CONN_RUNNING
					&& this_conn->recv_buf.n_full == 0) {
		/* We are in the process of shutting down and have just drained
		 * the buffer, so prevent issuing any new exs_recv() operations
		 * now. */
		TRACE_DBG("this_conn %p conn_state %s unset EXS_OK_TO_ISSUE_RECV",
					(void *)this_conn,
					our_conn_state_str(this_conn->conn_state));
		atomic_clear_bits(EXS_OK_TO_ISSUE_RECV, &this_conn->conn_flags);
	}
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, stream_sync_mutex);

	return ret;
}	/* our_immediate_recv */


static ssize_t
our_start_recv(struct exs_conn *this_conn, void *buffer, size_t length,
			int flags, struct evt_queue *qptr, void *ahandle,
			exs_mhandle_t mhandle)
{
	struct transaction_block	*this;
	struct exs_event		*event_ptr;
	ssize_t				ret;

	OUR_MEMBER_MUTEX_LOCK(this_conn, advert_sequence_mutex);

	TRACE_DBG("socket %d this_conn %p state %s qptr %p flags %#08x length %zu",
		this_conn->sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state),
		(void *)qptr, flags, length);

	ret = -1;
	this = this_conn->next_recv;

	if ((ret = our_try_get_next_block(this_conn, this, flags)) < 0) {
		errno = -ret;
		ret = -1;
		goto exit0;
	}

	ret = _get_empty_evt_queue_event(qptr, &event_ptr);
	if (ret < 0) {
		errno = -ret;
		ret = -1;
		goto exit1;
	}
	assert(event_ptr != NULL || (flags & EXS_UNSIGNALED));

	atomic_add(1, &this_conn->pending_recv_count);

	/* set block's flags field with flags from the exs_recv */
	this->flags = flags;

	/* fill in transaction's fields with parameters from the exs_recv */
	this->in_progress = 1;
	this->send_state = 0;
	this->qhandle = qptr;
	this->mhandle = mhandle;
	this->user_buffer = buffer;
	this->user_length = length;
	this->recv_byte_len = 0;
	if (event_ptr) {
		event_ptr->exs_evt_type = EXS_EVT_RECV;
		event_ptr->exs_evt_errno = 0;
		event_ptr->exs_evt_ahandle = ahandle;
		event_ptr->exs_evt_socket = this_conn->sockno;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_buffer = buffer;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_length = length;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_mhandle = mhandle;
		event_ptr->exs_evt_union.exs_evt_xfer.exs_evt_buffer_length = 0;
		this->user_event_ptr = event_ptr;
	}

	if (!(this_conn->conn_flags & EXS_OK_TO_ISSUE_RECV)) {
		if ((this_conn->conn_state == EXS_CONN_TERMINATED
				|| this_conn->conn_state == EXS_CONN_TERMINATING)
				&& (this_conn->conn_flags & EXS_NEED_EOF_RECV)) {
			errno = ECONNRESET;
			goto exit1;
		}
		/* Normal connection shutdown; post asynchronous event to user
		 * with length == 0 bytes */
		ret = our_exs_recv_done(this_conn, this);
	} else if (this_conn->is_seqpacket) {
		ret = our_send_advert(this_conn, this);
	} else {
		OUR_MEMBER_MUTEX_LOCK(this_conn, stream_sync_mutex);
		atomic_set_bits(RECV_READY_BIT, &this->send_state);
		if (this_conn->recv_buf.n_full == 0) {
			STATISTICS_INCREMENT(recv_buf_empty_on_recv);
		}
		ret = our_stream_recv_loop(this_conn);
		OUR_MEMBER_MUTEX_UNLOCK(this_conn, stream_sync_mutex);
	}

	if (ret < 0) {
		goto exit1;
	}

	TRACE_DBG("finished ok, ret %zd",
		ret);
	ret = 0;
	this_conn->next_recv = this->next;
	goto exit0;

exit1:
	this->in_progress = 0;
	if (this->user_event_ptr) {
		_put_empty_evt_queue_event(this->qhandle, this->user_event_ptr);
		this->user_event_ptr = NULL;
	}
	OUR_MEMBER_SEM_POST(this, sem);
exit0:
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, advert_sequence_mutex);
	return ret;
}	/* our_start_recv */


/*
 *	exs_recv - page 42 of Interconnect Software Consortium Issue 1.0
 *		asynchronously receive a message from a connected socket
 *
 *	returns	== 0  if async op initiated ok,
 *		>= 0  if sync op finished ok (value is no. of bytes read),
 *		== -1 if error (errno set to error value)
 *
 *	only flags values accepted:
 *
 *		EXS_DONTWAIT		(nonstandard)
 *			without this flag, this function may block until a
 *				posted_recv_slot becomes available (happens when
 *				user has too many outstanding exs_recv calls)
 *			with this flag, this function will return EAGAIN error
 *				if no posted_recv_slot is available
 *			with or without this flag, if this function returns 0
 *				then the data has not been received until the
 *				completion event has been posted to qhandle.
 *		EXS_BLOCK		(nonstandard)
 *			force this recv to block until it completes.  The
 *			qhandle and ahandle parameters are ignored.
 *		EXS_UNSIGNALED		(nonstandard)
 *			don't cause a completion event when this send completes.
 *			The qhandle and ahandle parameters are ignored.
 *
 *	qhandle values accepted:
 *		NULL
 *			cause an error
 *		EXS_QHANDLE_INVALID
 *			left over from a previous error, cause a new error
 *		anything else
 *			assume it is a user-created queue
 *
 *	this may cause the completion event to be posted before returning
 */
EXS_EXPORT ssize_t
exs_recv(int sockno, void *buffer, size_t length, int flags,
		exs_qhandle_t qhandle, exs_ahandle_t ahandle,
		exs_mhandle_t mhandle)
{
	ssize_t				ret;
	struct evt_queue		*qptr;
	struct evt_queue		*local_qhandle;
	struct exs_conn			*this_conn;
	exs_event_t			local_vector;

	TRACE_DBG("qhandle %p, length %zu, flags 0x%x",
						(void *)qhandle, length, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}
	if (_exs_global_flags & EXS_GLOBAL_BYPASS_PARAM_CHECKING) {
		goto no_parameter_checks;
	}

	/* verify that caller is giving us either a valid qhandle or NULL */
	if(qhandle == EXS_QHANDLE_INVALID)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* verify that flags are supported for this socket type or protocol */
	if((flags & ~(EXS_DONTWAIT|EXS_CREDIT_WAIT|EXS_BLOCK
					|EXS_UNSIGNALED|MSG_WAITALL)) != 0)
	{
		errno = EOPNOTSUPP;	/* Operation not supported */
		return -1;
	}

	/* if user specified a NULL qhandle, then ensure that either EXS_BLOCK
	 * or EXS_UNSIGNALED flag was specified. */
	if (qhandle == NULL && !(flags & (EXS_BLOCK|EXS_UNSIGNALED))) {
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* verify that length is at least 1 */
	if(length <= 0 || length > MAX_EXS_MESSAGE_SIZE)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

	/* ensure that buffer is not NULL,
	 * TO-DO -- and is not an outstanding recycle buffer,
	 * since that leads to a race condition
	 */
	if(buffer == NULL)
	{
		errno = EINVAL;		/* Invalid argument */
		return -1;
	}

no_parameter_checks:
	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	if ((flags & MSG_WAITALL) && this_conn->is_seqpacket) {
		/* This flag does not apply to a message-based socket, so we are
		 * allowed to ignore it. */
		flags &= ~MSG_WAITALL;
	}

	ret = -1;			/* assume failure */
	local_qhandle = NULL;		/* assume this is not created locally */

	if (!(this_conn->conn_flags & EXS_OK_TO_ISSUE_RECV)) {
		if ((this_conn->conn_state > EXS_CONN_FINISHED
				&& this_conn->conn_state < EXS_CONN_RUNNING)
				|| this_conn->conn_state == EXS_CONN_LISTENING){
			errno = ENOTCONN;
			goto out0;
		} else if ((this_conn->conn_state == EXS_CONN_TERMINATING
				|| this_conn->conn_state == EXS_CONN_TERMINATED)
				&& (this_conn->conn_flags & EXS_NEED_EOF)) {
			errno = ECONNRESET;
			goto out0;
		} else if (flags & EXS_BLOCK) {
			ret = 0;
			goto out0;
		}
		/* do not return error if we got or sent EOF on an asynchronous
		 * exs_recv; we will indicate EOF to the user asynchronously */
	}

	/* see if user gave us an event queue or wants a blocking send.  After
	 * this, qptr will be set or this function will have returned an error
	 * condition. */
	if (flags & EXS_BLOCK) {
		/* first try an immediate recv from the local stream recv
		 * buffer */
		ret = our_immediate_recv(this_conn, buffer, length, flags);
		if (ret != 0) {
			goto out0;
		}

		/* get qhandle from cache, dynamically creating one if needed */
		local_qhandle = our_get_from_queue_cache(this_conn);
		if(local_qhandle == NULL)
		{	/* couldn't create it, errno already set */
			ret = -1;
			goto out0;
		}
		qptr = local_qhandle;
	} else if (qhandle == NULL) {
		/* caller gave us no qhandle, so don't use one */
		qptr = NULL;
	} else if ((qptr = _cnv_qnumb_to_ptr(qhandle, 1,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* Invalid argument */
		errno = EINVAL;
		goto out0;
	}

	ret = our_start_recv(this_conn, buffer, length, flags, qptr,
							ahandle, mhandle);
	if (ret < 0 || !local_qhandle) {
		goto out1;
	}

	/* caller wants a blocking (synchronous) recv */

	TRACE_DBG("blocking recv, local_qhandle %p, ahandle %p, buffer %p",
		(void *)local_qhandle, ahandle, buffer);

	/* recv was started ok, now wait for it to complete */
	ret = _evt_queue_dequeue_vector(local_qhandle, &local_vector, 1, NULL);

	TRACE_DBG("back from dequeue, local_qhandle %p, ahandle %p, buffer %p, ret %ld, exs_evt_type %d, exs_evt_errno %d, exs_evt_ahandle %p, exs_evt_length %ld, exs_evt_buffer %p",
		(void *)local_qhandle, ahandle, buffer, ret,
		local_vector.exs_evt_type,
		local_vector.exs_evt_errno,
		local_vector.exs_evt_ahandle,
		local_vector.exs_evt_union.exs_evt_xfer
					.exs_evt_length,
		local_vector.exs_evt_union.exs_evt_xfer
					.exs_evt_buffer);

	if(ret != -1)
	{	/* no error on dequeue itself check completion */
		assert(local_vector.exs_evt_type == EXS_EVT_RECV);

		if((errno = local_vector.exs_evt_errno) != 0)
			ret = -1;
		else
		{	/* no error on recv, get bytes read */
			ret = local_vector.exs_evt_union
				.exs_evt_xfer.exs_evt_length;
		}
	}

out1:
	if (local_qhandle) {
		/* always return the locally created queue to the connection's cache */
		our_put_to_queue_cache(this_conn, local_qhandle);
	} else if (qptr && ret < 0) {
		qptr = _evt_queue_unref(qptr, 1, EVT_QUEUE_REF_PENDING);
	}

out0:
	/* return < 0 on error, >= 0 if completed ok */
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_recv */


/*
 *	exs_recvmsg - page 46 of Interconnect Software Consortium Issue 1.0
 *		asynchronously receive a message from a socket
 *
 *	returns 0 if async op initiated ok,
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_recvmsg(int sockno, exs_msghdr_t *message, int flags,
		exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	ssize_t		ret;
	struct evt_queue*qptr;
	struct exs_conn	*this_conn;

	TRACE_DBG("enter qhandle %p, flags 0x%x", (void *)qhandle, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if (_exs_global_flags & EXS_GLOBAL_BYPASS_PARAM_CHECKING) {
		goto no_parameter_checks;
	}

	/* verify that flags are supported for this socket type or protocol */
	if(flags != 0)
	{
		errno = EOPNOTSUPP;	/* Operation not supported */
		return -1;
	}

no_parameter_checks:
	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has already been connected */
	if(this_conn->conn_state != EXS_CONN_RUNNING)
	{
		if (this_conn->conn_state == EXS_CONN_TERMINATING)
		{	/* Connection reset by peer */
			errno = ECONNRESET;
		}
		else
		{	/* Transport endpoint is not connected */
			errno = ENOTCONN;
		}
		goto out;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	/* verify that caller is giving us a valid qhandle */
	if((qptr = _cnv_qnumb_to_ptr(qhandle, 0, 0)) == NULL)
	{	/* Invalid argument */
		goto out;
	}

	/*****
	ret = our_try_to_post_recv_block(cb, block);
	*****/

out:
	ret = -1;
	errno = ENOSYS;		/* Function not implemented */
	(void)message;
	(void)ahandle;

	TRACE_DBG("finished, returning %zd", ret);

	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_recvmsg */


/*
 *	exs_blocking_recv() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns >=0 as amount read ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_blocking_recv(int sockno, void *buffer, size_t max_bytes, int flags,
			exs_mhandle_t mhandle)
{

	return exs_recv(sockno, buffer, max_bytes, flags | EXS_BLOCK, NULL,
							NULL, mhandle);
}	/* exs_blocking_recv */


/*
 *	exs_read() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns >=0 as amount read ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_read(int sockno, void *buffer, size_t max_bytes)
{
	return exs_recv(sockno, buffer, max_bytes, EXS_BLOCK, NULL,
					NULL, EXS_MHANDLE_UNREGISTERED);
}	/* exs_read */


/*
 *	exs_blocking_send() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns >=0 as amount written ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_blocking_send(int sockno, void *buffer, size_t max_bytes, int flags,
			exs_mhandle_t mhandle)
{
	return exs_send(sockno, buffer, max_bytes, flags | EXS_BLOCK, NULL,
							NULL, mhandle);
}	/* exs_blocking_send */


/*
 *	exs_write() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns >=0 as amount written ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_write(int sockno, void *buffer, size_t max_bytes)
{
	return exs_send(sockno, buffer, max_bytes, EXS_BLOCK, NULL,
					NULL, EXS_MHANDLE_UNREGISTERED);
}	/* exs_write */


/*
 *	exs_sendfile - page 55 of Interconnect Software Consortium Issue 1.0
 *		initiate transmission of the contents of a file from a socket
 *
 *	returns 0 if async op initiated ok,
 *		else -1 with errno set to error value
 */
EXS_EXPORT ssize_t
exs_sendfile(int sockno, exs_xferfile_t *sendvec, int sendvec_cnt,
		int flags, exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	int				i;
	struct exs_conn			*this_conn;

	TRACE_DBG("enter qhandle %p, sendvec_cnt %d, flags 0x%x",
					(void *)qhandle, sendvec_cnt, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	if (_exs_global_flags & EXS_GLOBAL_BYPASS_PARAM_CHECKING) {
		goto no_parameter_checks;
	}

	/* verify that this socket has already been connected */
	if(this_conn->conn_state != EXS_CONN_RUNNING)
	{
		/* Transport endpoint is not connected */
		errno = ENOTCONN;
		goto err0;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	/* verify that flags are supported for this socket type or protocol */
	if(flags != 0)
	{
		errno = EOPNOTSUPP;	/* Operation not supported */
		goto err0;
	}

	/* verify that sendvec_cnt does not exceed the dynamic number of
	 * slots currently available
	 */
	if(sendvec_cnt <= 0)
	{
		errno = EINVAL;		/* Invalid argument */
		goto err0;
	}

no_parameter_checks:
	(void)qhandle;
	(void)sendvec;
	(void)ahandle;
	errno = ENOSYS;
	goto err0;

err0:
	i = errno;
	OUR_PERROR(i, "finished, this_conn %p, state %s, error",
		(void *)this_conn,
		our_conn_state_str(this_conn->conn_state));
	this_conn = our_socket_unref(this_conn);
	errno = i;
	return -1;
}	/* exs_sendfile */


static void
our_flush_stranded_send_blocks(struct exs_conn *this_conn)
{
	struct transaction_block*first, *block;
	int errcode;

	if ((this_conn->conn_state == EXS_CONN_TERMINATED
			|| this_conn->conn_state == EXS_CONN_TERMINATING)
			&& (this_conn->conn_flags & EXS_NEED_EOF_SEND)) {
		errcode = ECONNRESET;
	} else {
		errcode = EPIPE;
	}

	OUR_MEMBER_MUTEX_LOCK(this_conn, data_sequence_mutex);
	block = first = this_conn->next_send;
	do {
#if defined(ALLOW_EXS_TRACE)
		int sval;
		sem_getvalue(&block->sem, &sval);
		TRACE_DBG("block %p in_progress %d sem_value %d",
			(void *)block, block->in_progress, sval);
#endif

		if (! block->in_progress) {
			continue;
		}

		if (block->user_event_ptr) {
			/* report back to user "Software caused connection abort" */
			block->user_event_ptr->exs_evt_errno = errcode;

			/* report back to user that no bytes were transfered */
			/* FIXME: what if bytes *were* transferred? */
			block->user_event_ptr->exs_evt_union
					.exs_evt_xfer.exs_evt_length = 0;
		}

		block->outstanding_completion_count = 0;
		our_all_done_with_block(this_conn, block);
	} while ((block = block->next) != first);
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, data_sequence_mutex);
}	/* our_flush_stranded_send_blocks */


static void
our_flush_all_stranded_recv_blocks(struct exs_conn *this_conn)
{
	struct transaction_block*last, *block;

	/* FIXME: printing pthread_t nonportable */
	TRACE_DBG("[%lu]: conn %p, conn_state %s, conn_flags %0x",
			pthread_self(), (void *)this_conn,
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);

	OUR_MEMBER_MUTEX_LOCK(this_conn, advert_sequence_mutex);
	block = this_conn->next_recv_primed;
	last = this_conn->next_recv;
	if (block->in_progress) {
		do {
#if defined(ALLOW_EXS_TRACE)
			int sval;
			sem_getvalue(&block->sem, &sval);
			TRACE_DBG("block %p in_progress %d sem_value %d",
					(void *)block,
					block->in_progress,
					sval);
#endif

			if (!this_conn->is_seqpacket
					&& this_conn->stream_buf_size > 0
					&& this_conn->recv_buf.n_full > 0) {
				OUR_MEMBER_MUTEX_LOCK(this_conn,
						stream_sync_mutex);
				our_stream_recv_loop(this_conn);
				OUR_MEMBER_MUTEX_UNLOCK(this_conn,
						stream_sync_mutex);
			} else {
				/* report error back to user if the connection
				 * was reset */
				if ((this_conn->conn_state
						== EXS_CONN_TERMINATING
						|| this_conn->conn_state
						== EXS_CONN_TERMINATED)
						&& (this_conn->conn_flags
							& EXS_NEED_EOF)
						&& block->user_event_ptr) {
					block->user_event_ptr->exs_evt_errno
								= ECONNRESET;
				}

				our_exs_recv_done(this_conn, block);
			}
		} while ((block = block->next) != last && block->in_progress);
	}
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, advert_sequence_mutex);
}	/* our_flush_all_stranded_recv_blocks */


/*
 *	exs_poll - page 27 of Interconnect Software Consortium Issue 1.0
 *		modify the set of monitored conditions for multiple sockets
 *		to trigger asynchronous notification events on a specified event
 *
 *	returns nfds if all ok,
 *	else number of entries processed ok with errno set to error value
 */
EXS_EXPORT nfds_t
exs_poll(exs_pollfd_t fds[], nfds_t nfds, int flags, exs_qhandle_t qhandle)
{
	struct evt_queue *qptr;

	TRACE_DBG("enter qhandle %p, nfds %lu, flags 0x%x",
						(void *)qhandle, nfds, flags);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if(flags != 0)
	{
		errno = ENOTSUP;	/* Not supported */
		return -1;
	}

	/* verify that caller is giving us a valid qhandle */
	if((qptr = _cnv_qnumb_to_ptr(qhandle, 0, 0)) == NULL)
	{	/* Invalid argument */
		return -1;
	}

	/* all user-supplied parameters checked ok */

	(void)fds;
	(void)nfds;
	errno = ENOSYS;
	return -1;
}	/* exs_poll */


static void
our_free_conn_requested_list(struct our_list_head *this_list)
{
	struct conn_requested	*requested;

	while ((requested = exs_p_list_del_head(this_list))) {
		EXS_FREE(requested);
	}
	exs_p_list_destroy(this_list);
}	/* our_free_conn_requested_list */


/* called only from our_socket_free()
 *
 * this is the counterpart of our_init_pending()
 *
 * these lists should be non-empty only for listening_sockets
 * that did not do everything to completion
 */
static void
our_free_all_pending_lists(struct exs_conn *this_conn)
{
	/* free list that holds unused "struct pending_accept"
	 * for this listening socket
	 */
	our_free_new_accept_list(this_conn,
			&this_conn->pending_accept_list, 1);

	/* init 2 lists to track connect_request events (handled in
	 * our_con_man_thread()) with no pending accepts to match
	 */
	our_free_conn_requested_list(&this_conn->conn_requested_list);
	our_free_conn_requested_list(&this_conn->conn_requested_free_list);
}	/* our_free_all_pending_lists */


/*
 *	exs_socket() - not in Interconnect Software Consortium Issue 1.0
 *		       was added by us
 *
 *	returns >=0 fd of newly opened socket
 *			whose conn_state is EXS_CONN_UNBOUND
 *		else -1 with errno set to error value
 *
 *	what it does:
 *		checks for acceptable parameters
 *		creates conventional socket to give us an fd
 *		allocates and initializes new struct exs_conn for this socket
 *		creates new rdma event channel for this socket
 *		creates new "connection manager" (cm) id for this event channel
 *		add new struct exs_conn to master global list "conn_list"
 *			of all known connections of any type in any state
 *		creates instance of our_con_man_thread to deal with
 *			asynch cm events on this struct exs_conn
 */
EXS_EXPORT int
exs_socket(int family, int socktype, int protocol)
{
	int			fd = -1, ret;
	struct exs_conn		*new_conn = NULL;

	TRACE_DBG("enter family %d, socktype %d, protocol %d",
						family, socktype, protocol);

	if (!our_check_init()) {
		ret = EPERM;		/* Operation not permitted */
		goto err0;
	}

	if (family != AF_INET && family != AF_INET6) {
		ret = EAFNOSUPPORT;
		goto err0;
	}

	if (socktype != SOCK_SEQPACKET && socktype != SOCK_STREAM) {
		ret = ESOCKTNOSUPPORT;
		goto err0;
	}

	if (protocol != 0 && ((socktype == SOCK_STREAM
					&& protocol != IPPROTO_TCP)
				|| (socktype == SOCK_SEQPACKET
					&& protocol != IPPROTO_SCTP))) {
		ret = EPROTONOSUPPORT;
		goto err0;
	}

	/* allocates a new struct exs_conn, clears most fields to 0 or NULL,
	 * and initializes its state field to EXS_CONN_UNOPENED
	 */
	if((new_conn = our_socket_new()) == NULL)
	{
		ret = errno;
		goto err0;
	}

	/* set up the defaults for client to use in connection negotiation
	 * listeners don't use these directly, but every
	 * server agent inherits these defaults from its listener parent
	 */
	our_set_send_credit_limits(new_conn, DEFAULT_FLOW_CONTROL_CREDITS);
	new_conn->small_packet_max_size = SMALL_PACKET_MAX_SIZE;

	/* initialize completion condition variables */
	if((ret = our_init_completion_synch(new_conn)))
	{
		goto err;
	}

	new_conn->event_channel = rdma_create_event_channel();
	if(new_conn->event_channel == NULL)
	{
		ret = errno;
		OUR_PERROR(ret, "rdma_create_event_channel failed");
		goto err;
	}
	/* save the user's fd in this newly allocated structure */
	new_conn->sockno = fd = new_conn->event_channel->fd;

	/* redundant, but makes it easier to post exs_event_t events */
	new_conn->conn_pending->sockno = fd;

	TRACE_DBG("ALLOCATE %zu bytes for event_channel %p on socket %d",
		sizeof(*new_conn->event_channel),
		(void *)new_conn->event_channel, fd);

	/* Allocate a "communication identifier" pointer, which is the rdma
	 * equivalent of a socket fd.
	 *	This returns the new pointer in "cm_id".
	 *	Events associated with this "cm_id" will be reported on
	 *		the previously (above) created "event_channel".
	 *	Our unique tag associated with this "cm_id" will be "new_conn".
	 *	The "port space" to be used with this "cm_id" is TCP because
	 *		we want to use it for TCP connections.
	 *			Port space possibilities are:
	 *				RDMA_PS_TCP	TCP
	 *				RDMA_PS_UDP	UDP
	 */
	ret = rdma_create_id(new_conn->event_channel, &new_conn->cm_id,
						new_conn, RDMA_PS_TCP);
	if(ret)
	{
		OUR_PERROR(errno, "rdma_create_id failed");
		goto err;
	}
	TRACE_DBG("ALLOCATE %zu bytes for cm_id %p on socket %d",
		sizeof(*new_conn->cm_id), (void *)new_conn->cm_id,
		fd);

	new_conn->conn_state = EXS_CONN_UNBOUND;

	if(socktype == SOCK_SEQPACKET)
	{	/* for SOCK_SEQPACKET connections, message boundaries must be
	`	 * preserved.  Therefore, no nagling on sending side, and on
		 * receiving side short receives truncate long sends
		 */
		new_conn->fd_flags = EXS_FD_SP_COPY_UNREG|EXS_FD_NODELAY;
		new_conn->is_seqpacket = 0xff;
		new_conn->ops = &ops_seqpacket;
	} else {
		new_conn->fd_flags = 0;
		new_conn->ops = &ops_stream;
	}

	new_conn->comp_thread_cpu = INT_MAX;

	/* create our_con_man_thread to pick up asynch cm events */
	ret = create_our_con_man_thread(new_conn);
	if(ret)
	{
		goto err;
	}

	/* socket is completely set up, make it available to users
	 * refcount set to two: 1 for user and 1 for con man thread
	 * user ref is only decremented on exs_close(), so can never
	 * go to 0 without user calling exs_close() */
	our_add_socket_to_list(new_conn);

	TRACE_DBG("finished, fd %d, new_conn %p, state %s",
		fd, (void *)new_conn,
		our_conn_state_str(new_conn->conn_state));
	return fd;

err0:
	OUR_PERROR(ret, "failed, fd %d, new_conn %p, state %s",
		fd, (void *)new_conn,
		our_conn_state_str(new_conn->conn_state));

	if (ret)
		errno = ret;
	return fd;

err:
	new_conn = our_socket_unref(new_conn);
	fd = -1;
	goto err0;
}	/* exs_socket */


/*
 *	exs_bind() - not in Interconnect Software Consortium Issue 1.0
 *		       was added by us
 *
 *	returns 0 on success
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_bind(int fd, struct sockaddr *address, socklen_t addrlen)
{
	int		ret;
	struct exs_conn	*this_conn;

	TRACE_DBG("enter fd %d", fd);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* lookup fd in table */
	if((this_conn = our_lookup_socket(fd)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		fd, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	/* verify that this socket has not already been bound */
	if(this_conn->conn_state != EXS_CONN_UNBOUND)
	{
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
		goto out2;
	}

	/* We lock event_mutex to protect conn_state.
	 * At first glance this seems excessive, but at this time
	 * nothing else should be happening on this listener or client socket,
	 * so there should be no contention for this lock --
	 * we lock it to explicitly prevent anything else from happening!
	 */

	OUR_MEMBER_MUTEX_LOCK(this_conn, event_mutex);

	/* check again that this socket has not already been bound */
	if(this_conn->conn_state != EXS_CONN_UNBOUND)
	{
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
	}
	else
	{
		(void)addrlen; /* not required by rdma_bind_addr */
		ret = rdma_bind_addr(this_conn->cm_id, address);
		if(ret)
		{
			ret = errno;
			OUR_PERROR(ret, "rdma_bind_addr failed");
			errno = ret;
			ret = -1;
		}
		else
		{	/* the bind worked ok, advance this socket's state */
			this_conn->conn_state = EXS_CONN_BOUND;
		}
	}

	OUR_MEMBER_MUTEX_UNLOCK(this_conn, event_mutex);

out2:
	TRACE_DBG("finished, this_conn %p, state %s, ret %d",
		(void *)this_conn,
		our_conn_state_str(this_conn->conn_state),
		ret);
	this_conn = our_socket_unref(this_conn);

	return ret;
}	/* exs_bind */


/*
 *	exs_listen() - not in Interconnect Software Consortium Issue 1.0
 *		       was added by us
 *
 *	returns 0 on success
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_listen(int fd, int backlog)
{
	int		ret;
	struct exs_conn	*this_conn;

	TRACE_DBG("enter fd %d, backlog %d", fd, backlog);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that backlog is not negative */
	if(backlog < 0) {
		errno = EINVAL;	  /* Invalid argument */
		return -1;
	}

	/* lookup fd in table and
	 * verify that this socket has been created and optionally
	 * has been bound to the listening address
	 */
	if((this_conn = our_lookup_socket(fd)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		fd, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	/* verify that this socket has not already been connected or listened */
	if(this_conn->conn_state != EXS_CONN_UNBOUND
				&& this_conn->conn_state != EXS_CONN_BOUND)
	{
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
		goto out;
	}

	/* We lock event_mutex to protect conn_state.
	 * At first glance this seems excessive, but at this time
	 * nothing else should be happening on this listener socket,
	 * so there should be no contention for this lock --
	 * we lock it to explicitly prevent anything else from happening!
	 */

	OUR_MEMBER_MUTEX_LOCK(this_conn, event_mutex);

	/* check again that this socket has not gone beyond bound */
	if(this_conn->conn_state != EXS_CONN_UNBOUND
				&& this_conn->conn_state != EXS_CONN_BOUND)
	{
		errno = EINVAL;   /* Invalid argument */
		ret = -1;
	}
	else
	{
		/* the pending and conn_requested lists were already inited
		 * when this_conn (and every struct exs_conn) was allocated.
		 */

		ret = rdma_listen(this_conn->cm_id, backlog);
		if(ret)
		{
			ret = errno;
			OUR_PERROR(ret, "rdma_listen failed");
			errno = ret;
			ret = -1;
		}
		else
		{	/* the listen worked ok, advance the conn_state */
			this_conn->conn_state = EXS_CONN_LISTENING;
			this_conn->backlog_limit = backlog;
		}
	}

	OUR_MEMBER_MUTEX_UNLOCK(this_conn, event_mutex);

out:
	TRACE_DBG("finished, fd %d, this_conn %p, state %s, ret %d",
		fd, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state),
		ret);
	this_conn = our_socket_unref(this_conn);

	return ret;
}	/* exs_listen */


/*
 * called from our_start_close()
 * called only in a user thread context
 *
 * when called, parent_conn->event_mutex MUST be locked
 *	to avoid a race for it with connection manager thread
 *	this mutex protects conn_state
 *
 * Send an EOF message iff there are no pending send operations (if there are,
 * we wait for them to complete).  When the completion thread then gets a RECV
 * EOF completion, the completion thread will then issue rdma_disconnect() and
 * wake up the cm event thread.
 *
 * The connection is expected to be in state RUNNING when this function is
 * called.  The connection state will be set to CLOSING if the EOF was sent and
 * DELAY_CLOSING if the EOF was not sent.
 *
 * returns 0 and changes state if our_send_eof started ok,
 * returns -1, sets errno and leaves state unchanged if our_send_eof failed
 */
static int
our_try_to_send_eof(struct exs_conn *this_conn, struct transaction_block *block)
{
	int	send_flags;
	int	ret = 0;

	our_clear_conn_flags(this_conn, EXS_NEED_LOCAL_CLOSE
					| EXS_OK_TO_ISSUE_IO);
	if(this_conn->pending_send_count <= 0) {
		/* no send blocks are posted,
		 * ok to trigger local disconnect now
		 */

		TRACE_DBG("calling final our_send_eof for cb %p, cm_id %p",
				(void *)this_conn, (void *)this_conn->cm_id);

		send_flags = IBV_SEND_SIGNALED;
		ret = our_send_eof(this_conn, block,
					&send_flags, SHUT_RDWR);
		if (ret < 0 && errno == EBUSY) {
			ret = 0;
		}

		/* this connection has to be finished in the con-man-thread */
		this_conn->conn_state = EXS_CONN_CLOSING;
	} else {
		TRACE_DBG("conn %p socket %d: avoid calling our_send_eof because pending_send_count(%d) > 0, conn_state %s, conn_flags %06x",
			(void *)this_conn, this_conn->sockno,
			this_conn->pending_send_count,
			our_conn_state_str(this_conn->conn_state),
			this_conn->conn_flags);

		/* don't issue local rdma_disconnect until all posted sends complete */
		this_conn->conn_state = EXS_CONN_DELAY_CLOSING;
	}

	return ret;
}	/* our_try_to_send_eof */


/* Called from our_start_close() and our_start_shutdown() to close the RDMA
 * connection.  event_mutex MUST be locked prior to calling this function. */
static int
our_shutdown_conn(struct exs_conn *this_conn, struct transaction_block *block)
{
	int next_state;
	int ret = 0;

	TRACE_DBG("start, this_conn %p, conn_state %s",
					(void *)this_conn,
					our_conn_state_str(this_conn->conn_state));
	switch(this_conn->conn_state) {

	case EXS_CONN_CLOSING:
	case EXS_CONN_DELAY_CLOSING:
	case EXS_CONN_UNLISTENING:
	case EXS_CONN_ACCEPTING:
	case EXS_CONN_FINISHED:
	case EXS_CONN_TERMINATED:
	case EXS_CONN_UNOPENED:
	case EXS_CONN_CLOSED:
		/* Nothing to do other than unref the socket */
		break;

	case EXS_CONN_RUNNING:
		our_try_to_send_eof(this_conn, block);
		break;
	case EXS_CONN_TERMINATING:
		/* remotely triggered EVENT_DISCONNECT was last operation
		 * prior to this explicit call to exs_close()
		 *
		 * The next state is terminated if an EOF was not sent prior to
		 * the disconnect, or finished if an EOF was sent prior to the
		 * disconnect.
		 */
		next_state = (this_conn->conn_flags & EXS_NEED_EOF)
			? EXS_CONN_TERMINATED : EXS_CONN_FINISHED;
		this_conn->conn_state = next_state;
		this_conn->conn_pending->old_state = next_state;
		our_clear_conn_flags(this_conn, EXS_NEED_LOCAL_CLOSE
							| EXS_OK_TO_ISSUE_IO);
		break;

	case EXS_CONN_CONNECTING:
		/* rdma_connect() during exs_connect was last operation
		 * so asynchronous activity on connection still expected!
		 * but it doesn't matter, since we terminate the con_man thread
		 * before going further */
		our_terminate_con_man_thread(this_conn);
		our_connect_returning(this_conn, 5, ECONNABORTED);
		break;
	case EXS_CONN_ROUTING:
		/* rdma_resolve_route() during exs_connect was last operation
		 * so asynchronous activity on connection still expected!
		 * but it doesn't matter, since we terminate the con_man thread
		 * before going further */
		/* fall through */
	case EXS_CONN_ADDRESSING:
		/* rdma_resolve_addr() during exs_connect was last operation
		 * so asynchronous activity on connection still expected!
		 * but it doesn't matter, since we terminate the con_man thread
		 * before going further */
		our_terminate_con_man_thread(this_conn);
		our_connect_returning(this_conn, 1, ECONNABORTED);
		break;
	case EXS_CONN_LISTENING:
		/* exs_listen() was last successful operation */
		this_conn->conn_state = EXS_CONN_UNLISTENING;
	case EXS_CONN_BOUND:
		/* exs_bind() was last successful operation */
	case EXS_CONN_UNBOUND:
		/* exs_socket() was last successful operation */
		our_terminate_con_man_thread(this_conn);
		break;

	case EXS_CONN_CLIENT_SETUP:
		/* CHANGE THIS -- work it out later */
	case EXS_CONN_AGENT_SETUP:
		/* CHANGE THIS -- work it out later */

		/* rdma_accept() was last successful con_man operation
		 * (remote side has not yet caused RDMA_CM_EVENT_DISCONNECTED)
		 * so rdma_disconnect() must be called by exs_close() here
		 */

		if (atomic_clear_bits(EXS_NEED_LOCAL_CLOSE|EXS_NEED_DISCONNECT,
					&this_conn->conn_flags)
				& EXS_NEED_DISCONNECT) {
			our_rdma_disconnect(this_conn);
		}
		break;
	}	/* switch */

	TRACE_DBG("finished, this_conn %p, returning %d, conn_state %s",
					(void *)this_conn, ret,
					our_conn_state_str(this_conn->conn_state));
	return ret;
}	/* our_shutdown_conn */


static int
our_start_shutdown(struct exs_conn *this_conn, int how, int flags,
		struct evt_queue *qptr, void *ahandle)
{
	struct transaction_block *block;
	int			 prev_conn_flags;
	int			 delta_conn_flags;
	int			 ret;
	(void)flags;

	TRACE_DBG("socket %d, this_conn %p, pending_recv_count %d, pending_send_count %d, conn_state %s, conn_flags %#x",
		this_conn->sockno, (void *)this_conn,
		this_conn->pending_recv_count,
		this_conn->pending_send_count,
		our_conn_state_str(this_conn->conn_state),
		this_conn->conn_flags);

	/* Ensure that we are the only thread attempting a shutdown of this
	 * connection */
	OUR_MEMBER_MUTEX_LOCK(this_conn, event_mutex);

	switch (how) {
	case SHUT_RD:
		prev_conn_flags = our_clear_conn_flags(this_conn,
					EXS_OK_TO_ISSUE_RECV);
		delta_conn_flags = prev_conn_flags & EXS_OK_TO_ISSUE_RECV;
		break;
	case SHUT_WR:
		prev_conn_flags = our_clear_conn_flags(this_conn,
					EXS_OK_TO_ISSUE_SEND);
		delta_conn_flags = prev_conn_flags & EXS_OK_TO_ISSUE_SEND;
		break;
	case SHUT_RDWR:
		prev_conn_flags = our_clear_conn_flags(this_conn,
					EXS_OK_TO_ISSUE_IO);
		delta_conn_flags = prev_conn_flags & EXS_OK_TO_ISSUE_IO;
		break;
	default:
		errno = EINVAL;
		ret = -1;
		goto out;
	}

	if (!delta_conn_flags) {
		if (this_conn->eof_block_array[0].in_progress
				|| this_conn->eof_block_array[1].in_progress) {
			/* Shutdown currently in progress; report this
			 * synchronously as an error to avoid reporting a
			 * completion event before the corresponding shutdown
			 * event actually completes */
			errno = EBUSY;
			ret = -1;
			goto out;
		}
		/* The connection has already been shut down for what the user
		 * requested, so the operation is a no-op. */
		struct exs_event *evt = EXS_CALLOC(1, sizeof(*evt));
		evt->exs_evt_type = EXS_EVT_SHUTDOWN;
		evt->exs_evt_socket = this_conn->sockno;
		evt->exs_evt_ahandle = ahandle;
		ret = 0;
		_put_full_evt_queue_event(qptr, evt);
		goto out;
	}

	if (this_conn->conn_flags & EXS_OK_TO_ISSUE_IO) {
		block = &this_conn->eof_block_array[0];
	} else {
		block = &this_conn->eof_block_array[1];
	}

	ret = _get_empty_evt_queue_event(qptr, &block->user_event_ptr);
	if (ret < 0) {
		errno = -ret;
		ret = -1;
		goto out;
	}
	assert(block->user_event_ptr != NULL || (flags & EXS_UNSIGNALED));

	block->in_progress = 1;
	block->qhandle = qptr;
	block->recv_byte_len = 0;
	block->eof_buf->flags = htonl(how);
	if (block->user_event_ptr) {
		block->user_event_ptr->exs_evt_type = EXS_EVT_SHUTDOWN;
		block->user_event_ptr->exs_evt_errno = 0;
		block->user_event_ptr->exs_evt_ahandle = ahandle;
		block->user_event_ptr->exs_evt_socket = this_conn->sockno;
	}

	if (this_conn->pending_send_count == 0 || how == SHUT_RD) {
		if (this_conn->conn_flags & EXS_OK_TO_ISSUE_IO) {
			/* We've only shut down half of the connection, so send
			 * the EOF request but keep the connection alive. */
			ret = our_send_eof(this_conn, block, 0, how);
		} else {
			/* We are shutting down the whole connection, so tear it
			 * down (probably sending an EOF as part of this
			 * process) */
			ret = our_shutdown_conn(this_conn, block);
		}
	} else {
		TRACE_DBG("fd %d delaying block %p how %d",
				this_conn->sockno, (void *)block, how);
		ret = 0;
	}

out:
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, event_mutex);
	return ret;
}	/* our_start_shutdown */


/*
 *	exs_shutdown() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if started ok
 *		else -1 with errno set to error value
 *
 * If the EXS_BLOCK bit is set in flags, this function will block until the
 * operation completes (i.e., the operation will be synchronous).  In this case,
 * the qhandle and ahandle arguments are ignored.  No other flags are supported.
 */
EXS_EXPORT int
exs_shutdown(int sockno, int how, int flags,
				exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	int			ret;
	struct evt_queue	*qptr;
	exs_event_t		local_vector;
	struct exs_conn		*this_conn;

	TRACE_DBG("enter socket %d", sockno);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	if (flags & ~(EXS_BLOCK|EXS_UNSIGNALED)) {
		TRACE_ERR("bad flags");
		errno = EINVAL;
		return -1;
	}
	if (!(flags & (EXS_BLOCK|EXS_UNSIGNALED))
				&& qhandle == NULL) {
		TRACE_ERR("qhandle is null");
		errno = EINVAL;
		return -1;
	}

	if (how != SHUT_RD && how != SHUT_WR && how != SHUT_RDWR) {
		TRACE_ERR("bad how");
		errno = EINVAL;
		return -1;
	}

	/* verify that this socket has been created */
	if ((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	if ((this_conn->conn_state != EXS_CONN_FINISHED
			&& this_conn->conn_state < EXS_CONN_RUNNING)
			|| this_conn->conn_state == EXS_CONN_LISTENING) {
		errno = ENOTCONN;
		qptr = NULL;
		ret = -1;
		goto out;
	}

	/* see if user gave us an event queue or wants a blocking send.  After
	 * this, qptr will be set or this function will have returned an error
	 * condition. */
	if (flags & EXS_BLOCK) {
		/* get qhandle from cache, dynamically creating one if needed */
		qptr = _evt_queue_init_internal(1);
		if(qptr == NULL)
		{	/* couldn't create it, errno already set */
			ret = -1;
			goto out;
		}
		atomic_add(1, &qptr->refcount);
		atomic_add(1, &qptr->npending);
	} else if (qhandle == NULL) {
		/* caller gave us no qhandle, so don't use one */
		qptr = NULL;
	} else if ((qptr = _cnv_qnumb_to_ptr(qhandle, 1,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* Invalid argument */
		errno = EINVAL;
		ret = -1;
		goto out;
	}

	ret = our_start_shutdown(this_conn, how, flags, qptr, ahandle);
	this_conn = our_socket_unref(this_conn);

	if (ret < 0 || !(flags & EXS_BLOCK)) {
		goto out1;
	}

	/* caller wants a blocking (synchronous) shutdown */

	TRACE_DBG("blocking shutdown, local_qhandle %p, ahandle %p",
		(void *)qptr, ahandle);

	/* recv was started ok, now wait for it to complete */
	ret = _evt_queue_dequeue_vector(qptr, &local_vector, 1, NULL);

	TRACE_DBG("back from dequeue, qptr %p, ahandle %p, ret %d, exs_evt_type %d, exs_evt_errno %d, exs_evt_ahandle %p",
		(void *)qptr, ahandle, ret,
		local_vector.exs_evt_type,
		local_vector.exs_evt_errno,
		local_vector.exs_evt_ahandle);

	if (ret != -1)
	{	/* no error on dequeue itself check completion */
		if ((errno = local_vector.exs_evt_errno) != 0) {
			ret = -1;
		}
	}

out1:
	if (qptr) {
		if ((flags & EXS_BLOCK)) {
			/* always return the locally created queue to the connection's cache */
			qptr = _evt_queue_unref(qptr, 1, 0);
		} else if (ret) {
			qptr = _evt_queue_unref(qptr, 1, EVT_QUEUE_REF_PENDING);
		}
	}

out:
	/* return < 0 on error, >= 0 if completed ok */
	if (this_conn) {
		this_conn = our_socket_unref(this_conn);
	}
	return ret;
}	/* exs_shutdown */


static int
our_start_close(struct exs_conn *this_conn, int flags, struct evt_queue *qptr,
		void *ahandle)
{
	int ret;

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	TRACE_DBG("socket %d, this_conn %p, pending_recv_count %d, pending_send_count %d, conn_state %s, conn_flags %0x",
		this_conn->sockno, (void *)this_conn,
		this_conn->pending_recv_count,
		this_conn->pending_send_count,
		our_conn_state_str(this_conn->conn_state),
		this_conn->conn_flags);

	/* fill in ALL fields of struct pending_connect for this connection */
	this_conn->close_pending->qhandle = qptr;
	this_conn->close_pending->evt_type = EXS_EVT_CLOSE;
	this_conn->close_pending->err_numb = 0;
	this_conn->close_pending->ahandle = ahandle;
	this_conn->close_pending->sockno = this_conn->sockno;
	this_conn->conn_pending->old_state = EXS_CONN_CLOSED;
	this_conn->conn_pending->err_numb = ECONNABORTED;

	TRACE_DBG("this_conn %p will post event on qptr %p", (void *)this_conn, (void *)qptr);

	fflush(stdout);

	OUR_MEMBER_MUTEX_LOCK(this_conn, event_mutex);
	if ((flags & EXS_DONTLINGER) && our_clear_conn_flags(this_conn,
				EXS_NEED_DISCONNECT) != EXS_NEED_DISCONNECT) {
		/* DONTLINGER flag is set and we are still alive; disconnect */
		our_clear_conn_flags(this_conn, EXS_NEED_LOCAL_CLOSE
							| EXS_OK_TO_ISSUE_IO);
		our_rdma_disconnect(this_conn);
		this_conn->conn_state = EXS_CONN_CLOSING;
	}
	ret = our_shutdown_conn(this_conn, &this_conn->eof_block_array[1]);
	OUR_MEMBER_MUTEX_UNLOCK(this_conn, event_mutex);
	this_conn = our_socket_unref(this_conn);

	return ret;
}	/* our_start_close */


/*
 *	exs_close() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if started ok
 *		else -1 with errno set to error value
 *
 * If the EXS_BLOCK bit is set in flags, this function will block until the
 * operation completes (i.e., the operation will be synchronous).  In this case,
 * the qhandle and ahandle arguments are ignored.  No other flags are supported.
 */
EXS_EXPORT int
exs_close(int sockno, int flags, exs_qhandle_t qhandle, exs_ahandle_t ahandle)
{
	struct exs_conn *this_conn;
	struct evt_queue *qptr;
	int ret;

	TRACE_DBG("enter socket %d", sockno);

	if (flags & ~(EXS_BLOCK|EXS_DONTLINGER)) {
		errno = EINVAL;
		return -1;
	}

	if (flags & EXS_BLOCK) {
		return exs_blocking_close(sockno);
	}

	/* verify that caller is giving us a valid qhandle */
	if((qptr = _cnv_qnumb_to_ptr(qhandle, 1,
					EVT_QUEUE_REF_PENDING)) == NULL) {
		/* Invalid argument */
		return -1;
	}

	/* atomically get socket and remove it from fd table */
	our_socket_table_acquire(1);
	if ((this_conn = our_lookup_socket_(sockno))) {
		/* Make socket inaccessible */
		our_rem_socket_from_list(this_conn);
	}
	our_socket_table_release();
	if (!this_conn) {
		errno = EBADF;
		ret = -1;
		goto out;
	}

	ret = our_start_close(this_conn, flags, qptr, ahandle);

out:
	return ret;
}	/* exs_close */


/*
 *	exs_blocking_close() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if close finished ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_blocking_close(int sockno)
{
	ssize_t			ret;
	struct exs_conn		*this_conn;
	struct evt_queue	*local_qhandle;
	exs_ahandle_t		local_ahandle;
	exs_event_t		local_vector;

	TRACE_DBG("enter socket %d", sockno);

	/* dynamically create struct evt_queue (can't use connection's
	 * queue_cache because that connection is going away!
	 */
	local_qhandle = _evt_queue_init_internal(1);
	if(local_qhandle == NULL)
	{	/* couldn't create it, errno already set */
		local_vector.exs_evt_errno = errno;
		ret = -1;
		goto out0;
	}

	/* atomically get socket and remove it from fd table */
	our_socket_table_acquire(1);
	if ((this_conn = our_lookup_socket_(sockno))) {
		/* Make socket inaccessible */
		our_rem_socket_from_list(this_conn);
	}
	our_socket_table_release();
	if (!this_conn) {
		local_vector.exs_evt_errno = EBADF;
		ret = -1;
		goto out1;
	}

	/* increment queue refcount so close event does not destroy queue */
	atomic_add(1, &local_qhandle->refcount);
	atomic_add(1, &local_qhandle->npending);
	local_ahandle = (void *)local_qhandle;
	ret = our_start_close(this_conn, 0, local_qhandle, local_ahandle);

	TRACE_DBG("back from our_start_close, qhandle %p, ahandle %p, ret %zd",
				(void *)local_qhandle, local_ahandle, ret);

	if (ret) {
		local_vector.exs_evt_errno = errno;
	} else {
		/* caller wants blocking close, wait for completion event */

		TRACE_DBG("wait on qptr %p", (void *)local_qhandle);
		ret = _evt_queue_dequeue_vector(local_qhandle, &local_vector,
								1, NULL);

		TRACE_DBG("back from dequeue, qhandle %p, ahandle %p, ret %zd, exs_evt_type %d, exs_evt_errno %d, exs_evt_ahandle %p",
			(void *)local_qhandle, local_ahandle,
			ret, local_vector.exs_evt_type,
			local_vector.exs_evt_errno,
			local_vector.exs_evt_ahandle);

		if (ret < 0) {
			local_vector.exs_evt_errno = errno;
		} else {
			/* no error on dequeue itself, check the completion */
			if(local_vector.exs_evt_errno != 0) {
				ret = -1;
			}
			else
			{	/* no error on the close */
				ret = 0;
			}
		}
	}

out1:
	/* always delete the locally created queue (can't return it to
	 * connection's cache because connection is gone!  Anyway, we
	 * didn't get it from that cache for exactly that reason.
	 */
	local_qhandle = _evt_queue_unref(local_qhandle, 1, 0);
	if (ret < 0)
		errno = local_vector.exs_evt_errno;

out0:
	return ret;
}	/* exs_blocking_close */


/*
 *	exs_getsockname() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns 0 if name obtained ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_getsockname(int sockno, struct sockaddr *name, socklen_t *namelen)
{
	struct exs_conn		*this_conn;
	int			ret;

	TRACE_DBG("enter socket %d", sockno);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has already been connected */
	if(this_conn->conn_state <= EXS_CONN_UNBOUND)
	{
		errno = ENOTCONN;	/* Transport endpoint is not connected*/
		ret = -1;
		goto out;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	*namelen = our_copy_src_addr(this_conn->cm_id, name, *namelen);
	ret = 0;

out:
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_getsockname */


/*
 *	exs_getpeername() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns 0 if name obtained ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_getpeername(int sockno, struct sockaddr *name, socklen_t *namelen)
{
	struct exs_conn		*this_conn;
	int			ret;

	TRACE_DBG("enter socket %d", sockno);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	/* verify that this socket has already been connected */
	if(this_conn->conn_state != EXS_CONN_RUNNING)
	{
		errno = ENOTCONN;	/* Transport endpoint is not connected*/
		ret = -1;
		goto out;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		sockno, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	*namelen = our_copy_dst_addr(this_conn->cm_id, name, *namelen);
	ret = 0;

out:
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_getpeername */


/* set to non-zero to enable debug printing */
int _our_debug_flag = 0;

/*
 *	exs_set_debug_flag() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns previous value of global variable _our_debug_flag
 */
EXS_EXPORT int
exs_set_debug_flag(int value)
{
	int	ret;

	ret = _our_debug_flag;
	_our_debug_flag = value;
	if (value) {
		_exs_global_flags |= EXS_GLOBAL_TRACE;
	}
	return ret;
}	/* exs_set_debug_flag */


/*
 *	exs_fcntl() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if attribute's value was set ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT long
exs_fcntl(int fd, int command_name, long command_value)
{
	long			ret = 0;
	int			old_value; /*used only in GET/SETCOMPTHREADCPU*/
	struct exs_conn		*this_conn;

	TRACE_DBG("enter socket %d, command_name %d, command_value %0lx",
					fd, command_name, command_value);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(fd)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	TRACE_DBG("socket %d, this_conn %p, state %s",
		fd, (void *)this_conn,
		our_conn_state_str(this_conn->conn_state));

	switch(command_name)
	{
	case EXS_F_GETFD:		/* get file descriptor flags */
		ret = this_conn->fd_flags;
		break;

	case EXS_F_SETFD:		/* set file descriptor flags */
		if((command_value & ~(EXS_FD_ALL_FLAGS_MASK)) != 0)
		{	/* Illegal flag value */
			errno = EINVAL;
			ret = -1;
			goto out;
		}
		/* verify that this socket has not already been connected */
		if(this_conn->conn_state > EXS_CONN_BOUND
				&& this_conn->conn_state != EXS_CONN_LISTENING)
		{	/* Transport endpoint is already connected */
			errno = EISCONN;
			ret = -1;
			goto out;
		}

		/* ok to set these flags, return old flag settings */
		ret = this_conn->fd_flags;

		/* Bypass param checking per-fd is now deprecated. */
		command_value &= ~EXS_FD_BYPASS_PARAM_CHECKING;
		if (this_conn->is_seqpacket) {
			/* SOCK_SEQPACKET does not support Nagle.  Keep this
			 * turned on for API compatibility with older
			 * versions.  SOCK_SEQPACKET also requires advert
			 * messages. */
			command_value |= EXS_FD_NODELAY;
			command_value &= ~EXS_FD_NO_STREAM_ADVERT;
		} else {
			/* SOCK_STREAM does not support small packet copy
			 * unreg */
			command_value &= ~EXS_FD_SP_COPY_UNREG;
			if (command_value & EXS_FD_NODELAY) {
				/* New meaning of this option is simply to
				 * disable the stream receive buffer */
				this_conn->stream_buf_size = 0;
			}
		}

		this_conn->fd_flags = command_value;
		break;

	case EXS_F_GETSPMAXSIZE:	/* get small packet max size */
		ret = this_conn->small_packet_max_size;
		break;

	case EXS_F_SETSPMAXSIZE:	/* set small packet max size */
		/* verify that this socket has not already been connected */
		if(this_conn->conn_state > EXS_CONN_BOUND
				&& this_conn->conn_state != EXS_CONN_LISTENING)
		{	/* Transport endpoint is already connected */
			errno = EISCONN;
			ret = -1;
			goto out;
		}

		if(command_value < 0
				|| command_value > MAX_SMALL_PACKET_MAX_SIZE)
		{	/* small packet max size would become too big */
			errno = EINVAL;
			ret = -1;
			goto out;
		}
		/* new small_packet_max_size is ok, use it, return old value */
		ret = this_conn->small_packet_max_size;
		this_conn->small_packet_max_size = command_value;
		break;

	case EXS_F_GETINLINEMAXSIZE:	/* get inline max size */
		ret = this_conn->inline_limit;
		break;

	case EXS_F_SETINLINEMAXSIZE:	/* set inline max size */
		/* verify that this socket has not already been connected */
		if(this_conn->conn_state > EXS_CONN_BOUND
				&& this_conn->conn_state != EXS_CONN_LISTENING)
		{	/* Transport endpoint is already connected */
			errno = EISCONN;
			ret = -1;
			goto out;
		}

		/* accept large values since they will be negotiated down
		 * if the card doesn't support it */
		if (command_value < 0) {
			errno = EINVAL;
			ret = -1;
			goto out;
		}
		/* new small_packet_max_size is ok, use it, return old value */
		ret = this_conn->inline_limit;
		this_conn->inline_limit = command_value;
		break;

	case EXS_F_GETFLOWCONTROLCREDITS:	/* get send credits */
		ret = this_conn->posted_credits;
		break;

	case EXS_F_SETFLOWCONTROLCREDITS:	/* set send credits */
		/* verify that this socket has not already been connected */
		if(this_conn->conn_state > EXS_CONN_BOUND
				&& this_conn->conn_state != EXS_CONN_LISTENING)
		{	/* Transport endpoint is already connected */
			errno = EISCONN;
			ret = -1;
			goto out;
		}

		if(command_value <= 0)
		{	/* send credit max size would become too big */
			errno = EINVAL;
			ret = -1;
			goto out;
		}
		/* new send credit max size is ok, use it, return old value */
		ret = this_conn->posted_credits;
		our_set_send_credit_limits(this_conn, command_value);
		break;

	case EXS_F_GETSTREAMBUFSIZE:
		if (this_conn->is_seqpacket) {
			errno = EINVAL;
			ret = -1;
			goto out;
		}

		ret = this_conn->stream_buf_size;
		break;

	case EXS_F_SETSTREAMBUFSIZE:
		/* verify that this socket has not already been connected */
		if(this_conn->conn_state > EXS_CONN_BOUND
				&& this_conn->conn_state != EXS_CONN_LISTENING) {
			/* Transport endpoint is already connected */
			errno = EISCONN;
			ret = -1;
			goto out;
		}

		if (this_conn->is_seqpacket) {
			errno = EINVAL;
			ret = -1;
			goto out;
		}

		if(command_value < 0
				|| command_value > EXS_MAX_STREAM_BUF_SIZE) {
			errno = EINVAL;
			ret = -1;
			goto out;
		}

		/* new send credit max size is ok, use it, return old value */
		ret = this_conn->stream_buf_size;
		this_conn->stream_buf_size = command_value;

		/* Adjust EXS_FD_NODELAY value as necessary, for compatibility
		 * purposes */
		if (command_value == 0) {
			this_conn->fd_flags |= EXS_FD_NODELAY;
		} else {
			this_conn->fd_flags &= ~EXS_FD_NODELAY;
		}
		break;

	case EXS_F_GETCOMPTHREADCPU:
		if(this_conn->conn_state >= EXS_CONN_CONNECTING
				&& this_conn->conn_state != EXS_CONN_LISTENING){
			/* Completion thread already started; get/set its
			 * affinity normally */
			if ((ret = _exs_set_thread_affinity(
					this_conn->comp_thread,
					INT_MAX, &old_value)) >= 0) {
				ret = old_value;
			}
		} else {
			ret = this_conn->comp_thread_cpu;
		}
		break;

	case EXS_F_SETCOMPTHREADCPU: /* set cpu for completion thread */
		if(this_conn->conn_state >= EXS_CONN_CONNECTING
				&& this_conn->conn_state != EXS_CONN_LISTENING){
			/* Completion thread already started; get/set its
			 * affinity normally */
			if ((ret = _exs_set_thread_affinity(
					this_conn->comp_thread,
					command_value, &old_value)) >= 0) {
				ret = old_value;
			}
		} else {
			ret = this_conn->comp_thread_cpu;
			if (command_value != INT_MAX) {
				this_conn->comp_thread_cpu = command_value;
			}
		}

		break;

	default:
		/* caller gave us an invalid command name */
		errno = EINVAL;		/* Invalid argument */
		ret = -1;
		break;
	}	/* switch */

out:
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_fcntl */


/*
 *	exs_getsockopt() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if attribute's value was retrieved ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_getsockopt(int sockfd, int level, int optname, void *optval, socklen_t *optlen)
{
	int ret;
	struct exs_conn		*this_conn;

	TRACE_DBG("start, sockfd %d", sockfd);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if ((this_conn = our_lookup_socket(sockfd)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	TRACE_DBG("start socket %d, this_conn %p", sockfd, (void *)this_conn);

	/* assume the worst */
	ret = -1;

	/* we don't support any TCP/IP options for now */
	if (level != SOL_RDMA) {
		errno = EOPNOTSUPP;
		goto out0;
	}

	/* cast so gcc checks that we handle all enum values */
	switch ((enum sockopt_rdma)optname) {
	case RDMA_INLINE:
		if (*optlen < sizeof(this_conn->inline_limit)) {
			errno = EINVAL;
			goto out0;
		}
		memcpy(optval, &this_conn->inline_limit,
				sizeof(this_conn->inline_limit));
		*optlen = sizeof(this_conn->inline_limit);
		ret = 0;
		break;
	case RDMA_SQSIZE:
	case RDMA_RQSIZE:
	case RDMA_IOMAPSIZE:
		errno = EOPNOTSUPP;
		goto out0;
	}

out0:
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_getsockopt */


/*
 *	exs_setsockopt() - not in Interconnect Software Consortium Issue 1.0
 *			      was added by us
 *
 *	returns 0 if attribute's value was set ok
 *		else -1 with errno set to error value
 */
EXS_EXPORT int
exs_setsockopt(int sockfd, int level, int optname,
		const void *optval, socklen_t optlen)
{
	int ret;
	struct exs_conn		*this_conn;

	TRACE_DBG("start, sockfd %d", sockfd);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if ((this_conn = our_lookup_socket(sockfd)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	TRACE_DBG("start socket %d, this_conn %p", sockfd, (void *)this_conn);

	/* assume the worst */
	ret = -1;

	/* we don't support any TCP/IP options for now */
	if (level != SOL_RDMA) {
		errno = EOPNOTSUPP;
		goto out0;
	}

	/* cast so gcc checks that we handle all enum values */
	switch ((enum sockopt_rdma)optname) {
	case RDMA_INLINE:
		if (optlen > sizeof(this_conn->inline_limit)) {
			errno = EINVAL;
			goto out0;
		}
		memcpy(&this_conn->inline_limit, optval,
				sizeof(this_conn->inline_limit));
		ret = 0;
		break;
	case RDMA_SQSIZE:
	case RDMA_RQSIZE:
	case RDMA_IOMAPSIZE:
		errno = EOPNOTSUPP;
		goto out0;
	}

out0:
	this_conn = our_socket_unref(this_conn);
	return ret;
}	/* exs_setsockopt */


EXS_EXPORT int
exs_get_version(char *buf, size_t size)
{
	size_t len = strlen(our_version);
	if (len >= size) {
		errno = ENOSPC;
		return -1;
	}
	strncpy(buf, our_version, size);
	return 0;
}	/* exs_get_version */


/*
 *	exs_print_connection_attributes() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	prints out attributes of a connection
 */
EXS_EXPORT int
exs_print_connection_attributes(FILE *fptr, int sockno, char *message)
{
	struct exs_conn		*this_conn;

	TRACE_DBG("enter socket %d", sockno);

	if (!our_check_init()) {
		errno = EPERM;		/* Operation not permitted */
		return -1;
	}

	/* verify that this socket has been created */
	if((this_conn = our_lookup_socket(sockno)) == NULL)
	{
		errno = EBADF;		/* Bad file number */
		return -1;
	}

	fprintf(fptr, "%s fd %d: %36s %u.%u.%u\n", message,
		sockno, "exs version", EXS_VERSION, EXS_SUBVERSION,
		EXS_SUBSUBVERSION);

	fprintf(fptr, "%s fd %d: %36s %u\n", message,
			sockno, "posted credits",
			this_conn->posted_credits);

	fprintf(fptr, "%s fd %d: %36s %u\n", message,
			sockno, "small packet max size",
			this_conn->small_packet_max_size);

	fprintf(fptr, "%s fd %d: %36s %u\n", message,
			sockno, "small packet max size limit",
			MAX_SMALL_PACKET_MAX_SIZE);

	fprintf(fptr, "%s fd %d: %36s %3u (requested %3u)\n", message,
			sockno, "qp max_inline_data",
			this_conn->obtained_cap.max_inline_data,
			this_conn->requested_cap.max_inline_data);

	fprintf(fptr, "%s fd %d: %36s %3u (requested %3u)\n", message,
			sockno, "qp max_send_wr",
			this_conn->obtained_cap.max_send_wr,
			this_conn->requested_cap.max_send_wr);

	fprintf(fptr, "%s fd %d: %36s %3u (requested %3u)\n", message,
			sockno, "qp max_recv_wr",
			this_conn->obtained_cap.max_recv_wr,
			this_conn->requested_cap.max_recv_wr);

	fprintf(fptr, "%s fd %d: %36s %3u (requested %3u)\n", message,
			sockno, "number of completion queue elements",
			this_conn->obtained_cq_nelements,
			this_conn->requested_cq_nelements);

	fprintf(fptr, "%s fd %d: %36s %d\n", message,
			sockno, "path mtu size",
			this_conn->obtained_path_mtu_size);

	fflush(fptr);
	this_conn = our_socket_unref(this_conn);
	return 0;
}	/* exs_print_connection_attributes */

static void
print_send_wr(FILE *fptr, struct ibv_send_wr *wr)
{
	char opcode[40];
	int i;
	int has_imm_data;
	int has_rdma;

	if (! TRACING_IS_ON) {
		return;
	}

	while (wr != NULL) {
		switch (wr->opcode) {
		case IBV_WR_RDMA_WRITE:
			strcpy(opcode, "IBV_WR_RDMA_WRITE");
			has_rdma = 1;
			has_imm_data = 0;
			break;
		case IBV_WR_RDMA_WRITE_WITH_IMM:
			strcpy(opcode, "IBV_WR_RDMA_WRITE_WITH_IMM");
			has_rdma = has_imm_data = 1;
			break;
		case IBV_WR_SEND:
			strcpy(opcode, "IBV_WR_SEND");
			has_rdma = has_imm_data = 0;
			break;
		case IBV_WR_SEND_WITH_IMM:
			strcpy(opcode, "IBV_WR_SEND_WITH_IMM");
			has_rdma = 0;
			has_imm_data = 1;
			break;
		case IBV_WR_RDMA_READ:
			strcpy(opcode, "IBV_WR_RDMA_READ");
			has_rdma = 1;
			has_imm_data = 0;
			break;
		default:
			has_rdma = 0;
			has_imm_data = 0;
			snprintf(opcode, sizeof(opcode) - 1,
						"(unknown opcode %d)", wr->opcode);
		}

		fprintf(fptr, "ibv_send_wr wr_id=%016" PRIx64 " opcode=%s\nFlags: ",
				wr->wr_id, opcode);

		if (wr->send_flags & IBV_SEND_SIGNALED) {
			fprintf(fptr, "IBV_SEND_SIGNALED ");
		}
		if (wr->send_flags & IBV_SEND_INLINE) {
			fprintf(fptr, "IBV_SEND_INLINE ");
		}

		if (has_imm_data) {
			fprintf(fptr, "\nimm_data=%" PRIu32, ntohl(wr->imm_data));
		}

		if (has_rdma) {
			fprintf(fptr, "\nrdma: rkey=%08" PRIx32 " remote_addr=%016" PRIx64,
				wr->wr.rdma.rkey, wr->wr.rdma.remote_addr);
		}

		fprintf(fptr, "\nnum_sge=%d\n", wr->num_sge);

		for (i = 0; i < wr->num_sge; ++i) {
			fprintf(fptr, "sg_list[%d]: addr=%016" PRIx64 " length=%" PRIu32 " lkey=%08" PRIx32 "\n",
					i, wr->sg_list[i].addr, wr->sg_list[i].length,
					wr->sg_list[i].lkey);
		}

		fprintf(fptr, "\n");
		wr = wr->next;
	}
}	/* print_send_wr */


/*
 *	exs_get_global_flags() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns current connection flags
 */
EXS_EXPORT int
exs_get_global_flags(void)
{
	return _exs_global_flags;
}	/* exs_get_global_flags */


/*
 *	exs_set_global_flags() - not in Interconnect Software Consortium Issue 1.0
 *		      was added by us
 *
 *	returns current connection flags
 */
EXS_EXPORT void
exs_set_global_flags(int value)
{
	_exs_global_flags = value;
	if (value & EXS_GLOBAL_TRACE) {
		_our_debug_flag = 1;
	}
}	/* exs_set_global_flags */


/* Operations structure definition for SOCK_SEQPACKET */
static const struct exs_conn_ops ops_seqpacket = {
	.xmit_if_match = &our_xmit_if_match_seqpacket,
	.enqueue_send = &our_enqueue_send_seqpacket,
	.handle_recv_advert = &our_handle_recv_advert_seqpacket,
	.handle_send_advert = &our_handle_send_advert_seqpacket,
	.handle_send_write = &our_handle_send_write_seqpacket,
	.handle_recv_write_notify = &our_handle_recv_write_notify_seqpacket,
};

/* Operations structure definition for SOCK_STREAM */
static const struct exs_conn_ops ops_stream = {
	.xmit_if_match = &our_xmit_if_match_stream,
	.enqueue_send = &our_enqueue_send_stream,
	.handle_recv_advert = &our_handle_recv_advert_stream,
	.handle_send_advert = &our_handle_send_advert_stream,
	.handle_send_write = &our_handle_send_write_stream,
	.handle_recv_write_notify = &our_handle_recv_write_notify_stream,
};

/* vi: set autoindent tabstop=8 shiftwidth=8 : */
